
# CMake module to search for Proj library
#
# If it's found it sets PROJ_FOUND to TRUE
# and following variables are set:
#    PROJ_INCLUDE_DIR
#    PROJ_LIBRARY


macro(add_library_for_config dest prefix input fallback)
    if(${input})
        set(${dest} ${${dest}} ${prefix} ${${input}})
    elseif(${fallback})
        set(${dest} ${${dest}} ${prefix} ${${fallback}})
    endif()
endmacro()


# FIND_PATH and FIND_LIBRARY normally search standard locations
# before the specified paths. To search non-standard paths first,
# FIND_* is invoked first with specified paths and NO_DEFAULT_PATH
# and then again with no specified paths to search the default
# locations. When an earlier FIND_* succeeds, subsequent FIND_*s
# searching for the same item do nothing.
FIND_PATH(PROJ_INCLUDE_DIR proj_api.h
  "$ENV{LIB_DIR}/include/proj"
  "$ENV{LIB_DIR}/include"
  #mingw
  c:/msys/local/include
  ${CMAKE_SOURCE_DIR}/../proj/src
  ${CMAKE_SOURCE_DIR}/../3rdparty/proj/src
  ${CMAKE_SOURCE_DIR}/externals/proj/src
  NO_DEFAULT_PATH
  )
FIND_PATH(PROJ_INCLUDE_DIR proj_api.h)


# Find release (optimized) lib
FIND_LIBRARY(PROJ_LIBRARY_RELEASE NAMES proj PATHS
  "$ENV{LIB_DIR}/lib"
  #mingw
  c:/msys/local/lib
  ${CMAKE_SOURCE_DIR}/../proj/lib
  ${CMAKE_SOURCE_DIR}/../3rdparty/proj/lib
  ${CMAKE_SOURCE_DIR}/externals/proj/lib
  NO_DEFAULT_PATH
  )
FIND_LIBRARY(PROJ_LIBRARY_RELEASE NAMES proj)

# Find debug (optimized) lib
FIND_LIBRARY(PROJ_LIBRARY_DEBUG NAMES projd PATHS
  "$ENV{LIB_DIR}/lib"
  #mingw
  c:/msys/local/lib
  ${CMAKE_SOURCE_DIR}/../proj/lib
  ${CMAKE_SOURCE_DIR}/../3rdparty/proj/lib
  ${CMAKE_SOURCE_DIR}/externals/proj/lib
  NO_DEFAULT_PATH
  )
FIND_LIBRARY(PROJ_LIBRARY_DEBUG NAMES projd)

# Compose lists
SET(PROJ_LIBRARY "")
add_library_for_config(PROJ_LIBRARY optimized PROJ_LIBRARY_RELEASE PROJ_LIBRARY_DEBUG)
add_library_for_config(PROJ_LIBRARY debug     PROJ_LIBRARY_DEBUG   PROJ_LIBRARY_RELEASE)

IF (PROJ_INCLUDE_DIR AND PROJ_LIBRARY)
   SET(PROJ_FOUND TRUE)
ENDIF (PROJ_INCLUDE_DIR AND PROJ_LIBRARY)


IF (PROJ_FOUND)

   IF (NOT PROJ_FIND_QUIETLY)
      MESSAGE(STATUS "Found Proj: ${PROJ_LIBRARY}")
   ENDIF (NOT PROJ_FIND_QUIETLY)

ELSE (PROJ_FOUND)

   IF (PROJ_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "Could not find Proj")
   ENDIF (PROJ_FIND_REQUIRED)

ENDIF (PROJ_FOUND)
