
import QtQuick 2.0
import QtQuick.Window 2.2

import QmlToolbox.Base 1.0
import QmlToolbox.Controls 1.0

import gloperate.rendering 1.0


ApplicationWindow
{
    id: window

    visible:    true
    width:      1024
    height:     768
    visibility: Window.Windowed // Window.FullScreen

    // Stage
    property string stage: 'DemoPipeline'

    // Pipeline interface
    GlOperatePipeline
    {
        id: gloperatePipeline

        onCanvasChanged:
        {
            if (canvas != null)
            {
                canvas.onStageInputChanged(function(slot, status)
                {
                    gloperatePipeline.slotChanged('root', slot, status);
                });
            }
        }

        onSlotChanged: // (string path, string slot, var status)
        {
            if (path == 'root' && slot == 'updateTrigger')
            {
                timer.start();
            }
        }

        Timer
        {
            id: timer

            interval: 10
            repeat:   false

            onTriggered:
            {
                timeline.update();
                timeline.emitUpdate();
            }
        }
    }

    // Renderer
    RenderItem
    {
        id: render

        anchors.fill: parent

        stage: window.stage
        focus: true

        onCanvasInitialized:
        {
            gloperatePipeline.canvas = gloperate.canvas0;
        }

        TimeLine
        {
            id: timeline

            anchors.left:    parent.left
            anchors.right:   parent.right
            anchors.bottom:  parent.bottom
            anchors.margins: 16
            height:          32

            properties: gloperatePipeline
            timeSpans: 1
        }
    }

    // Keyboard shortcuts
    Shortcut
    {
        sequence: "F10"

        onActivated:
        {
            window.visibility = (window.visibility == Window.Windowed) ?  Window.FullScreen : Window.Windowed;
        }
    }
}
