
import QtQuick 2.0

import QmlToolbox.Base 1.0
import QmlToolbox.Controls 1.0


/**
*  TimeLine
*
*  Time line item for temporal filtering
*/
Item
{
    id: item

    property var  properties: null
    property real minTime:    0.0
    property real maxTime:    0.0
    property alias minTime1:  timespan1.beginTime
    property alias maxTime1:  timespan1.endTime
    property alias minTime2:  timespan2.beginTime
    property alias maxTime2:  timespan2.endTime
    property int timeSpans:   1

    clip: true

    Rectangle
    {
        id: background

        anchors.fill:    parent
        anchors.margins: 2

        color:  '#66888888'
        radius: 2
    }

    TimeSpan
    {
        id: timespan1

        visible: item.timeSpans >= 1
        color:  '#660000ff'
        minTime: item.minTime
        maxTime: item.maxTime

        onTimeSpanChanged:
        {
            item.emitUpdate();
        }
    }

    TimeSpan
    {
        id: timespan2

        visible: item.timeSpans >= 2
        color:  '#66ff0000'
        minTime: item.minTime
        maxTime: item.maxTime

        onTimeSpanChanged:
        {
            item.emitUpdate();
        }
    }

    function update()
    {
        // Get minimum and maximum time values
        item.minTime = properties.getSlot('root.TemporalFilter', 'minTime').value;
        item.maxTime = properties.getSlot('root.TemporalFilter', 'maxTime').value;

        // Reset time spans
        if (item.timeSpans == 2) {
            var mid = item.minTime + (item.maxTime - item.minTime) * 0.5;
            item.minTime1 = item.minTime;
            item.maxTime1 = mid;
            item.minTime2 = mid;
            item.maxTime2 = item.maxTime;
        } else {
            item.minTime1 = item.minTime;
            item.maxTime1 = item.maxTime;
            item.minTime2 = item.minTime;
            item.maxTime2 = item.maxTime;
        }
    }

    function emitUpdate()
    {
        item.properties.setValue('root.TemporalFilter', 'minTime1', timespan1.beginTime);
        item.properties.setValue('root.TemporalFilter', 'maxTime1', timespan1.endTime);
        item.properties.setValue('root.TemporalFilter', 'minTime2', timespan2.beginTime);
        item.properties.setValue('root.TemporalFilter', 'maxTime2', timespan2.endTime);
    }
}
