
import QtQuick 2.0

import QmlToolbox.Base 1.0
import QmlToolbox.Controls 1.0


/**
*  TimeSpan
*
*  Dragable time span in a time line
*/
Rectangle
{
    id: handle

    signal timeSpanChanged(real beginTime, real endTime)

    property real minTime:   0.0 ///< Start time of the data set
    property real maxTime:   0.0 ///< End time of the data set
    property real beginTime: 0.0 ///< Start time of the time span
    property real endTime:   0.0 ///< End time of the time span

    anchors.top:    parent.top
    anchors.bottom: parent.bottom

    x:     parent.width * (beginTime - handle.minTime) / (handle.maxTime - handle.minTime)
    width: parent.width * (endTime   - handle.minTime) / (handle.maxTime - handle.minTime) - x

    color:  '#66ff0000'
    radius: 2

    property real minTimeDiff: 20.0 / parent.width * (parent.maxTime - parent.minTime)

    MouseArea
    {
        id: maLeft

        anchors.left:   parent.left
        anchors.top:    parent.top
        anchors.bottom: parent.bottom
        width:          20

        cursorShape: Qt.SizeHorCursor

        onPositionChanged:
        {
            var x = mapToItem(handle.parent, mouse.x, 0).x;
            var t = handle.minTime + (handle.maxTime - handle.minTime) * (x / handle.parent.width);

            handle.beginTime = Math.max(Math.min(t, handle.endTime - handle.minTimeDiff), handle.minTime);

            handle.timeSpanChanged(handle.beginTime, handle.endTime);
        }
    }

    MouseArea
    {
        id: maMiddle

        anchors.left:   maLeft.right
        anchors.right:  maRight.left
        anchors.top:    parent.top
        anchors.bottom: parent.bottom

        cursorShape: Qt.SizeAllCursor

        onPressed:
        {
            var x = mapToItem(handle.parent, mouse.x, 0).x;
            var t = handle.minTime + (handle.maxTime - handle.minTime) * (x / handle.parent.width);
            lastTime = t;
        }

        onPositionChanged:
        {
            var x = mapToItem(handle.parent, mouse.x, 0).x;
            var t = handle.minTime + (handle.maxTime - handle.minTime) * (x / handle.parent.width);

            var deltaT = t - lastTime;
            handle.beginTime += deltaT;
            handle.endTime   += deltaT;

            var correction = 0.0;
            if (handle.beginTime < handle.minTime) correction = handle.minTime - handle.beginTime;
            if (handle.endTime   > handle.maxTime) correction = handle.maxTime - handle.endTime;
            handle.beginTime += correction;
            handle.endTime   += correction;

            lastTime = t;

            handle.timeSpanChanged(handle.beginTime, handle.endTime);
        }

        property real lastTime: 0.0
    }

    MouseArea
    {
        id: maRight

        anchors.right:  parent.right
        anchors.top:    parent.top
        anchors.bottom: parent.bottom
        width:          20

        cursorShape: Qt.SizeHorCursor

        onPositionChanged:
        {
            var x = mapToItem(handle.parent, mouse.x, 0).x;
            var t = handle.minTime + (handle.maxTime - handle.minTime) * (x / handle.parent.width);

            handle.endTime = Math.min(Math.max(t, handle.beginTime + handle.minTimeDiff), handle.maxTime);

            handle.timeSpanChanged(handle.beginTime, handle.endTime);
        }
    }
}
