
var animationActive = false;
var animationCount  = 0.0;

var pulsationActive = false;
var pulsationCount  = 0.0;

function setConfiguration()
{
    // Set camera
    gloperate.canvas0.setValue('root.Trackball', 'pitch', -51.00544738769531);
    gloperate.canvas0.setValue('root.Trackball', 'yaw', 97.82893371582031);
    gloperate.canvas0.setValue('root.Trackball', 'zoom', 0.04626892879605293);
    gloperate.canvas0.setValue('root.Trackball', 'rotateX', -123.80043029785156);
    gloperate.canvas0.setValue('root.Trackball', 'rotateY', -61.09987258911133);

    // Set primary attribute
    gloperate.canvas0.setValue('root', 'primaryAttribute', 'Zero');

    // Set configuration
    var stage = 'root.Trajectories.config1';
    gloperate.canvas0.setValue(stage, 'geometryType', 'Tubes');
    gloperate.canvas0.setValue(stage, 'tessellation', 16.0);
    gloperate.canvas0.setValue(stage, 'texture', 'tube_texture_grid.png');
    gloperate.canvas0.setValue(stage, 'colorMap', 'color_gradient_rev');
    gloperate.canvas0.setValue(stage, 'mapHeight', 'Node - height');
    gloperate.canvas0.setValue(stage, 'height', 0.0);
    gloperate.canvas0.setValue(stage, 'mapRadius', 'None');
    gloperate.canvas0.setValue(stage, 'radius', 0.5);
    gloperate.canvas0.setValue(stage, 'mapColor', 'Node - velocity');
    gloperate.canvas0.setValue(stage, 'color', 0.5);
    gloperate.canvas0.setValue(stage, 'mapAlpha', 'None');
    gloperate.canvas0.setValue(stage, 'alpha', 1.0);
    gloperate.canvas0.setValue(stage, 'mapTextureX', 'Node - length');
    gloperate.canvas0.setValue(stage, 'textureX', 0.0);
    gloperate.canvas0.setValue(stage, 'mapTextureY', 'None');
    gloperate.canvas0.setValue(stage, 'textureY', 0.0);
    gloperate.canvas0.setValue(stage, 'minRadius', 0.000015);
    gloperate.canvas0.setValue(stage, 'maxRadius', 0.00008);
    gloperate.canvas0.setValue(stage, 'sizeOfSingleStretch', 0.2);
    gloperate.canvas0.setValue(stage, 'sizeOfSingleTorsion', 1.0);
    gloperate.canvas0.setValue(stage, 'animationSpeed', 1.0);
}

function setTimeSpan(from, to)
{
    var minTime = gloperate.canvas0.getValue('root.TemporalFilter', 'minTime');
    var maxTime = gloperate.canvas0.getValue('root.TemporalFilter', 'maxTime');

    var begin = minTime + (maxTime - minTime) * from;
    var end   = minTime + (maxTime - minTime) * to;

    gloperate.canvas0.setValue('root.TemporalFilter', 'minTime1', begin);
    gloperate.canvas0.setValue('root.TemporalFilter', 'maxTime1', end);
    gloperate.canvas0.setValue('root.TemporalFilter', 'minTime2', begin);
    gloperate.canvas0.setValue('root.TemporalFilter', 'maxTime2', end);
}

gloperate.input.onInput(function(device, type, key, modifier, button, x, y, wx, wy)
{
    if (device === 'keyboard' && type === 'ButtonRelease') {
        if (key == 32) { // SPACE
            setConfiguration();
            animationCount = 0.0;

            if (!animationActive) {
                animationActive = true;
                timer = gloperate.timer.start(10, function() {
                    animationCount += 0.0004;
                    if (animationCount > 1.0) animationCount -= 1.0;

                    setTimeSpan(0, animationCount);
                });
            } else {
                animationActive = false;
                gloperate.timer.stopAll();
            }
        }

        else if (key == 80) { // 'P'
            pulsationCount = 0.0;

            if (!pulsationActive) {
                pulsationActive = true;
                gloperate.timer.start(10, function() {
                    console.log('pulsate ' + pulsationCount);
                    pulsationCount += 0.01;
                    if (pulsationCount > 2.0) pulsationCount -= 2.0;

                    if (pulsationCount < 1.0) {
                        gloperate.canvas0.setValue('root.Trajectories.config1', 'radius', pulsationCount);
                    } else {
                        gloperate.canvas0.setValue('root.Trajectories.config1', 'radius', 2.0 - pulsationCount);
                    }
                });
            } else {
                pulsationActive = false;
                gloperate.timer.stopAll();
            }
        }

        else if (key == 49) { // '1'
            gloperate.canvas0.setValue('root.Trajectories.config1', 'geometryType', 'Tubes');
            gloperate.canvas0.setValue('root.Trajectories.config1', 'texture', 'tube_texture_grid.png');
            gloperate.canvas0.setValue('root.Trajectories.config1', 'minRadius', 0.000015);
            gloperate.canvas0.setValue('root.Trajectories.config1', 'maxRadius', 0.00008);
        }

        else if (key == 50) { // '2'
            gloperate.canvas0.setValue('root.Trajectories.config1', 'geometryType', 'Tubes');
            gloperate.canvas0.setValue('root.Trajectories.config1', 'texture', 'tube_texture_none.png');
            gloperate.canvas0.setValue('root.Trajectories.config1', 'minRadius', 0.000008);
            gloperate.canvas0.setValue('root.Trajectories.config1', 'maxRadius', 0.000002);
        }

        else if (key == 51) { // '3'
            gloperate.canvas0.setValue('root.Trajectories.config1', 'geometryType', 'Spheres');
            gloperate.canvas0.setValue('root.Trajectories.config1', 'texture', 'tube_texture_grid.png');
            gloperate.canvas0.setValue('root.Trajectories.config1', 'minRadius', 0.000015);
            gloperate.canvas0.setValue('root.Trajectories.config1', 'maxRadius', 0.00008);
        }
    }
});

gloperate.timer.once(1000, function() {
    setConfiguration();
});

console.log('');
console.log('Tube rendering demonstration.');
console.log('');
console.log('SPACE: Start animation.');
console.log('P:     Start pulsation.');
console.log('1:     Switch to tube rendering.');
console.log('2:     Switch to line rendering.');
console.log('3:     Switch to sphere rendering.');
