
// Wait for GTX to initialize
gloperate.timer.once(1500, function() {

    // Load dataset
    console.log('Loading dataset');
    gloperate.canvas0.setValue('root', 'filename', 'datasets/hyfly/hyfly99c.graphml');

    // When loaded, set visualization options
    gloperate.canvas0.onRendered(function() {
        console.log('Setting options');

        // Set camera
        gloperate.canvas0.setValue('root.Trackball', 'pitch',  -29.0);
        gloperate.canvas0.setValue('root.Trackball', 'yaw',     44.5);
        gloperate.canvas0.setValue('root.Trackball', 'zoom',    2.5);
        gloperate.canvas0.setValue('root.Trackball', 'rotateX', 0);
        gloperate.canvas0.setValue('root.Trackball', 'rotateY', 0);

        // Map polulation to node color and size
        gloperate.canvas0.setValue('root', 'nodeColor', 'Node - population');
        gloperate.canvas0.setValue('root', 'nodeSize',  'Node - population');

        // Map edge weight to color
        gloperate.canvas0.setValue('root', 'edgeColor', 'Edge - weight');

        // Display only edges above weight threshold
        gloperate.canvas0.setValue('root.Filter', 'edge_weight', [1.0, 1.6]);

        // Get time span of dataset
        var minTime = gloperate.canvas0.getValue('root.TemporalFilter', 'minTime');
        var maxTime = gloperate.canvas0.getValue('root.TemporalFilter', 'maxTime');
        var day     = 0;

        // Select the first day
        // 1 day = 24 * 60 * 60 seconds
        gloperate.canvas0.setValue('root.TemporalFilter', 'minTime1', minTime + day * (24 * 60 * 60));
        gloperate.canvas0.setValue('root.TemporalFilter', 'maxTime1', minTime + (day + 1) * (24 * 60 * 60));

        // Define keyboard shortcuts
        gloperate.input.onInput(function(device, type, key, modifier, button, x, y, wx, wy)
        {
            // Key press event
            if (device === 'keyboard' && type === 'ButtonRelease') {
                // Key ','
                if (key == 44) {
                    // Decrease day
                    day = Math.max(day - 1, 0);

                    // Update time filter
                    console.log('Day ' + day);
                    gloperate.canvas0.setValue('root.TemporalFilter', 'minTime1', minTime + day * (24 * 60 * 60));
                    gloperate.canvas0.setValue('root.TemporalFilter', 'maxTime1', minTime + (day + 1) * (24 * 60 * 60));
                }

                // Key '.'
                if (key == 46) {
                    // Increase day
                    day = Math.min(day + 1, 364);

                    // Update time filter
                    console.log('Day ' + day);
                    gloperate.canvas0.setValue('root.TemporalFilter', 'minTime1', minTime + day * (24 * 60 * 60));
                    gloperate.canvas0.setValue('root.TemporalFilter', 'maxTime1', minTime + (day + 1) * (24 * 60 * 60));
                }
            }
        });

    });

});
