
#version 330


////////////////////////////////////////////////////////////////////////////////////////////////////
// Code below kindly provided by FNC: Library for Focus + Context Visualization
////////////////////////////////////////////////////////////////////////////////////////////////////

//
// Supported
//
#define BLEND_ZERO                         0
#define BLEND_ONE                          1
#define BLEND_DST_COLOR                    2
#define BLEND_SRC_COLOR                    3
#define BLEND_ONE_MINUS_DST_COLOR          4
#define BLEND_ONE_MINUS_SRC_COLOR          5
#define BLEND_SRC_ALPHA                    6
#define BLEND_ONE_MINUS_SRC_ALPHA          7
#define BLEND_DST_ALPHA                    8
#define BLEND_ONE_MINUS_DST_ALPHA          9

//
// Blending function
//
vec4 performBlending(in vec4 CS, in vec4 CD, in vec4 S, in vec4 D)
{
    return vec4(CS.r*S.r+CD.r*D.r, CS.g*S.g+CD.g*D.g, CS.b*S.b+CD.b*D.b, CS.a*S.a+CD.a*D.a);
}

//
// Map blend factors ... no switch statement...too bad :|
//
vec4 mapBlendFactor(in int modeSource, in vec4 source, in vec4 destination)
{
    switch (modeSource)
    {
        case BLEND_ZERO:                  return vec4(0.0);
        case BLEND_ONE:                   return vec4(1.0);
        case BLEND_DST_COLOR:             return destination;
        case BLEND_SRC_COLOR:             return source;
        case BLEND_ONE_MINUS_DST_COLOR:   return vec4(1.0) - destination;
        case BLEND_ONE_MINUS_SRC_COLOR:   return vec4(1.0) - source;
        case BLEND_SRC_ALPHA:             return vec4(source.a);
        case BLEND_ONE_MINUS_SRC_ALPHA:   return vec4(1.0) - vec4(source.a);
        case BLEND_DST_ALPHA:             return vec4(destination.a);
        case BLEND_ONE_MINUS_DST_ALPHA:   return vec4(1.0) - vec4(destination.a);
        default:                          return vec4(1.0);
    }
}

//
// Put it together
//
vec4 blendFunction(int modeSource, int modeDestination, vec4 source, vec4 destination)
{
    return performBlending(
        mapBlendFactor(modeSource, source, destination),
        mapBlendFactor(modeDestination, source, destination),
        source,
        destination);
}
