
#version 330


////////////////////////////////////////////////////////////////////////////////////////////////////
// Lighting and shading
////////////////////////////////////////////////////////////////////////////////////////////////////

vec4 lightingAniso(vec3 position, vec3 tangent, vec3 normal, vec4 color)
{
    // Light source definition
    vec3 lightPos  = vec3(0.0, 0.0, 0.0);
    vec4 lightDiff = vec4(1.0, 1.0, 1.0, 1.0);
    vec4 lightSpec = vec4(1.0, 1.0, 1.0, 1.0);

    // Material definition
    vec4  diffuse   = vec4(1.0, 1.0, 1.0, 1.0);
    vec4  specular  = vec4(0.3, 0.3, 0.3, 1.0);
    float shininess = 4.0;

    // Normalize input
    vec3 T = normalize(tangent);
    vec3 N = normalize(normal);
    vec3 V = position;

    // Compute vectors
    vec3 L = normalize(lightPos - V);
    vec3 H = normalize(L - normalize(V.xyz));
    vec3 R = reflect(-L, N);
    vec3 E = vec3(0.0, 0.0, 1.0);

    // Diffuse
    float LdotT = dot(L, T);
    float VdotT = dot(normalize(V), T);
    float Idiff = sqrt(1.0-LdotT*LdotT);

    // Specular
    float Ispec = pow(max(0.0, dot(R, E)), shininess);

    // Calculate color
    float self = max(0.0, dot(N, L));
    vec4 res   = self * color * (diffuse*lightDiff*Idiff + specular*lightSpec*Ispec);

    // Preserve transparency, but make sure that all pixels of the sphere have the same transparency value
    res.a = color.a;
    return res;
}
