
#version 330


#define WIDTH 1024


////////////////////////////////////////////////////////////////////////////////////////////////////
// Uniforms
////////////////////////////////////////////////////////////////////////////////////////////////////

// Textures
uniform sampler2D nodeSelection;

// Options
uniform bool  selection    = false;
uniform float neighborhood = 1.0;
uniform int   hoveredID    = -1;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Functions
////////////////////////////////////////////////////////////////////////////////////////////////////

float getNeighborhood(int nodeId)
{
    int x = nodeId % WIDTH;
    int y = nodeId / WIDTH;

    if (selection) return texelFetch(nodeSelection, ivec2(x, y), 0).r;
    else           return 100.0;
}

bool isSelected(int nodeId)
{
    float value = getNeighborhood(nodeId);

    return (value < 100.0 && value <= neighborhood) || !selection;
}

bool isEdgeSelected(int nodeId1, int nodeId2)
{
    float w = min(getNeighborhood(nodeId1), getNeighborhood(nodeId2));
    return (w <= neighborhood - 1) || !selection;
}

bool isHovered(int nodeId)
{
    float value = getNeighborhood(nodeId);

    return (nodeId == hoveredID || value == 1.0);
}
