
#version 330


////////////////////////////////////////////////////////////////////////////////////////////////////
// Uniforms
////////////////////////////////////////////////////////////////////////////////////////////////////

// The visualized area (lat/lon/lat/lon in radians)
uniform vec4 area;

// Temporal filters
uniform vec2 timeSpan1;
uniform vec2 timeSpan2;
uniform vec2 timeSpan3;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Input data
////////////////////////////////////////////////////////////////////////////////////////////////////

// Position (lat / lon / height) in radians
layout (location = 0) in vec4 position;

// Node-ID / Line-ID / Time / -
layout (location = 1) in vec4 attrs;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Output to geometry shader
////////////////////////////////////////////////////////////////////////////////////////////////////

out vec3  v_position; // Position
out float v_visible1; // Visiblity (timespan #1)
out float v_visible2; // Visiblity (timespan #2)
out float v_visible3; // Visiblity (timespan #3)
out float v_density;  // Density value


////////////////////////////////////////////////////////////////////////////////////////////////////
// Imported functions
////////////////////////////////////////////////////////////////////////////////////////////////////

bool filterTimeSpan1(float time);
bool filterTimeSpan2(float time);
bool filterTimeSpan3(float time);


////////////////////////////////////////////////////////////////////////////////////////////////////
// Vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

void main(void)
{
    v_position = vec3( 2.0 * (position.y - area.y) / (area.w - area.y) - 1.0,
                       2.0 * (position.x - area.x) / (area.z - area.x) - 1.0,
                       0.0 );

    v_visible1 = filterTimeSpan1(attrs.z) ? 0.0 : 1.0;
    v_visible2 = filterTimeSpan2(attrs.z) ? 0.0 : 1.0;
    v_visible3 = filterTimeSpan3(attrs.z) ? 0.0 : 1.0;

    v_density = 0.5;
}
