
#version 330


////////////////////////////////////////////////////////////////////////////////////////////////////
// Uniforms
////////////////////////////////////////////////////////////////////////////////////////////////////

// Matrices
uniform mat4 viewProjectionMatrix;

// Area of map overlay
uniform vec4 area;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Input data
////////////////////////////////////////////////////////////////////////////////////////////////////

// Position (x / y / z / w)
layout (location = 0) in vec4 position;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Output to fragment shader
////////////////////////////////////////////////////////////////////////////////////////////////////

out vec2 v_uv; // Texture coordinates


////////////////////////////////////////////////////////////////////////////////////////////////////
// Imported functions
////////////////////////////////////////////////////////////////////////////////////////////////////

vec4 fromLatLon(vec3 latLonHeight);


////////////////////////////////////////////////////////////////////////////////////////////////////
// Vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

void main(void)
{
    // Get position in range [0..1]
    vec2 p = (position.xy + vec2(1.0, 1.0)) / 2.0;

    // Get position in lat/lon coordinates
    vec3 pos = vec3(mix(area.x, area.z, p.x), mix(area.y, area.w, p.y), 0.0);

    // Transform point
    gl_Position = viewProjectionMatrix * fromLatLon(pos);

    // Set attributes
    v_uv = vec2(p.y, p.x);
}
