
#version 330


////////////////////////////////////////////////////////////////////////////////////////////////////
// Uniforms
////////////////////////////////////////////////////////////////////////////////////////////////////

// Mapping options
uniform int edgeColor  = -1;
uniform int edgeWidth  = -1;
uniform int nodeHeight = -1;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Input data
////////////////////////////////////////////////////////////////////////////////////////////////////

// Position (lat / lon / height) in radians
layout (location = 0) in vec4 position;

// Node-ID / Edge-ID / Time / -
layout (location = 1) in vec4 attrs;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Output to geometry shader
////////////////////////////////////////////////////////////////////////////////////////////////////

flat out int   v_nodeId;  // Node ID
flat out int   v_edgeId;  // Edge ID
     out vec4  v_pos;     // Position (world space)
     out vec3  v_latlon;  // Position (lat/lon/height)
     out float v_visible; // Edge visiblity
     out float v_width;   // Edge width (0..1)
     out float v_color;   // Edge color (0..1, mapped on color map)


////////////////////////////////////////////////////////////////////////////////////////////////////
// Imported functions
////////////////////////////////////////////////////////////////////////////////////////////////////

vec4 fromLatLon(vec3 latLonHeight);
float attributeValue(int nodeIndex, int edgeIndex, int attr);
float normalizedAttribute(int nodeIndex, int edgeIndex, int attr);
float normalizedAttributeLinear(int nodeIndex, int edgeIndex, int attr);
float filterAttrs(int nodeIndex, int edgeIndex);
bool filterTimeSpan1(float time);


////////////////////////////////////////////////////////////////////////////////////////////////////
// Vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

void main(void)
{
    // Get height
    float h = position.z;
    if (nodeHeight > -1) {
        h = attributeValue(int(attrs.x), int(attrs.y), nodeHeight);
    }

    // Transform point
    v_latlon    = vec3(position.x, position.y, h);
    v_pos       = fromLatLon(vec3(position.x, position.y, 0.0));

    // Set attributes
    v_nodeId  = int(attrs.x);
    v_edgeId  = int(attrs.y);
    v_visible = filterAttrs(int(attrs.x), int(attrs.y)) * (filterTimeSpan1(attrs.z) ? 0.0 : 1.0);
    v_color   = normalizedAttribute(int(attrs.x), int(attrs.y), edgeColor);
    v_width   = normalizedAttribute(int(attrs.x), int(attrs.y), edgeWidth);
}
