
#version 330


////////////////////////////////////////////////////////////////////////////////////////////////////
// Uniforms
////////////////////////////////////////////////////////////////////////////////////////////////////

// Matrices
uniform mat4 projectionMatrix;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Input from geometry shader
////////////////////////////////////////////////////////////////////////////////////////////////////

flat in int   g_id;      // ID
     in vec3  g_pos;     // Fragment position
     in vec3  g_center;  // Sphere center
     in vec3  g_up;      // Up vector
     in vec4  g_color;   // Sphere color (0..1, mapped on color map)
     in float g_radius;  // Sphere radius
     in float g_hovered; // Is the element currently hovered by the mouse?


////////////////////////////////////////////////////////////////////////////////////////////////////
// Output to frame buffer
////////////////////////////////////////////////////////////////////////////////////////////////////

layout (location = 0) out vec4  color;
layout (location = 1) out float id;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Imported functions
////////////////////////////////////////////////////////////////////////////////////////////////////

vec4 lightingAniso(vec3 position, vec3 tangent, vec3 normal, vec4 color);


////////////////////////////////////////////////////////////////////////////////////////////////////
// Fragment shader
////////////////////////////////////////////////////////////////////////////////////////////////////

void main()
{
    // Ray tracing center -> pos
    vec3 origin = vec3(0.0, 0.0, 0.0);
    vec3 dst = origin - g_center;
    vec3 dir = normalize(g_pos-origin);
    float r = 0.0;
    float B = dot(dst, dir);
    float C = dot(dst, dst) - g_radius*g_radius;
    float D = B*B - C;
    if (D<0.0) discard;
    if (D==0.0) {
        r = -B;
    } else {
        float Ds = sqrt(D);
        float t =  -B - Ds;
        if (t>0.0) {
            r = t;
        } else {
            t =  -B + Ds;
            if (t>0.0) {
                r = t;
            } else {
                discard;
            }
        }
    }

    vec3 pK = r*dir;
    vec3 normal = pK-g_center;
    vec3 tangent = cross(normal, g_up);
    vec4 proj = projectionMatrix * vec4(pK, 1.0);

    vec4 highlight = vec4(1.0, 0.0, 0.0, 0.5 * g_hovered);
    color = lightingAniso(pK, tangent, normal, g_color) * (1.0 - highlight.a) + highlight * highlight.a;
    id    = float(g_id);
}
