
#version 330


////////////////////////////////////////////////////////////////////////////////////////////////////
// Uniforms
////////////////////////////////////////////////////////////////////////////////////////////////////

// Matrices
uniform mat4 viewMatrix;
uniform mat4 viewProjectionMatrix;

// Mapping options
uniform int nodeColor  = -1;
uniform int nodeSize   = -1;
uniform int nodeHeight = -1;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Input data
////////////////////////////////////////////////////////////////////////////////////////////////////

// Position (lat / lon / height) in radians
layout (location = 0) in vec4 position;

// Node-ID / - / Time / -
layout (location = 1) in vec4 attrs;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Output to geometry shader
////////////////////////////////////////////////////////////////////////////////////////////////////

     out vec4  v_pos;     // Position
flat out int   v_id;      // Node ID
     out float v_visible; // Node visiblity
     out float v_color;   // Node color (0..1, mapped on color map)
     out float v_size;    // Node size (0..1)


////////////////////////////////////////////////////////////////////////////////////////////////////
// Imported functions
////////////////////////////////////////////////////////////////////////////////////////////////////

vec4 fromLatLon(vec3 latLonHeight);
float attributeValue(int nodeIndex, int edgeIndex, int attr);
float normalizedAttribute(int nodeIndex, int edgeIndex, int attr);
float normalizedAttributeLinear(int nodeIndex, int edgeIndex, int attr);
float filterNodeAttrs(int nodeIndex);
float filterTemporal(float time);


////////////////////////////////////////////////////////////////////////////////////////////////////
// Vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

void main(void)
{
    // Get height
    float h = position.z;
    if (nodeHeight > -1) {
        h = attributeValue(int(attrs.x), int(attrs.y), nodeHeight);
    }

    // Transform point
    v_pos = viewMatrix * fromLatLon(vec3(position.x, position.y, h));
    gl_Position = v_pos;

    // Set attributes
    v_id      = int(attrs.x);
    v_visible = filterNodeAttrs(int(attrs.x)) * filterTemporal(attrs.z);
    v_color   = normalizedAttributeLinear(int(attrs.x), int(attrs.y), nodeColor);
    v_size    = normalizedAttribute(int(attrs.x), int(attrs.y), nodeSize);
}
