
#version 330


layout(lines) in;
layout(points, max_vertices = 2) out;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Uniforms
////////////////////////////////////////////////////////////////////////////////////////////////////

// Options
uniform int selectedIds[50];
uniform int idAttributeIndex = -1;

// Textures
uniform sampler2D nodeSelection;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Input from vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

flat in int v_nodeId[2];  // Node ID
flat in int v_edgeId[2];  // Edge ID


////////////////////////////////////////////////////////////////////////////////////////////////////
// Output to fragment shader
////////////////////////////////////////////////////////////////////////////////////////////////////

out float g_value;


////////////////////////////////////////////////////////////////////////////////////////////////////
// Imported functions
////////////////////////////////////////////////////////////////////////////////////////////////////

float attributeValue(int nodeIndex, int edgeIndex, int attr);


////////////////////////////////////////////////////////////////////////////////////////////////////
// Geometry shader
////////////////////////////////////////////////////////////////////////////////////////////////////

float getNeighborhood(int nodeId)
{
    int x = nodeId % 1024;
    int y = nodeId / 1024;

    return texelFetch(nodeSelection, ivec2(x, y), 0).r;
}

int getId(int nodeId)
{
    if (idAttributeIndex > -1) {
        return int(attributeValue(nodeId, v_edgeId[0], idAttributeIndex));
    } else {
        return nodeId;
    }
}

void computeEdge(int sourceId, int destId)
{
    bool selected = false;
    for (int i=0; i<50; i++) {
        if (getId(selectedIds[i]) == getId(destId)) selected = true;
    }

    float value = selected ? 1.0 : getNeighborhood(sourceId) + 1.0;
    g_value = min(value, 100.0);

    int px = destId % 1024;
    int py = destId / 1024;
    float x = (float(px) / 1024.0) * 2.0 - 1.0 + 0.5 / 1024.0;
    float y = (float(py) / 1024.0) * 2.0 - 1.0 + 0.5 / 1024.0;
    gl_Position = vec4(x, y, 0.0, 1.0);

    EmitVertex();
    EndPrimitive();
}

void main(void)
{
    int nodeId1 = v_nodeId[0];
    int nodeId2 = v_nodeId[1];

    computeEdge(nodeId1, nodeId2);
    computeEdge(nodeId2, nodeId1);
}
