
#version 330


// Definitions
#define PI 3.14159265359
#define k0 1.0


// Uniforms
uniform sampler2D      worldTexture;
uniform sampler2DArray mapLayersTexture;


// Input
in vec2 v_uv;


// Output
layout (location = 0) out vec4 color;


// Imported functions
vec4 mapColor(vec2 uv);


void main(void)
{
    // Get position
    float x = v_uv.x * PI;       // (-1.0 .. 1.0) -> (-PI .. +PI)
    float y = v_uv.y * 2.0 * PI; // (-0.5 .. 0.5) -> (-PI .. +PI)

    // Inverse tranformation
    float D = (y / k0);
    float xDash = x / k0;

    float lat = asin(sin(D) / cosh(xDash));
    float lon = atan(sinh(xDash) / cos(D));

    // Correct longitude for values outside of atan() range
    if (y > 0.5 * PI)
        lon += PI;
    if (y < -0.5 * PI)
        lon -= PI;

    // Compute tex coords
    float texU = 0.5 + (lon / PI / 2);
    float texV = 0.5 + (lat / PI);

    if (texU <  0.0) texU += 1.0;
    if (texU >= 1.0) texU -= 1.0;
    if (texV <  0.0) texV += 1.0;
    if (texV >= 1.0) texV -= 1.0;

    // Set color
    color = mapColor(vec2(texU, texV));
}
