
#version 330


#define RADIUS_EQUATOR        1.0 // 0.81 // 6378137.0
#define RADIUS_POLAR          1.0 // 0.81 // 6356752.3142
#define ECCENTRICITY_SQUARED  0.0 // 0.00669438
#define E_DASH_SQUARED        0.0 // 0.006739497


uniform float heightScale  = 0.000001;
uniform float heightOffset = 0.2;


// Convert Lat/Long/Height coordinate to absolute coordinates
vec4 fromLatLon(vec3 latLongHeight)
{
    float sin_lat = sin(latLongHeight.x);
    float cos_lat = cos(latLongHeight.x);
    float N = RADIUS_EQUATOR / sqrt(1.0 - ECCENTRICITY_SQUARED*sin_lat*sin_lat);

    float h = (latLongHeight.z + heightOffset) * heightScale;

    vec4 result;
    result.x = (N+h)*cos_lat*cos(-latLongHeight.y);
    result.y = (N+h)*cos_lat*sin(-latLongHeight.y);
    result.z = (N*(1.0-ECCENTRICITY_SQUARED)+h)*sin_lat;
    result.w = 1.0;

    // [TODO]
    return result.xzyw;
}

// Get latitude from given absolute coordinates
float getLatFromAbsolute(vec3 absolutePos)
{
    float p = length(absolutePos.xy);
    float theta = atan(absolutePos.z * RADIUS_EQUATOR, p * RADIUS_POLAR);
    float sinTheta = sin(theta);
    float cosTheta = cos(theta);

    return atan(absolutePos.z + E_DASH_SQUARED * RADIUS_POLAR * sinTheta * sinTheta * sinTheta,
                p - ECCENTRICITY_SQUARED * RADIUS_EQUATOR * cosTheta * cosTheta * cosTheta);
}

// Move the given absolute coordinate to the earth surface
vec3 correctHeight(vec3 absolutePos, float height)
{
    float lat = getLatFromAbsolute(absolutePos);
    float sinlat2 = sin(lat) * sin(lat);
    float coslat2 = 1.0 - sinlat2;
    float newRadius = RADIUS_EQUATOR * sqrt(
        (coslat2 + (1-ECCENTRICITY_SQUARED) * (1-ECCENTRICITY_SQUARED) * sinlat2) /
        (1 - ECCENTRICITY_SQUARED * sinlat2));
    return normalize(absolutePos) * (newRadius + (height + heightOffset) * heightScale);
}
