
#version 330


#define PI 3.14159265359

#define MERC_MINX  (-1.0)
#define MERC_MAXX  ( 1.0)

#define MERC_MINY  (-0.5)
#define MERC_MAXY  ( 0.5)

#define MERC_WIDTH  (MERC_MAXX - MERC_MINX)
#define MERC_HEIGHT (MERC_MAXY - MERC_MINY)


uniform float heightScale  = 1.0;
uniform float heightOffset = 0.0;


// Convert Lat/Long/Height coordinate to absolute coordinates
vec4 fromLatLon(vec3 latLongHeight)
{
    float lat = latLongHeight.x;
    float lon = latLongHeight.y;

    if (lon >  PI)   lon -= 2 * PI;
    if (lon < -PI)   lon += 2 * PI;
    if (lat >  PI/2) lat -= PI;
    if (lat < -PI/2) lat += PI;

    float h = latLongHeight.z + heightOffset;

    float xr = clamp(lon / PI, -1.0, 1.0);
    float yr = clamp(lat / (0.5*PI), -1.0, 1.0);

    return vec4(xr * MERC_MAXX, yr * MERC_MAXY, h * heightScale, 1.0);
}

// Move the given absolute coordinate to the earth surface
vec3 correctHeight(vec3 absolutePos, float heightDelta)
{
    absolutePos.z += heightDelta * heightScale;

    return absolutePos;
}
