
#version 330


#define PI 3.14159265359
#define k0 1.0


uniform float tmercRadius  = 1.0;
uniform float heightScale  = 1.0;
uniform float heightOffset = 0.0;


// Convert Lat/Long/Height coordinate to absolute coordinates
vec4 fromLatLon(vec3 latLongHeight)
{
    float lat = mod(latLongHeight.x + PI * 0.5, PI) - PI * 0.5;
    float lon = mod(latLongHeight.y + PI, 2.0 * PI) - PI;

    float B = cos(lat) * sin(lon);
    float x = k0 * tmercRadius * 0.5 * log((1.0 + B) / (1.0 - B)) / PI;
    float y = k0 * tmercRadius * atan(tan(lat) / cos(lon)) / PI;

    if (lon > PI * 0.5 || lon < -PI * 0.5) {
        if (lat > 0.0) {
            y = k0 * tmercRadius + y;
        } else {
            y = -k0 * tmercRadius + y;
        }
    } 

    return vec4(x * 0.9, y * 0.5, (latLongHeight.z + heightOffset) * heightScale, 1.0);
}

// Move the given absolute coordinate to the earth surface
vec3 correctHeight(vec3 absolutePos, float heightDelta)
{
    absolutePos.z += heightDelta * heightScale;
    
    return absolutePos;
}
