
#include <QString>
#include <QSurfaceFormat>

#include <cppexpose/scripting/ScriptContext.h>

#include <gloperate/base/Environment.h>

#include <gloperate-qt/base/GLContext.h>
#include <gloperate-qt/base/GLContextFactory.h>

#include <gloperate-qtquick/Application.h>

#include <gtx/gtx-version.h>
#include <gtxlib-core/gtxlib-core.h>

#include <gtxlib-stages/AirTrafficPipeline.h>


using namespace gloperate_qtquick;
using namespace gtx;


int main(int argc, char * argv[])
{
    // Initialize
    Application::initialize();
    AirTrafficPipeline::initialize();

    // Initialize Qt application
    Application app(argc, argv);
    app.setApplicationName   (GTX_PROJECT_NAME);
    app.setApplicationVersion(GTX_VERSION);
    app.setOrganizationName  (GTX_AUTHOR_ORGANIZATION);
    app.setOrganizationDomain(GTX_AUTHOR_DOMAIN);

    // Override default OpenGL context format
    gloperate::GLContextFormat format;
    format.setVersion(3, 2);
    format.setProfile(gloperate::GLContextFormat::Profile::Core);
    format.setForwardCompatible(false);

    QSurfaceFormat qFormat = gloperate_qt::GLContextFactory::toQSurfaceFormat(format);
    QSurfaceFormat::setDefaultFormat(qFormat);

    // Locate data path
    auto dataPath = gtx::dataPath();

    // Load and show QML
    app.loadQml(QString::fromStdString(dataPath) + "/qml/AirTrafficViewer.qml");

    // Load and run script
    app.environment().scriptContext()->evaluate("gloperate.system.load('" + dataPath + "/scripts/AirTraffic.js')");

    // Run main loop
    return app.exec();
}
