
#pragma once


#include <gtxlib-core/base/Position.h>


namespace gtx
{


/**
*  @brief
*    Geographic area
*/
class GTXLIB_CORE_API Area
{
public:
    /**
    *  @brief
    *    Constructor
    */
    Area();

    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] minPos
    *    Position #1
    *  @param[in] maxPos
    *    Position #2
    */
    Area(const Position & minPos, const Position & maxPos);

    /**
    *  @brief
    *    Constructor
    *
    *  @param area
    *    Area to copy
    */
    Area(const Area & area);

    /**
    *  @brief
    *    Destructor
    */
    ~Area();

    /**
    *  @brief
    *    Comparison operator
    */
    bool operator ==(const Area & area) const;

    /**
    *  @brief
    *    Comparison operator
    */
    bool operator !=(const Area & area) const;

    /**
    *  @brief
    *    Assignment operator
    */
    Area & operator =(const Area & area);

    /**
    *  @brief
    *    Check if area is empty
    *
    *  @return
    *    'true' if area is empty, else 'false'
    */
    bool empty() const;

    /**
    *  @brief
    *    Clear area
    */
    void clear();

    /**
    *  @brief
    *    Get position #1
    *
    *  @return
    *    Position
    */
    Position minPos() const;

    /**
    *  @brief
    *    Set position #1
    *
    *  @param pos
    *    Position
    */
    void setMinPos(const Position & pos);

    /**
    *  @brief
    *    Get position #2
    *
    *  @return
    *    Position
    */
    Position maxPos() const;

    /**
    *  @brief
    *    Set position #2
    *
    *  @param pos
    *    Position
    */
    void setMaxPos(const Position & pos);

    /**
    *  @brief
    *    Expand area
    *
    *  @param pos
    *    Position
    */
    void expand(const Position & pos);

    /**
    *  @brief
    *    Get center position
    *
    *  @return
    *    Position
    */
    Position center() const;

    /**
    *  @brief
    *    Get width
    *
    *  @return
    *    Width
    */
    float width() const;

    /**
    *  @brief
    *    Get height
    *
    *  @return
    *    Height
    */
    float height() const;


protected:
    bool     m_empty;   ///< Is the area still empty?
    Position m_minPos;  ///< Position #1
    Position m_maxPos;  ///< Position #2
};


} // namespace gtx
