
#pragma once


#include <glm/glm.hpp>

#include <gtxlib-core/gtxlib-core_api.h>


namespace gtx
{


/**
*  @brief
*    Mapping configuration
*/
struct Configuration
{
    float geometryType;           ///< 1 for tubes, 2 for spheres
    float tessellation;           ///< Tessellation factor (default: 16)
    float textureID;              ///< ID of texture to use
    float colorMapID;             ///< ID of color map to use
    float height;                 ///< Attribute or value for node height
    float radius;                 ///< Attribute or value for tube radius
    float color;                  ///< Attribute or value for color
    float alpha;                  ///< Attribute or value for alpha
    float textureX;               ///< Attribute or value for texture stretch
    float textureY;               ///< Attribute or value for texture torsion
    float minRadius;              ///< Minimum radius (used when radius == 0.0)
    float maxRadius;              ///< Maximum radius (used when radius == 1.0)
    float sizeOfSingleStretch;    ///< Length of one texture before it repeats
    float sizeOfSingleTorsion;    ///< Length of one texture torsion
    float animationSpeed;         ///< Speed of animation (in seconds)
};


/**
*  @brief
*    A vertex configuration
*/
struct Vertex {
    glm::vec3 position;            ///< Vertex position
    int       geometryType;        ///< 1 for tubes, 2 for spheres
    float     tessellation;        ///< Tessellation factor (default: 16)
    int       textureID;           ///< ID of texture to use
    int       colorMapID;          ///< ID of color map to use
    float     radius;              ///< Attribute or value for tube radius
    float     color;               ///< Attribute or value for color
    float     alpha;               ///< Attribute or value for alpha
    float     textureX;            ///< Attribute or value for texture stretch
    float     textureY;            ///< Attribute or value for texture torsion
    float     minRadius;           ///< Minimum radius (used when radius == 0.0)
    float     maxRadius;           ///< Maximum radius (used when radius == 1.0)
    float     sizeOfSingleStretch; ///< Length of one texture before it repeats
    float     sizeOfSingleTorsion; ///< Length of one texture torsion
    float     animationSpeed;      ///< Speed of animation (in seconds)
    int       lineId;              ///< ID of line to which the vertex belongs
};


} // namespace gtx
