
#pragma once


#include <vector>
#include <cstdint>
#include <memory>

#include <gtxlib-core/gtxlib-core_api.h>


namespace gtx
{


/**
*  @brief
*    Drop in replacement for QDataStream, based on http://stackoverflow.com/a/31687943
*/
class GTXLIB_CORE_API DataStream
{
public:
    typedef std::vector<uint8_t> Storage;
    enum ByteOrder { BigEndian, LittleEndian };
    enum Status { Ok, ReadPastEnd };


public:
    DataStream(const std::vector<uint8_t> * data);
    DataStream(std::shared_ptr<Storage> data);
    DataStream(std::unique_ptr<Storage> && data);
    DataStream(Storage && data);

    bool atEnd() const;

    DataStream & operator>>(int8_t & i);
    DataStream & operator>>(int16_t & i);
    DataStream & operator>>(int32_t & i);
    DataStream & operator>>(uint8_t & i);
    DataStream & operator>>(uint16_t & i);
    DataStream & operator>>(uint32_t & i);
    DataStream & operator>>(float & i);
    DataStream & operator>>(double & i);
    DataStream & operator>>(std::string & s);

    void setByteOrder(ByteOrder b);
    ByteOrder byteOrder() const;

    size_t position() const;


private:
    std::shared_ptr<const Storage> m_ptr;
    const Storage * m_data;
    size_t m_idx;
    Status m_status;
    ByteOrder m_byteOrder, m_systemOrder;


private:
    static ByteOrder systemByteOrder();
    bool has(size_t count) const;
    
    template <typename T>
    DataStream & read(T & i);
};


} // namespace gtx


#include "DataStream.hpp"
