
#pragma once


#include <glm/vec3.hpp>

#include <gtxlib-core/gtxlib-core_api.h>


namespace gtx
{


/**
*  @brief
*    Geographic position (X/Y/Z)
*/
class GTXLIB_CORE_API Position : public glm::vec3
{
public:
    static Position LatLong(float latitude, float longitude, float height = 0.0f);


public:
    /**
    *  @brief
    *    Constructor
    */
    Position();

    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] x
    *    X/lat position
    *  @param[in] y
    *    Y/lon position
    *  @param[in] z
    *    Z/h position
    */
    Position(float x, float y, float z);

    /**
    *  @brief
    *    Constructor
    *
    *  @param v
    *    Position vector
    */
    Position(const glm::vec3 & v);

    /**
    *  @brief
    *    Constructor
    *
    *  @param position
    *    Position to copy
    */
    Position(const Position & position);

    /**
    *  @brief
    *    Destructor
    */
    ~Position();

    /**
    *  @brief
    *    Comparison operator
    */
    bool operator ==(const Position & position) const
    {
        return (x == position.x && y == position.y && z == position.z);
    }

    /**
    *  @brief
    *    Assignment operator
    */
    Position &operator =(const Position & position)
    {
        return (Position&)glm::vec3::operator =(position);
    }

    /**
    *  @brief
    *    Get latitude
    *
    *  @return
    *    Latitude
    */
    float getLat() const;

    /**
    *  @brief
    *    Set latitude
    *
    *  @param latitude
    *    Latitude
    */
    void setLat(float latitude);

    /**
    *  @brief
    *    Get longitude
    *
    *  @return
    *    Longitude
    */
    float getLong() const;

    /**
    *  @brief
    *    Set longitude
    *
    *  @param longitude
    *    Longitude
    */
    void setLong(float longitude);

    /**
    *  @brief
    *    Get height
    *
    *  @return
    *    Height (m)
    */
    float getHeight() const;

    /**
    *  @brief
    *    Set height
    *
    *  @param height
    *    Height (m)
    */
    void setHeight(float height);
};


} // namespace gtx
