
#pragma once


#include <string>

#include <gtxlib-core/gtxlib-core_api.h>


namespace gtx
{


/**
*  @brief
*    Value describing a texture file
*/
class GTXLIB_CORE_API TextureMap
{
public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] filename
    *    File name
    */
    TextureMap(const std::string & filename = "");

    /**
    *  @brief
    *    Copy constructor
    *
    *  @param source
    *    Source texture map
    */
    TextureMap(const TextureMap & source);

    /**
    *  @brief
    *    Destructor
    */
    ~TextureMap();

    /**
    *  @brief
    *    Copy operator
    *
    *  @param source
    *    Source texture map
    */
    const TextureMap & operator =(const TextureMap & source);

    /**
    *  @brief
    *    Get filename
    *
    *  @return
    *    File name
    */
    std::string filename() const;

    /**
    *  @brief
    *    Set filename
    *
    *  @param[in] filename
    *    File name
    */
    void setFilename(const std::string & filename);


protected:
    std::string m_filename; ///< Texture filename
};


} // namespace gtx
