
#pragma once


#include <string>

#include <gtxlib-core/base/DataTypes.h>

#include <gtxlib-core/gtxlib-core_api.h>


namespace gtx
{


/**
*  @brief
*    Time functions
*/
class GTXLIB_CORE_API Time
{
public:
    /**
    *  @brief
    *    Convert time string to time
    *
    *  @param[in] str
    *    String containing a date and time ("%Y-%m-%d %T")
    *
    *  @return
    *    Time
    */
    static time_point fromTimeString(const std::string & str);

    /**
    *  @brief
    *    Convert date string to time
    *
    *  @param[in] str
    *    String containing a date ("%Y-%m-%d")
    *
    *  @return
    *    Time
    */
    static time_point fromDateString(const std::string & str);

    /**
    *  @brief
    *    Convert time to string
    *
    *  @param[in] time
    *    Time
    *
    *  @return
    *    String containing a date and time ("%Y-%m-%d %T")
    */
    static std::string toString(const time_point & time);

    /**
    *  @brief
    *    Convert time point to integer number (number of seconds)
    *
    *  @param[in] time
    *    Time point
    *
    *  @return
    *    Integer representation of time
    */
    static unsigned long toInt(const time_point & time);
};


} // namespace gtx
