
#pragma once


#include <string>

#include <gtxlib-core/gtxlib-core.h>


namespace gtx
{


/**
*  @brief
*    Attribute of a data item
*/
class GTXLIB_CORE_API Attribute
{
public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] name
    *    Attribute name
    */
    Attribute(const std::string & name);

    /**
    *  @brief
    *    Destructor
    */
    ~Attribute();

    /**
    *  @brief
    *    Get name
    *
    *  @return
    *    Attribute name
    */
    const std::string & name() const;

    /**
    *  @brief
    *    Get value
    *
    *  @return
    *    Attribute value
    */
    float value() const;

    /**
    *  @brief
    *    Set value
    *
    *  @param[in] value
    *    Attribute value
    */
    void setValue(float value);


protected:
    std::string m_name;  ///< Attribute name
    float       m_value; ///< Attribute value
};


} // namespace gtx
