
#pragma once


#include <gtxlib-core/base/Position.h>
#include <gtxlib-core/data/Item.h>


namespace gtx
{


class Node;


/**
*  @brief
*    Edge between two nodes
*/
class GTXLIB_CORE_API Edge : public Item
{
public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] id
    *    ID
    */
    Edge(id_t id = null_id);

    /**
    *  @brief
    *    Destructor
    */
    virtual ~Edge();

    /**
    *  @brief
    *    Get node #1
    *
    *  @return
    *    Node
    */
    Node * node1() const;

    /**
    *  @brief
    *    Set node #1
    *
    *  @param node
    *    Node
    */
    void setNode1(Node * node);

    /**
    *  @brief
    *    Get node #2
    *
    *  @return
    *    Node
    */
    Node * node2() const;

    /**
    *  @brief
    *    Set node #2
    *
    *  @param node
    *    Node
    */
    void setNode2(Node * node);

    /**
    *  @brief
    *    Get start position (position of node #1)
    *
    *  @return
    *    2D position
    */
    Position startPos() const;

    /**
    *  @brief
    *    Get end position (position of node #2)
    *
    *  @return
    *    2D position
    */
    Position endPos() const;


protected:
    Node * m_node1; ///< Node #1
    Node * m_node2; ///< Node #2
};




} // namespace gtx
