
#pragma once


#include <cppassist/memory/make_unique.h>


namespace gtx
{


template <typename T>
ItemGroup<T>::ItemGroup()
: m_firstId(null_id)
, m_lastId(null_id)
, m_nextId(0)
{
}

template <typename T>
ItemGroup<T>::~ItemGroup()
{
}

template <typename T>
void ItemGroup<T>::clear()
{
    // Destroy owned items
    m_ownItems.clear();

    // Clear list and reset IDs
    m_items.clear();
    m_firstId = null_id;
    m_lastId  = null_id;
    m_nextId  = 0;
}

template <typename T>
bool ItemGroup<T>::empty() const
{
    return m_items.size() == 0;
}

template <typename T>
size_t ItemGroup<T>::size() const
{
    return m_items.size();
}

template <typename T>
bool ItemGroup<T>::contains(id_t id) const
{
    auto it = m_items.find(id);
    return (it != m_items.end());
}

template <typename T>
const std::unordered_map<id_t, Item *> & ItemGroup<T>::items() const
{
    auto * itemGroup = reinterpret_cast<const ItemGroup<Item> *>(this);
    return itemGroup->map();
}

template <typename T>
const Item * ItemGroup<T>::item(id_t id) const
{
    if (!contains(id))
    {
        return nullptr;
    }

    return m_items.at(id);
}

template <typename T>
Item * ItemGroup<T>::item(id_t id)
{
    if (!contains(id))
    {
        return nullptr;
    }

    return m_items.at(id);
}

template <typename T>
const std::unordered_map<id_t, T *> & ItemGroup<T>::map() const
{
    return m_items;
}

template <typename T>
const T * ItemGroup<T>::get(id_t id) const
{
    if (!contains(id))
    {
        return nullptr;
    }

    return m_items.at(id);
}

template <typename T>
T * ItemGroup<T>::get(id_t id)
{
    if (!contains(id))
    {
        return nullptr;
    }

    return m_items.at(id);
}

template <typename T>
T * ItemGroup<T>::create(id_t idIn)
{
    // Create new item
    auto item = cppassist::make_unique<T>();
    T * itemPtr = item.get();

    // Assign ID
    id_t id = idIn;
    if (id == null_id)
    {
        id = m_nextId;
        m_nextId++;
    }
    item->setId(id);

    // Add item to list
    m_ownItems.push_back(std::move(item));
    m_items[id] = itemPtr;

    // update min/max ID
    if (id < m_firstId || m_firstId == null_id) {
        m_firstId = id;
    }
    if (id > m_lastId  || m_lastId  == null_id) {
        m_lastId = id;
    }

    // Return the new item
    return itemPtr;
}


} // namespace gtx
