
#pragma once


#include <vector>

#include <gtxlib-core/data/Item.h>


namespace gtx
{


class Node;


/**
*  @brief
*    Line (list of connected nodes)
*/
class GTXLIB_CORE_API Line : public Item
{
public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] id
    *    ID
    */
    Line(id_t id = null_id);

    /**
    *  @brief
    *    Destructor
    */
    virtual ~Line();

    /**
    *  @brief
    *    Clear nodes
    */
    void clear();

    /**
    *  @brief
    *    Get number of nodes
    *
    *  @return
    *    Number of nodes
    */
    size_t numNodes() const;

    /**
    *  @brief
    *    Get nodes
    *
    *  @return
    *    List of nodes
    */
    const std::vector<Node *> & nodes() const;

    /**
    *  @brief
    *    Get nodes
    *
    *  @return
    *    List of nodes
    */
    std::vector<Node *> & nodes();

    /**
    *  @brief
    *    Get node
    *
    *  @param[in] index
    *    Nodes index
    *
    *  @return
    *    Nodes, null if node doesn't exist
    */
    const Node * node(size_t index) const;

    /**
    *  @brief
    *    Get node
    *
    *  @param[in] index
    *    Nodes index
    *
    *  @return
    *    Nodes, null if node doesn't exist
    */
    Node * node(size_t index);

    /**
    *  @brief
    *    Add node to line
    *
    *  @param[in] node
    *    Node (must NOT be null!)
    */
    void add(Node * node);


protected:
    std::vector<Node *> m_nodes; ///< List of nodes that belong to the line
};




} // namespace gtx
