
#pragma once


#include <gtxlib-core/data/Item.h>

#include <gtxlib-core/base/DataTypes.h>
#include <gtxlib-core/base/Position.h>


namespace gtx
{


/**
*  @brief
*    Basic node (vertex/point) data item
*/
class GTXLIB_CORE_API Node : public Item
{
public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] id
    *    ID
    */
    Node(id_t id = null_id);

    /**
    *  @brief
    *    Destructor
    */
    virtual ~Node();

    /**
    *  @brief
    *    Get edge or line ID
    *
    *  @return
    *    ID of line/edge to which the node belongs
    */
    id_t edgeId() const;

    /**
    *  @brief
    *    Set edge or line ID
    *
    *  @param id
    *    ID of line/edge to which the node belongs
    */
    void setEdgeId(id_t id);

    /**
    *  @brief
    *    Get position
    *
    *  @return
    *    Position
    */
    const Position & position() const;

    /**
    *  @brief
    *    Set position
    *
    *  @param[in] pos
    *    Position
    */
    void setPosition(const Position & pos);

    /**
    *  @brief
    *    Get time
    *
    *  @return
    *    Time point
    */
    time_point time() const;

    /**
    *  @brief
    *    Set time
    *
    *  @param[in] time
    *    Time point
    */
    void setTime(time_point time);

    /**
    *  @brief
    *    Get time as integer
    *
    *  @return
    *    Time point as integer number
    */
    unsigned long timeInt() const;


protected:
    id_t          m_edgeId;   ///< ID of line/edge to which the node belongs
    Position      m_position; ///< Node position
    time_point    m_time;     ///< Time point
    unsigned long m_timeInt;  ///< Time point as integer number
};


} // namespace gtx
