
#pragma once


#include <unordered_map>
#include <array>
#include <vector>

#include <glm/vec3.hpp>

#include <glbinding/gl/types.h>

#include <gtxlib-core/gtxlib-core_api.h>


namespace gtx
{


/**
*  @brief
*    Icosahedron geometry that can be refined dynamically
*/
class GTXLIB_CORE_API Icosahedron
{
public:
    using Face = std::array<gl::GLushort, 3>;


public:
    static std::array<glm::vec3, 12> baseVertices();
    static std::array<Face, 20> baseIndices(); /// individual triangle indices (no strip, no fan)

    /**
    *  @brief
    *    Iterative triangle refinement: split each triangle into 4 new ones and
    *    create points and indices appropriately
    */
    static void refine(
        std::vector<glm::vec3> & vertices
    ,   std::vector<Face> & indices
    ,   unsigned char levels);


public:
    Icosahedron();
    virtual ~Icosahedron();

    void generateGeometry(gl::GLsizei iterations = 0);

    const std::vector<glm::vec3> & vertices() const;
    const std::vector<std::array<gl::GLushort, 3>> & indices() const;


private:
    /**
    *  @brief
    *    Splits a triangle edge by adding an appropriate new point (normalized
    *    on sphere) to the points (if not already cached) and returns the index
    *    to this point.
    */
    static gl::GLushort split(
        gl::GLushort a
    ,   gl::GLushort b
    ,   std::vector<glm::vec3> & points
    ,   std::unordered_map<glm::uint, gl::GLushort> & cache);


private:
    std::vector<glm::vec3>  m_vertices;
    std::vector<Face>       m_indices;
};


} // namespace gtx
