
#pragma once


#include <memory>

#include <globjects/VertexArray.h>

#include <gloperate/rendering/AbstractDrawable.h>

#include <gtxlib-core/data/DataSet.h>


namespace globjects
{
    class Buffer;
}


namespace gtx
{


/**
*  @brief
*    Geometry for lines
*/
class GTXLIB_CORE_API LineGeometry : public gloperate::AbstractDrawable
{
public:
    /**
    *  @brief
    *    Constructor
    */
    LineGeometry();

    /**
    *  @brief
    *    Destructor
    */
    virtual ~LineGeometry();

    /**
    *  @brief
    *    Create geometry for lines
    *
    *  @param[in] dataset
    *    Data set
    */
    void setData(const DataSet & dataset);

    /**
    *  @brief
    *    Get size
    *
    *  @return
    *    Number of elements
    */
    unsigned int size() const;

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    globjects::VertexArray * vao();

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    const globjects::VertexArray * vao() const;

    // Virtual gloperate::AbstractDrawable interface
    virtual void draw() const override;


protected:
    std::unique_ptr<globjects::VertexArray> m_vao;              ///< Vertex array object
    std::unique_ptr<globjects::Buffer>      m_bufferVertices;   ///< Buffer for vertices
    std::unique_ptr<globjects::Buffer>      m_bufferParameters; ///< Buffer for vertex parameters
    unsigned int                            m_size;             ///< Number of elements
};


} // namespace gtx
