
#pragma once


#include <memory>

#include <globjects/VertexArray.h>

#include <gloperate/rendering/AbstractDrawable.h>

#include <gtxlib-core/data/DataSet.h>


namespace globjects
{
    class Buffer;
}


namespace gtx
{


/**
*  @brief
*    Geometry for nodes
*/
class GTXLIB_CORE_API NodeGeometry : public gloperate::AbstractDrawable
{
public:
    /**
    *  @brief
    *    Constructor
    */
    NodeGeometry();

    /**
    *  @brief
    *    Destructor
    */
    virtual ~NodeGeometry();

    /**
    *  @brief
    *    Create geometry for nodes
    *
    *  @param[in] dataset
    *    Data set
    *  @param[in] maxNodes
    *    Maximum number of nodes
    */
    void setData(const DataSet & dataset, unsigned int maxNodes);

    /**
    *  @brief
    *    Get number of elements
    *
    *  @return
    *    Number of elements
    */
    unsigned int numElements() const;

    /**
    *  @brief
    *    Get size
    *
    *  @return
    *    Size (in bytes)
    */
    unsigned int size() const;

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    globjects::VertexArray * vao();

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    const globjects::VertexArray * vao() const;

    /**
    *  @brief
    *    Draw nodes as patches
    */
    void drawPatches() const;

    // Virtual gloperate::AbstractDrawable interface
    virtual void draw() const override;


protected:
    std::unique_ptr<globjects::VertexArray> m_vao;              ///< Vertex array object
    std::unique_ptr<globjects::Buffer>      m_bufferVertices;   ///< Buffer for vertices
    std::unique_ptr<globjects::Buffer>      m_bufferParameters; ///< Buffer for vertex parameters
    unsigned int                            m_numElements;      ///< Number of elements
    unsigned int                            m_size;             ///< Size of geometry (in bytes)
};


} // namespace gtx
