
#pragma once


#include <gloperate/rendering/AbstractDrawable.h>

#include <gtxlib-core/gtxlib-core_api.h>


namespace globjects
{
    class VertexArray;
}


namespace gtx
{


class NodeGeometry;


/**
*  @brief
*    Drawable that renders a node geometry as patches instead of points
*/
class GTXLIB_CORE_API PatchGeometry : public gloperate::AbstractDrawable
{
public:
    /**
    *  @brief
    *    Constructor
    */
    PatchGeometry();

    /**
    *  @brief
    *    Destructor
    */
    virtual ~PatchGeometry();

    /**
    *  @brief
    *    Get node geometry
    *
    *  @return
    *    Node geometry
    */
    NodeGeometry * nodeGeometry() const;

    /**
    *  @brief
    *    Set node geometry
    *
    *  @param[in] nodeGeometry
    *    Node geometry
    */
    void setNodeGeometry(NodeGeometry * nodeGeometry);

    /**
    *  @brief
    *    Get number of elements
    *
    *  @return
    *    Number of elements
    */
    unsigned int numElements() const;

    /**
    *  @brief
    *    Get size
    *
    *  @return
    *    Size (in bytes)
    */
    unsigned int size() const;

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    globjects::VertexArray * vao();

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    const globjects::VertexArray * vao() const;

    // Virtual gloperate::AbstractDrawable interface
    virtual void draw() const override;


protected:
    NodeGeometry * m_nodeGeometry; ///< Node geometry
};


} // namespace gtx
