
#pragma once


#include <memory>

#include <globjects/VertexArray.h>

#include <gloperate/rendering/AbstractDrawable.h>

#include <gtxlib-core/data/DataSet.h>


namespace globjects
{
    class Buffer;
}


namespace gtx
{


/**
*  @brief
*    Geometry for line plots
*/
class GTXLIB_CORE_API PlotGeometry : public gloperate::AbstractDrawable
{
public:
    /**
    *  @brief
    *    Constructor
    */
    PlotGeometry();

    /**
    *  @brief
    *    Destructor
    */
    virtual ~PlotGeometry();

    /**
    *  @brief
    *    Create geometry for line plots
    *
    *  @param[in] dataset
    *    Data set
    *  @param[in] nodeId
    *    Currently selected node
    */
    void setData(const DataSet & dataset, int nodeId);

    /**
    *  @brief
    *    Get number of elements
    *
    *  @return
    *    Number of elements
    */
    unsigned int numElements() const;

    /**
    *  @brief
    *    Get size
    *
    *  @return
    *    Size (in bytes)
    */
    unsigned int size() const;

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    globjects::VertexArray * vao();

    /**
    *  @brief
    *    Get vertex array
    *
    *  @return
    *    Vertex array object
    */
    const globjects::VertexArray * vao() const;

    // Virtual gloperate::AbstractDrawable interface
    virtual void draw() const override;


protected:
    std::unique_ptr<globjects::VertexArray> m_vao;            ///< Vertex array object
    std::unique_ptr<globjects::Buffer>      m_bufferVertices; ///< Buffer for vertices
    unsigned int                            m_numElements;    ///< Number of elements
    unsigned int                            m_size;           ///< Size of geometry (in bytes)
};


} // namespace gtx
