
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <string>
#include <vector>
#include <map>

#include <gloperate/base/Loader.h>

#include <gtx/gtx-version.h>
#include <gtxlib-core/data/DataSet.h>


namespace gtx
{


class DataSet;


/**
*  @brief
*    Load data sets from CSV files containing an adjacency matrix
*/
class GTXLIB_CORE_API LoaderAjdMat : public gloperate::Loader<DataSet>
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        LoaderAjdMat, gloperate::AbstractLoader
      , "" // Tags
      , "" // Icon
      , "" // Annotations
      , "Load data sets from CSV files containing an adjacency matrix"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the loader belongs (must NOT be null!)
    */
    LoaderAjdMat(gloperate::Environment * environment);

    /**
    *  @brief
    *    Destructor
    */
    virtual ~LoaderAjdMat();


public:
    // Virtual gloperate::AbstractLoader functions
    virtual bool canLoad(const std::string & ext) const;
    virtual std::vector<std::string> loadingTypes() const;
    virtual std::string allLoadingTypes() const;

    // Virtual gloperate::Loader<DataSet> functions
    virtual DataSet * load(const std::string & filename, const cppexpose::Variant & options, std::function<void(int, int)> progress) const override;


protected:
    using NodeAttributes     = std::map<std::string, std::string>;
    using NodeAttributesList = std::vector<NodeAttributes>;
    using AttributeMapping   = std::map<std::string, std::string>;


protected:
    void readNodeAttributes(const std::string & filename, NodeAttributesList & nodeAttributes) const;
    void readAdjacencyMatrix(const std::string & filename, DataSet & dataset, int layer, const NodeAttributesList & nodeAttributes, const AttributeMapping & mapping) const;
};


} // namespace gtx
