
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/base/Loader.h>

#include <gtx/gtx-version.h>
#include <gtxlib-core/data/DataSet.h>


namespace gtx
{


/**
*  @brief
*    Load geographic networks in .graphml format
*/
class GTXLIB_CORE_API LoaderGraphML2 : public gloperate::Loader<DataSet>
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        LoaderGraphML2, gloperate::AbstractLoader
      , "" // Tags
      , "" // Icon
      , "" // Annotations
      , "Load geographic networks in .graphml format"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the loader belongs (must NOT be null!)
    */
    LoaderGraphML2(gloperate::Environment * environment);

    /**
    *  @brief
    *    Destructor
    */
    virtual ~LoaderGraphML2();


public:
    // Virtual gloperate::AbstractLoader functions
    virtual bool canLoad(const std::string & ext) const;
    virtual std::vector<std::string> loadingTypes() const;
    virtual std::string allLoadingTypes() const;

    // Virtual gloperate::Loader<DataSet> functions
    virtual DataSet * load(const std::string & filename, const cppexpose::Variant & options, std::function<void(int, int)> progress) const override;


protected:
    DataSet * convertDataSet(DataSet * dataset) const;
    void exportDataSet(DataSet * dataset, const std::string & filename) const;
};


} // namespace gtx
