
#pragma once


#include <vector>
#include <string>
#include <unordered_map>

#include <cppexpose/plugin/plugin_api.h>

#include <glbinding/gl/enum.h>

#include <gloperate/base/Loader.h>

#include <gtx/gtx-version.h>
#include <gtxlib-core/base/Position.h>


namespace gtx
{


class Dataset;
class AttributesDesc;
class Attributes;
class EdgeBundle;
class DataStream;


/**
*  @brief
*    Load a gtx data set
*
*  Supported options:
*    none
*/
class GTXLIB_CORE_API LoaderGtx : public gloperate::Loader<Dataset>
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        LoaderGtx, gloperate::AbstractLoader
      , "" // Tags
      , "" // Icon
      , "" // Annotations
      , "Load a gtx data set"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the loader belongs (must NOT be null!)
    */
    LoaderGtx(gloperate::Environment * environment);

    /**
    *  @brief
    *    Destructor
    */
    virtual ~LoaderGtx();

    // Virtual gloperate::AbstractLoader functions
    virtual bool canLoad(const std::string & ext) const;
    virtual std::vector<std::string> loadingTypes() const;
    virtual std::string allLoadingTypes() const;

    // Virtual gloperate::Loader<Dataset> functions
    virtual Dataset * load(const std::string & filename, const cppexpose::Variant & options, std::function<void(int, int)> progress) const override;


protected:
    uint32_t magicNumber() const;

    void readAttributesDesc(DataStream & stream, AttributesDesc * desc) const;
    void readAttributes(DataStream & stream, Attributes * attrs) const;

    Position readVec3(DataStream & stream) const;
    EdgeBundle * readBundle(DataStream & stream, Dataset * dataset) const;
};


} // namespace gtx
