
#include <gtxlib-core/base/Area.h>


namespace gtx
{


Area::Area()
: m_empty (true)
, m_minPos(0.0f, 0.0f, 0.0f)
, m_maxPos(0.0f, 0.0f, 0.0f)
{
}

Area::Area(const Position & minPos, const Position & maxPos)
: m_empty (false)
, m_minPos(minPos)
, m_maxPos(maxPos)
{
}

Area::Area(const Area & area)
: m_empty (area.m_empty)
, m_minPos(area.m_minPos)
, m_maxPos(area.m_maxPos)
{
}

Area::~Area()
{
}

bool Area::operator ==(const Area & area) const
{
    // Compare area
    return (m_empty == area.m_empty && m_minPos == area.m_minPos && m_maxPos == area.m_maxPos);
}

bool Area::operator !=(const Area & area) const
{
    // Compare area
    return (m_empty != area.m_empty || m_minPos != area.m_minPos || m_maxPos != area.m_maxPos);
}

Area & Area::operator =(const Area & area)
{
    // Copy area
    m_empty  = area.m_empty;
    m_minPos = area.m_minPos;
    m_maxPos = area.m_maxPos;
    return *this;
}

bool Area::empty() const
{
    return m_empty;
}

void Area::clear()
{
    // Reset values
    m_minPos = Position();
    m_maxPos = Position();
    m_empty  = true;
}

Position Area::minPos() const
{
    // Return position
    return m_minPos;
}

void Area::setMinPos(const Position & pos)
{
    // Set position
    m_minPos = pos;
    m_empty  = false;
}

Position Area::maxPos() const
{
    // Return position
    return m_maxPos;
}

void Area::setMaxPos(const Position & pos)
{
    // Set position
    m_maxPos = pos;
    m_empty  = false;
}

void Area::expand(const Position & pos)
{
    // Is area empty?
    if (m_empty) {
        // Set first position
        m_minPos = m_maxPos = pos;
        m_empty  = false;
    } else {
        // Extend area
        if (pos.x < m_minPos.x) m_minPos.x = pos.x;
        if (pos.y < m_minPos.y) m_minPos.y = pos.y;
        if (pos.z < m_minPos.z) m_minPos.z = pos.z;
        if (pos.x > m_maxPos.x) m_maxPos.x = pos.x;
        if (pos.y > m_maxPos.y) m_maxPos.y = pos.y;
        if (pos.z > m_maxPos.z) m_maxPos.z = pos.z;
    }
}

Position Area::center() const
{
    float minLat = m_minPos.getLat();
    float minLon = m_minPos.getLong();
    float maxLat = m_maxPos.getLat();
    float maxLon = m_maxPos.getLong();
    float centerLat = (minLat + maxLat) / 2.0f;
    float centerLon = (minLon + maxLon) / 2.0f;
    return Position::LatLong(centerLat, centerLon, 0.0f);
}

float Area::width() const
{
    // Calculate width
    return m_maxPos.getLong() - m_minPos.getLong();
}

float Area::height() const
{
    // Calculate height
    return m_maxPos.getLat() - m_minPos.getLat();
}


} // namespace gtx
