
#include <gtxlib-core/base/DataStream.h>

#include <cassert>
#include <cstring>
#include <algorithm>
//#include <codecvt>
#include <locale>


namespace gtx
{


DataStream::ByteOrder DataStream::systemByteOrder()
{
    const uint32_t t = 1;
    return (*reinterpret_cast<const uint8_t*>(&t)) ? LittleEndian : BigEndian;
}

bool DataStream::has(size_t count) const
{
    return m_idx + count <= m_data->size();
}

DataStream::DataStream(const std::vector<uint8_t> * data)
: m_data(data)
, m_idx(0)
, m_status(Ok)
, m_byteOrder(BigEndian)
, m_systemOrder(systemByteOrder())
{
}

DataStream::DataStream(std::shared_ptr<Storage> data)
: m_ptr(data)
, m_data(m_ptr.get())
, m_idx(0)
, m_status(Ok)
, m_byteOrder(BigEndian)
, m_systemOrder(systemByteOrder())
{
}

DataStream::DataStream(std::unique_ptr<Storage> && data)
: m_ptr(data.release())
, m_data(m_ptr.get())
, m_idx(0)
, m_status(Ok)
, m_byteOrder(BigEndian)
, m_systemOrder(systemByteOrder())
{
}

DataStream::DataStream(Storage && data)
: m_ptr(new Storage(std::move(data)))
, m_data(m_ptr.get())
, m_idx(0)
, m_status(Ok)
, m_byteOrder(BigEndian)
, m_systemOrder(systemByteOrder())
{
}

bool DataStream::atEnd() const
{
    return m_idx == m_data->size();
}

DataStream & DataStream::operator>>(int8_t & i)
{
    return read(i);
}

DataStream & DataStream::operator>>(int16_t & i)
{
    return read(i);
}

DataStream & DataStream::operator>>(int32_t & i)
{
    return read(i);
}

DataStream & DataStream::operator>>(uint8_t & i)
{
    return read(i);
}

DataStream & DataStream::operator>>(uint16_t & i)
{
    return read(i);
}

DataStream & DataStream::operator>>(uint32_t & i)
{
    return read(i);
}

DataStream & DataStream::operator>>(float & i)
{
    union {
        float val1;
        uint32_t val2;
    } x;
    read(x.val2);
    i = x.val1;
    return *this;
}

DataStream & DataStream::operator>>(double & i)
{
    union {
        double val1;
        uint64_t val2;
    } x;
    read(x.val2);
    i = x.val1;
    return *this;
}

DataStream & DataStream::operator>>(std::string &)
{
    uint32_t len;
    *this >> len;
    if (len == 0)
        return *this;

    if (has(len) && DataStream::Ok == m_status) {
        auto temp = std::u16string(reinterpret_cast<const char16_t *>(&(*m_data)[m_idx]), len / 2);
        if (m_byteOrder != m_systemOrder) {
            for (auto & c : temp)
            {
                c = (c & 0xFF) << 8 | (c >> 8);
            }
        }

        // [TODO]
//      s = std::wstring_convert<std::codecvt_utf8_utf16<char16_t>, char16_t>{}.to_bytes(temp);
    } else {
        m_status = ReadPastEnd;
    }
    m_idx += len;
    return *this;
}

void DataStream::setByteOrder(ByteOrder b)
{
    m_byteOrder = b;
}

DataStream::ByteOrder DataStream::byteOrder() const
{
    return m_byteOrder;
}

size_t DataStream::position() const
{
    return m_idx;
}


} // namespace gtx
