
#include <gtxlib-core/base/Time.h>

#include <cppassist/string/manipulation.h>
#include <cppassist/string/conversion.h>


using namespace cppassist;


namespace gtx
{


using days   = std::chrono::duration<int, std::ratio_multiply<std::ratio<24>, std::chrono::hours::period> >;
using weeks  = std::chrono::duration<int, std::ratio_multiply<std::ratio<7>, days::period> >;
using years  = std::chrono::duration<int, std::ratio_multiply<std::ratio<146097, 400>, days::period> >;
using months = std::chrono::duration<int, std::ratio_divide<years::period, std::ratio<12> > >;


time_point Time::fromTimeString(const std::string & str)
{
    time_point tp;

    // Parse date-time-string
    auto items = string::split(str, ' ');
    if (items.size() == 2) {
        // Get parts
        auto date = items[0];
        auto time = items[1];

        // Parse date
        items = string::split(date, '-');
        if (items.size() == 3) {
            auto y = items[0];
            auto m = items[1];
            auto d = items[2];

            tp += years(string::fromString<int>(y) - 1970)
                + months(string::fromString<int>(m) - 1)
                + days(string::fromString<int>(d) - 1);
        }

        // Parse time
        items = string::split(time, ':');
        if (items.size() == 3) {
            auto hrs = items[0];
            auto min = items[1];
            auto sec = items[2];

            tp += std::chrono::hours(string::fromString<int>(hrs))
                + std::chrono::minutes(string::fromString<int>(min))
                + std::chrono::seconds(string::fromString<int>(sec));
        }
    }

    // Return time
    return tp;
}

time_point Time::fromDateString(const std::string & str)
{
    time_point tp;

    // Parse date
    auto items = string::split(str, '-');
    if (items.size() == 3) {
        auto y = items[0];
        auto m = items[1];
        auto d = items[2];

        tp += years(string::fromString<int>(y) - 1970)
            + months(string::fromString<int>(m) - 1)
            + days(string::fromString<int>(d) - 1);
    }

    // Return time
    return tp;
}

std::string Time::toString(const time_point & time)
{
    time_point tp = time;

    auto y = std::chrono::duration_cast<years>(tp.time_since_epoch());
    tp -= y;

    auto m = std::chrono::duration_cast<months>(tp.time_since_epoch());
    tp -= m;

    auto d = std::chrono::duration_cast<days>(tp.time_since_epoch());
    tp -= d;

    auto hrs = std::chrono::duration_cast<std::chrono::hours>(tp.time_since_epoch());
    tp -= hrs;

    auto min = std::chrono::duration_cast<std::chrono::minutes>(tp.time_since_epoch());
    tp -= min;

    auto sec = std::chrono::duration_cast<std::chrono::seconds>(tp.time_since_epoch());
    tp -= sec;

    auto ms = std::chrono::duration_cast<std::chrono::milliseconds>(tp.time_since_epoch());
    tp -= ms;


    return string::padLeft(string::toString<int>(y.count() + 1970), 4, '0') + "-"
         + string::padLeft(string::toString<int>(m.count() + 1),    2, '0') + "-"
         + string::padLeft(string::toString<int>(d.count() + 1),    2, '0') + " "
         + string::padLeft(string::toString<int>(hrs.count()),      2, '0') + ":"
         + string::padLeft(string::toString<int>(min.count()),      2, '0') + ":"
         + string::padLeft(string::toString<int>(sec.count()),      2, '0') + "."
         + string::padLeft(string::toString<int>(ms.count()),       3, '0') + "000000";
}

unsigned long Time::toInt(const time_point & time)
{
    return std::chrono::duration_cast<std::chrono::seconds>(time.time_since_epoch()).count();
}


} // namespace gtx
