
#include <gtxlib-core/data/AttributeInfo.h>


namespace gtx
{


AttributeInfo::AttributeInfo(const std::string & name)
: m_name(name)
, m_statistics(true)
, m_numValues(0)
, m_min(0.0f)
, m_max(0.0f)
, m_sum(0.0f)
, m_mean(0.0f)
, m_stdDev(0.0f)
{
}

AttributeInfo::~AttributeInfo()
{
}

const std::string & AttributeInfo::name() const
{
    return m_name;
}

bool AttributeInfo::autoCalculateStatistics() const
{
    return m_statistics;
}

void AttributeInfo::setAutoCalculateStatistics(bool statistics)
{
    m_statistics = statistics;
}

float AttributeInfo::minimumValue() const
{
    return m_min;
}

void AttributeInfo::setMinimumValue(float value)
{
    m_min = value;
}

float AttributeInfo::maximumValue() const
{
    return m_max;
}

void AttributeInfo::setMaximumValue(float value)
{
    m_max = value;
}

float AttributeInfo::sum() const
{
    return m_sum;
}

void AttributeInfo::setSum(float value)
{
    m_sum = value;
    calculateMean();
}

unsigned long AttributeInfo::numValues() const
{
    return m_numValues;
}

void AttributeInfo::setNumValues(unsigned long numValues)
{
    m_numValues = numValues;
    calculateMean();
}

float AttributeInfo::meanValue() const
{
    return m_mean;
}

void AttributeInfo::setMeanValue(float value)
{
    m_mean = value;
}

float AttributeInfo::standardDeviation() const
{
    return m_stdDev;
}

void AttributeInfo::setStandardDeviation(float value)
{
    m_stdDev = value;
}

void AttributeInfo::addValue(float value)
{
    if (m_numValues < 1)
    {
        m_sum = m_min = m_max = value;
        m_numValues = 1;
    }

    else
    {
        if (value < m_min) m_min = value;
        if (value > m_max) m_max = value;
        m_sum += value;
        m_numValues++;
    }
}

void AttributeInfo::calculateMean()
{
    if (m_numValues > 0) {
        m_mean = m_sum / m_numValues;
    } else {
        m_mean = 0.0f;
    }
}


} // namespace gtx
