
#include <gtxlib-core/data/Item.h>

#include <cppassist/memory/make_unique.h>


namespace gtx
{


Item::Item(id_t id)
: m_id(id)
{
}

Item::~Item()
{
}

id_t Item::id() const
{
    return m_id;
}

void Item::setId(id_t id)
{
    m_id = id;
}

const std::string & Item::name() const
{
    return m_name;
}

void Item::setName(const std::string & name)
{
    m_name = name;
}

const std::vector<std::string> & Item::attributes() const
{
    return m_names;
}

bool Item::hasAttribute(const std::string & name) const
{
    auto it = m_attributes.find(name);
    return (it != m_attributes.end());
}

const Attribute * Item::attribute(const std::string & name) const
{
    auto it = m_attributes.find(name);

    if (it != m_attributes.end())
    {
        return it->second.get();
    }

    return nullptr;
}

Attribute * Item::attribute(const std::string & name)
{
    Attribute * attribute = nullptr;

    auto it = m_attributes.find(name);
    if (it != m_attributes.end())
    {
        attribute = it->second.get();
    }

    if (!attribute)
    {
        auto ptr = cppassist::make_unique<Attribute>(name);
        attribute = ptr.get();

        m_attributes[name] = std::move(ptr);
        m_names.push_back(name);
    }

    return attribute;
}


} // namespace gtx
