
#include <gtxlib-core/loader/LoaderDemo.h>

#include <cppassist/string/conversion.h>
#include <cppassist/string/manipulation.h>

#include <gtxlib-core/base/Time.h>

#include <gtxlib-core/data/DataSet.h>
#include <gtxlib-core/data/Node.h>
#include <gtxlib-core/data/Line.h>


using namespace cppassist;


namespace gtx
{


CPPEXPOSE_COMPONENT(LoaderDemo, gloperate::AbstractLoader)


LoaderDemo::LoaderDemo(gloperate::Environment * environment)
: gloperate::Loader<DataSet>(environment)
{
}

LoaderDemo::~LoaderDemo()
{
}

bool LoaderDemo::canLoad(const std::string & ext) const
{
    return (string::toLower(ext) == "dat" || string::toLower(ext) == "demo");
}

std::vector<std::string> LoaderDemo::loadingTypes() const
{
    std::vector<std::string> types;
    types.push_back("Data files (*.dat *.demo)");
    return types;
}

std::string LoaderDemo::allLoadingTypes() const
{
    // Return supported types
    return "*.dat *.demo";
}

DataSet * LoaderDemo::load(const std::string & filename, const cppexpose::Variant &, std::function<void(int, int)> progress) const
{
    if (string::hasSuffix(filename, ".dat")) {
        return loadDemoNetwork(progress);
    } else if (string::hasSuffix(filename, ".demo")) {
        return loadDemoLines(progress);
    } else {
        if (progress) progress(-1, 0);
        return nullptr;
    }
}

DataSet * LoaderDemo::loadDemoNetwork(std::function<void(int, int)> progress) const
{
    // Create data set
    DataSet * dataset = new DataSet();

    // Create nodes and edges
    auto & nodes = dataset->nodes();
    auto & edges = dataset->edges();

    auto * node1 = nodes.create();
    node1->setPosition(Position(0.0f, -30.0f, 0.0f));
    node1->attribute("At1")->setValue(0.0f);

    auto * node2 = nodes.create();
    node2->setPosition(Position(0.0f, 30.0f, 0.0f));
    node2->attribute("At1")->setValue(1.0f);

    auto * edge = edges.create();
    edge->setNode1(node1);
    edge->setNode2(node2);
    edge->attribute("At2")->setValue(0.5f);

    // Finalize dataset
    dataset->fixIds();
    dataset->calculateEdgeLength();
    dataset->addStandardAttributes();

    // Done
    if (progress) progress(-1, 0);
    return dataset;
}

DataSet * LoaderDemo::loadDemoLines(std::function<void(int, int)> progress) const
{
    // Create data set
    DataSet * dataset = new DataSet();

    // Create nodes and lines
    auto & nodes = dataset->nodes();
    auto & lines = dataset->lines();

    for (size_t a=0; a<3; a++) {
        for (size_t b=0; b<3; b++) {

    size_t t = 0;
    for (size_t z=0; z<50; z++) {

        // Create line
        auto * line = lines.create();

        for (size_t y=0; y<50; y++) {
            bool reverse = (y%2 == 0);

            // Create nodes
            for (size_t x=0; x<10; x++) {
                // Calculate position
                auto _time = Time::fromTimeString("2010-10-01 00:13:00") + std::chrono::seconds(t * 100);
                float lat = 50.9089f + a * 1.0f + y * 0.01f;
                float lon = 7.08747f + b * 1.0f + (reverse ? x * 0.08f : (9-x) * 0.08f);
                float h   = z * 250.0f;
                t++;

                // Create node
                auto * node = nodes.create();
                node->setPosition(Position(lat, lon, h));
                node->setTime(_time);
                node->attribute("time")->setValue((float)node->timeInt());
                node->attribute("height")->setValue(h);
                node->attribute("velocity")->setValue(y / 100.0f + (z / 100.0f));

                // Add node to line
                line->add(node);
            }
        }
    }

        }
    }

    // Finalize dataset
    dataset->analyzeTrajectories();
    dataset->fixIds();
    dataset->addStandardAttributes();

    // Done
    if (progress) progress(-1, 0);
    return dataset;
}


} // namespace gtx
