
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <memory>
#include <string>
#include <vector>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace cppexpose
{
    class Variant;
}

namespace globjects
{
    class Texture;
}


namespace gtx
{


class AttributeStorage;


/**
*  @brief
*    Stage that loads the selected color maps
*/
class GTXLIB_STAGES_API ColorMapStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        ColorMapStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that loads the selected color maps"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<std::string> nodeColorMap; ///< Name of node color map
    Input<std::string> edgeColorMap; ///< Name of edge color map
    Input<std::string> lineColorMap; ///< Name of line color map

    // Outputs
    Output<std::vector<std::string>> colorMapNames;       ///< List of color maps
    Output<globjects::Texture *>     colorMapsTexture;    ///< Texture array containing all color maps
    Output<globjects::Texture *>     nodeColorMapTexture; ///< Texture of node color map
    Output<globjects::Texture *>     edgeColorMapTexture; ///< Texture of edge color map
    Output<globjects::Texture *>     lineColorMapTexture; ///< Texture of line color map


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    ColorMapStage(gloperate::Environment * environment, const std::string & name = "ColorMapStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~ColorMapStage();

    /**
    *  @brief
    *    Get available color maps
    *
    *  @return
    *    List of color maps
    */
    const cppexpose::Variant & colormaps() const;


protected:
    // Virtual Stage interface
    virtual void onContextInit(gloperate::AbstractGLContext * context) override;
    virtual void onContextDeinit(gloperate::AbstractGLContext * context) override;
    virtual void onProcess() override;


protected:
    void createTexture();


protected:
    std::vector<std::string>            m_filenames;   ///< List of filenames
    std::unique_ptr<globjects::Texture> m_colorMaps;   ///< 2D texture array containing all color maps
    std::unique_ptr<globjects::Texture> m_nodeTexture; ///< Texture of node color map
    std::unique_ptr<globjects::Texture> m_edgeTexture; ///< Texture of edge color map
    std::unique_ptr<globjects::Texture> m_lineTexture; ///< Texture of line color map
};


} // namespace gtx
