
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class AbstractStringSource;
}


namespace gtx
{


class DataSet;


/**
*  @brief
*    Stage that loads a GTX data set from file
*/
class GTXLIB_STAGES_API DataLoaderStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        DataLoaderStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that loads a GTX data set from file"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<cppfs::FilePath> filePath; ///< Path to the data file

    // Outputs
    Output<gtx::DataSet *> dataset; ///< Dataset object


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    DataLoaderStage(gloperate::Environment * environment, const std::string & name = "DataLoaderStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~DataLoaderStage();


protected:
    // Virtual Stage interface
    virtual void onProcess() override;


protected:
    std::unique_ptr<gtx::DataSet> m_dataset; ///< Geographic data set
};


} // namespace gtx
