
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <string>

#include <gloperate/pipeline/Pipeline.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/stages/interfaces/CanvasInterface.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace gloperate
{
    class TimerStage;
    class BasicFramebufferStage;
    class RasterizationStage;
    class ClearStage;
    class ColorRenderTarget;
    class DepthRenderTarget;
    class BlitStage;
}


namespace gtx
{


class TrackballStage;
class WorldStage;
class TrajectoryStage;
class DataLoaderStage;
class GpuDataStage;
class MappingStage;
class TemporalFilterStage;
class ColorMapStage;
class TextureArrayStage;


/**
*  @brief
*    GTX pipeline for demo visualization
*/
class GTXLIB_STAGES_API DemoPipeline : public gloperate::Pipeline
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        DemoPipeline, gloperate::Pipeline
      , "RenderStage Gtx" // Tags
      , ""                // Icon
      , ""                // Annotations
      , "Main GTX pipeline"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Interfaces
    gloperate::CanvasInterface canvasInterface;            ///< Interface for rendering into a viewer
    Input<gloperate::ColorRenderTarget *>  colorTarget;    ///< Color render target
    Input<gloperate::DepthRenderTarget *>  depthTarget;    ///< Depth render target
    Output<gloperate::ColorRenderTarget *> colorTargetOut; ///< Color render target (passthrough)


public:
    static void initialize();


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    DemoPipeline(gloperate::Environment * environment, const std::string & name = "");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~DemoPipeline();


protected:
    // Stages
    std::unique_ptr<ColorMapStage>                    m_colorMap;       ///< Load the color maps
    std::unique_ptr<TextureArrayStage>                m_textures;       ///< Load the textures
    std::unique_ptr<TrackballStage>                   m_trackball;      ///< Camera navigation
    std::unique_ptr<gloperate::TimerStage>            m_timer;          ///< Maintains a virtual time

    std::unique_ptr<DataLoaderStage>                  m_dataLoader;     ///< Loads the data set
    std::unique_ptr<GpuDataStage>                     m_gpuData;        ///< Creates GPU representations of the data
    std::unique_ptr<MappingStage>                     m_mapping;        ///< Stage that selects an attribute and determines its index
    std::unique_ptr<TemporalFilterStage>              m_temporalFilter; ///< Stage that enables temporal filtering

    std::unique_ptr<WorldStage>                       m_world;          ///< Creates the world map
    std::unique_ptr<TrajectoryStage>                  m_trajectories;   ///< Renders trajectories

    std::unique_ptr<gloperate::BasicFramebufferStage> m_framebuffer;    ///< Framebuffer to render into
    std::unique_ptr<gloperate::ClearStage>            m_clear;          ///< Clears the framebuffer
    std::unique_ptr<gloperate::RasterizationStage>    m_rasterization;  ///< Executes rendering

    std::unique_ptr<gloperate::BlitStage>             m_blit;           ///< Blits the final image
};


} // namespace gtx
