
#pragma once


#include <glm/glm.hpp>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Pipeline.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>
#include <gloperate/base/ExtendedProperties.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
}

namespace gloperate
{
    class AbstractDrawable;
    class ProgramStage;
    class RenderPassStage;
    class TextureRenderTargetStage;
    class ClearStage;
    class RasterizationStage;
    class TextureFromRenderTargetExtractionStage;
}


namespace gtx
{


/**
*  @brief
*    Stage that creates a density map
*/
class GTXLIB_STAGES_API DensityMapStage : public gloperate::Pipeline
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        DensityMapStage, gloperate::Pipeline
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that creates a density map"
      , GTX_AUTHOR_ORGANIZATION
      , "v0.1.0"
    )


public:
    // Inputs
    Input<glm::vec4> viewport;                         ///< Viewport
    Input<gloperate::AbstractDrawable *> lineGeometry; ///< Geometry containing the lines of the data set
    Input<glm::vec4> area;                             ///< Area of the overlay
    Input<glm::vec2> timeSpan;                         ///< Entire time span
    Input<glm::vec2> timeSpan1;                        ///< Temporal filter span #1
    Input<glm::vec2> timeSpan2;                        ///< Temporal filter span #2
    Input<glm::vec2> timeSpan3;                        ///< Temporal filter span #3
    Input<glm::vec2> timeSpan4;                        ///< Temporal filter span #4

    // Outputs
    Output<globjects::Texture *> texture; ///< Texture containing the density map


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    DensityMapStage(gloperate::Environment * environment, const std::string & name = "DensityMapStage");

    /**
    * @brief
    *   Destructor
    */
    ~DensityMapStage();


protected:
    // GPU-based visualization pipeline
    std::unique_ptr<gloperate::ProgramStage>                           m_program;      ///< Builds the program for rendering
    std::unique_ptr<gloperate::RenderPassStage>                        m_renderPass;   ///< Builds the render pass for rendering
    std::unique_ptr<gloperate::TextureRenderTargetStage>               m_renderTarget; ///< Create the render target for the density map
    std::unique_ptr<gloperate::ClearStage>                             m_clear;        ///< Clears the density map
    std::unique_ptr<gloperate::RasterizationStage>                     m_rasterize;    ///< Rasterizes the density map
    std::unique_ptr<gloperate::TextureFromRenderTargetExtractionStage> m_texture;      ///< Extracts the density map texture
};


} // namespace gtx
