
#pragma once


#include <memory>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
}


namespace gtx
{


class DataSet;


/**
*  @brief
*    Stage that controls the filtering of items
*/
class GTXLIB_STAGES_API FilterStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        FilterStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that controls the filtering of items"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<gtx::DataSet *> dataset; ///< Data set

    // Outputs
    Output<globjects::Texture *> filtersTexture; ///< Texture that contains the filter information for each attribute


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    FilterStage(gloperate::Environment * environment, const std::string & name = "FilterStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~FilterStage();


protected:
    // Virtual Stage interface
    virtual void onContextDeinit(gloperate::AbstractGLContext * context) override;
    virtual void onInputValueChanged(gloperate::AbstractSlot * slot) override;
    virtual void onInputValueInvalidated(gloperate::AbstractSlot * slot) override;
    virtual void onProcess() override;


protected:
    void updateFilters();
    void updateTexture();


protected:
    std::unique_ptr<globjects::Texture> m_filtersTexture; ///< Texture containing the filter information
    bool                                m_updateFilters;  ///< 'true' if the filters need do be updated, else 'false'
};


} // namespace gtx
