
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <memory>
#include <vector>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-core/base/Configuration.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
}


namespace gtx
{


/**
*  @brief
*    Stage that loads a number of mapping configurations onto the GPU
*/
class GTXLIB_STAGES_API GpuConfigurationStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        GpuConfigurationStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that loads a number of mapping configurations onto the GPU"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Outputs
    Output<std::vector<Configuration *>> configurations; ///< List of configurations
    Output<globjects::Texture *>         texture;        ///< Output texture containing the configurations


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    GpuConfigurationStage(gloperate::Environment * environment, const std::string & name = "GpuConfigurationStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~GpuConfigurationStage();


protected:
    // Virtual Stage interface
    virtual void onProcess() override;


protected:
    void createTexture();


protected:
    std::unique_ptr<globjects::Texture> m_texture; ///< Output texture containing the configurations
};


} // namespace gtx
