
#pragma once


#include <glm/glm.hpp>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Pipeline.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
    class Shader;
}

namespace gloperate
{
    class AbstractDrawable;
    class Camera;
    class ProgramStage;
    class RenderPassStage;
}


namespace gtx
{


/**
*  @brief
*    Stage that renders the lines of a data set
*/
class GTXLIB_STAGES_API LineStage : public gloperate::Pipeline
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        LineStage, gloperate::Pipeline
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that renders the lines of a data set"
      , GTX_AUTHOR_ORGANIZATION
      , "v0.1.0"
    )


public:
    // Inputs
    Input<glm::vec4> viewport;                           ///< Viewport
    Input<gloperate::Camera *> camera;                   ///< Camera configuration
    Input<globjects::Shader *> projectionVertexShader;   ///< Vertex shader for transforming geographic coordinates into world coordinates
    Input<globjects::Shader *> projectionGeometryShader; ///< Geometry shader for transforming geographic coordinates into world coordinates
    Input<gloperate::AbstractDrawable *> lineGeometry;   ///< Geometry containing the lines of the data set
    Input<globjects::Texture *> lineColorMapTexture;     ///< Color map for the lines
    Input<globjects::Texture *> staticAttributesTexture; ///< Texture buffer containing the static attributes
    Input<globjects::Texture *> nodeAttributesTexture;   ///< Texture buffer containing the node attributes
    Input<globjects::Texture *> lineAttributesTexture;   ///< Texture buffer containing the line attributes
    Input<globjects::Texture *> filtersTexture;          ///< Texture buffer containing the current filter values
    Input<int> numStaticAttributes;                      ///< Number of static attributes
    Input<int> numNodeAttributes;                        ///< Number of node attributes
    Input<int> numEdgeAttributes;                        ///< Number of edge attributes
    Input<int> numLineAttributes;                        ///< Number of line attributes
    Input<int> lineColorIndex;                           ///< Index of attribute which is mapped to line color
    Input<int> lineWidthIndex;                           ///< Index of attribute which is mapped to line width
    Input<int> nodeHeightIndex;                          ///< Index of attribute which is mapped to node height
    Input<float> widthScale;                             ///< Line width scale factor

    // Outputs
    Output<gloperate::AbstractDrawable *> renderPass; ///< Render pass that renders the lines


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    LineStage(gloperate::Environment * environment, const std::string & name = "LineStage");

    /**
    * @brief
    *   Destructor
    */
    ~LineStage();


protected:
    std::unique_ptr<gloperate::ProgramStage>    m_program;    ///< Builds the program for line rendering
    std::unique_ptr<gloperate::RenderPassStage> m_renderPass; ///< Builds the render pass for line rendering
};


} // namespace gtx
