
#pragma once


#include <glm/glm.hpp>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Pipeline.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>
#include <gloperate/base/ExtendedProperties.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
    class Shader;
}

namespace gloperate
{
    class Camera;
    class AbstractDrawable;
    class ShapeStage;
    class ProgramStage;
    class RenderPassStage;
}


namespace gtx
{


/**
*  @brief
*    Stage that puts an overlay over the map
*/
class GTXLIB_STAGES_API MapOverlayStage : public gloperate::Pipeline
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        MapOverlayStage, gloperate::Pipeline
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that puts an overlay over the map"
      , GTX_AUTHOR_ORGANIZATION
      , "v0.1.0"
    )


public:
    // Inputs
    Input<glm::vec4>            viewport;               ///< Viewport
    Input<gloperate::Camera *>  camera;                 ///< Camera configuration
    Input<globjects::Texture *> texture;                ///< Texture thas is displayed on top of the map
    Input<glm::vec4>            area;                   ///< Area of the overlay
    Input<globjects::Shader *>  projectionVertexShader; ///< Vertex shader for transforming geographic coordinates into world coordinates

    // Outputs
    Output<gloperate::AbstractDrawable *> renderPass; ///< Render pass that renders the overlay


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    MapOverlayStage(gloperate::Environment * environment, const std::string & name = "MapOverlayStage");

    /**
    * @brief
    *   Destructor
    */
    ~MapOverlayStage();


protected:
    // GPU-based visualization pipeline
    std::unique_ptr<gloperate::ShapeStage>      m_shape;        ///< Creates a quad geometry
    std::unique_ptr<gloperate::ProgramStage>    m_program;      ///< Builds the program for rendering
    std::unique_ptr<gloperate::RenderPassStage> m_renderPass;   ///< Builds the render pass for rendering
};


} // namespace gtx
