
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <vector>
#include <string>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace gtx
{


class AttributeStorage;


/**
*  @brief
*    Stage that calculates the indices of the selected attributes
*/
class GTXLIB_STAGES_API MappingStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        MappingStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that calculates the indices of the selected attributes"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<std::vector<std::string>> attributes;            ///< List of attributes
    Input<std::string>              nodeColor;             ///< Name of node color attribute
    Input<std::string>              nodeSize;              ///< Name of node size attribute
    Input<std::string>              nodeHeight;            ///< Name of node height attribute
    Input<std::string>              edgeColor;             ///< Name of edge color attribute
    Input<std::string>              edgeWidth;             ///< Name of edge width attribute
    Input<std::string>              lineColor;             ///< Name of line color attribute
    Input<std::string>              lineWidth;             ///< Name of line width attribute
    Input<std::string>              primaryAttribute;      ///< Name of primary attribute

    // Outputs
    Output<int>                     nodeColorIndex;        ///< Index of node color attribute
    Output<int>                     nodeSizeIndex;         ///< Index of node size attribute
    Output<int>                     nodeHeightIndex;       ///< Index of node height attribute
    Output<int>                     edgeColorIndex;        ///< Index of edge color attribute
    Output<int>                     edgeWidthIndex;        ///< Index of edge width attribute
    Output<int>                     lineColorIndex;        ///< Index of line color attribute
    Output<int>                     lineWidthIndex;        ///< Index of line width attribute
    Output<int>                     primaryAttributeIndex; ///< Index of primary attribute


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    MappingStage(gloperate::Environment * environment, const std::string & name = "MappingStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~MappingStage();


protected:
    int getAttributeIndex(const std::string & attribute) const;


protected:
    // Virtual Stage interface
    virtual void onProcess() override;
};


} // namespace gtx
