
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace gloperate
{
    class ColorRenderTarget;
}


namespace gtx
{


/**
*  @brief
*    Stage that prints out performance measurements
*/
class GTXLIB_STAGES_API MeasurementStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        MeasurementStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that prints out performance measurements"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<gloperate::ColorRenderTarget *> colorTarget; ///< Color target (pass-through)
    Input<bool> measureRendering;                      ///< 'true' to measure rendering time, else 'false'
    Input<int> drawMode;                               ///< 1: cpu-triangles-indexed, 2: cpu-strips-multiple-calls, 3: cpu-strips-primitive-restart, 5: gpu
    Input<unsigned int> maxNodes;                      ///< Maximum number of nodes to render
    Input<unsigned int> size;                          ///< Size of geometry (in bytes)
    Input<uint64_t> renderTime;                        ///< Time it took to render the geometry (in nanoseconds)

    // Outputs
    Output<gloperate::ColorRenderTarget *> colorTargetOut; ///< Color target (output)
    Output<unsigned int> frame;                            ///< Frame counter for measurements


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    MeasurementStage(gloperate::Environment * environment, const std::string & name = "MeasurementStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~MeasurementStage();


protected:
    // Virtual Stage interface
    virtual void onInputValueChanged(gloperate::AbstractSlot * slot) override;
    virtual void onProcess() override;


protected:
    unsigned int m_frame; ///< Frame counter
    uint64_t     m_sum;   ///< Sum of all measured times
};


} // namespace gtx
