
#pragma once


#include <memory>

#include <glm/glm.hpp>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Pipeline.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
}

namespace gloperate
{
    class ProgramStage;
    class ShapeStage;
    class RenderPassStage;
    class AbstractDrawable;
}


namespace gtx
{


/**
*  @brief
*    Stage that outputs the final image to the screen
*/
class GTXLIB_STAGES_API OutputStage : public gloperate::Pipeline
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        OutputStage, gloperate::Pipeline
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that outputs the final image to the screen"
      , GTX_AUTHOR_ORGANIZATION
      , "v0.1.0"
    )


public:
    // Inputs
    Input<glm::vec4>            viewport;            ///< Viewport
    Input<globjects::Texture *> mapColorTexture;     ///< Texture containing the color buffer
    Input<globjects::Texture *> mapDepthTexture;     ///< Texture containing the depth buffer
    Input<globjects::Texture *> overlayColorTexture; ///< Texture containing the color buffer
    Input<globjects::Texture *> overlayDepthTexture; ///< Texture containing the depth buffer
    Input<bool>                 unsharpMasking;      ///< Enable unsharp masking?
    Input<float>                unsharpFactor;       ///< Factor by which the effect is intensified

    // Outputs
    Output<gloperate::AbstractDrawable *> renderPass; ///< Render pass that renders the nodes


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    OutputStage(gloperate::Environment * environment, const std::string & name = "OutputStage");

    /**
    * @brief
    *   Destructor
    */
    ~OutputStage();


protected:
    std::unique_ptr<gloperate::ShapeStage>      m_shape;      ///< Creates a quad geometry
    std::unique_ptr<gloperate::ProgramStage>    m_program;    ///< Builds the program for node rendering
    std::unique_ptr<gloperate::RenderPassStage> m_renderPass; ///< Builds the render pass for node rendering
};


} // namespace gtx
