
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace gloperate
{
    class AbstractDrawable;
}


namespace gtx
{


class DataSet;
class PlotGeometry;


/**
*  @brief
*    Stage that creates the GPU representations for a line plot
*/
class GTXLIB_STAGES_API PlotGeometryStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        PlotGeometryStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that creates the GPU representations for a line plot"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<gtx::DataSet *> dataset;    ///< Dataset object
    Input<int>            selectedId; ///< Currently selected node

    // Outputs
    Output<gloperate::AbstractDrawable *> plotGeometry; ///< Plot geometry drawable (as line strips)


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    PlotGeometryStage(gloperate::Environment * environment, const std::string & name = "PlotGeometryStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~PlotGeometryStage();


protected:
    // Virtual Stage interface
    virtual void onContextDeinit(gloperate::AbstractGLContext * context) override;
    virtual void onProcess() override;


protected:
    std::unique_ptr<PlotGeometry> m_plotGeometry; ///< Plot geometry
};


} // namespace gtx
