
#pragma once


#include <glm/glm.hpp>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Pipeline.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>
#include <gloperate/base/ExtendedProperties.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace gloperate
{
    class AbstractDrawable;
    class ProgramStage;
    class RenderPassStage;
    class ShapeStage;
}


namespace gtx
{


class DataSet;
class PlotGeometryStage;


/**
*  @brief
*    Stage that plots item values and displays them as a line plot
*/
class GTXLIB_STAGES_API PlotStage : public gloperate::Pipeline
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        PlotStage, gloperate::Pipeline
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that plots item values and displays them as a line plot"
      , GTX_AUTHOR_ORGANIZATION
      , "v0.1.0"
    )


public:
    // Inputs
    Input<gtx::DataSet *>   dataset;         ///< Dataset object
    Input<int>              selectedId;      ///< Currently selected node
    Input<glm::vec4>        viewport;        ///< Viewport
    Input<gloperate::Color> color;           ///< Plot color
    Input<gloperate::Color> backgroundColor; ///< Background color

    // Outputs
    Output<gloperate::AbstractDrawable *> bgRenderPass; ///< Render pass that renders the background
    Output<gloperate::AbstractDrawable *> renderPass;   ///< Render pass that renders the plot


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    PlotStage(gloperate::Environment * environment, const std::string & name = "PlotStage");

    /**
    * @brief
    *   Destructor
    */
    ~PlotStage();


protected:
    std::unique_ptr<gloperate::ShapeStage>      m_bgGeometry;   ///< Builds the geometry for the background
    std::unique_ptr<gloperate::ProgramStage>    m_bgProgram;    ///< Builds the program for the background
    std::unique_ptr<gloperate::RenderPassStage> m_bgRenderPass; ///< Builds the render pass for the background
    std::unique_ptr<PlotGeometryStage>          m_geometry;     ///< Builds the geometry for the line plot
    std::unique_ptr<gloperate::ProgramStage>    m_program;      ///< Builds the program for plot rendering
    std::unique_ptr<gloperate::RenderPassStage> m_renderPass;   ///< Builds the render pass for plot rendering
};


} // namespace gtx
