
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <string>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace gloperate
{
    class ColorRenderTarget;
}


namespace gtx
{


/**
*  @brief
*    Stage that takes a screenshot
*/
class GTXLIB_STAGES_API ScreenshotStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        ScreenshotStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that takes a screenshot"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<gloperate::ColorRenderTarget *> colorTarget; ///< Render target to capture
    Input<std::string>                    path;         ///< File path

    // Outputs
    Output<gloperate::ColorRenderTarget *> colorTargetOut; ///< Render target (pass-through)


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    ScreenshotStage(gloperate::Environment * environment, const std::string & name = "ScreenshotStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~ScreenshotStage();


protected:
    // Virtual Stage interface
    virtual void onProcess() override;
};


} // namespace gtx
