
#pragma once


#include <memory>

#include <glm/glm.hpp>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>
#include <gloperate/base/ExtendedProperties.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
}


namespace gtx
{


class DataSet;


/**
*  @brief
*    Stage that controls temporal filtering
*/
class GTXLIB_STAGES_API TemporalFilterStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        TemporalFilterStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that controls temporal filtering"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<gtx::DataSet *> dataset;  ///< Data set
    Input<float>          minTime1;
    Input<float>          maxTime1;
    Input<float>          minTime2;
    Input<float>          maxTime2;
    Input<float>          minTime3;
    Input<float>          maxTime3;
    Input<float>          minTime4;
    Input<float>          maxTime4;

    // Outputs
    Output<float>     minTime;   ///< Minimum time in data set
    Output<float>     maxTime;   ///< Maximum time in data set
    Output<glm::vec2> timeSpan;  ///< Entire time span
    Output<glm::vec2> timeSpan1; ///< Temporal filter span #1
    Output<glm::vec2> timeSpan2; ///< Temporal filter span #2
    Output<glm::vec2> timeSpan3; ///< Temporal filter span #3
    Output<glm::vec2> timeSpan4; ///< Temporal filter span #4


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    TemporalFilterStage(gloperate::Environment * environment, const std::string & name = "TemporalFilterStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~TemporalFilterStage();


protected:
    float m_minTime;
    float m_maxTime;


protected:
    // Virtual Stage interface
    virtual void onProcess() override;
};


} // namespace gtx
