
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <string>
#include <vector>
#include <memory>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
}


namespace gtx
{


/**
*  @brief
*    Stage that loads a list of textures and puts them into a 2D texture array
*/
class GTXLIB_STAGES_API TextureArrayStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        TextureArrayStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that loads a list of textures and puts them into a 2D texture array"
      , GTX_AUTHOR_ORGANIZATION
      , "v1.0.0"
    )


public:
    // Inputs
    Input<std::string>  path;   ///< Directory path
    Input<unsigned int> width;  ///< Texture width
    Input<unsigned int> height; ///< Texture height

    // Outputs
    Output<std::vector<std::string>> filenames; ///< File names
    Output<globjects::Texture *>     texture;   ///< Output texture


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    TextureArrayStage(gloperate::Environment * environment, const std::string & name = "TextureArrayStage");

    /**
    *  @brief
    *    Destructor
    */
    virtual ~TextureArrayStage();


protected:
    // Virtual Stage interface
    virtual void onProcess() override;


protected:
    void createTexture();


protected:
    std::unique_ptr<globjects::Texture> m_texture; ///< 2D texture array of all images
};


} // namespace gtx
