
#pragma once


#include <memory>
#include <string>
#include <vector>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
}


namespace gtx
{


/**
*  @brief
*    Tool functions
*/
class GTXLIB_STAGES_API Tools
{
public:
    /**
    *  @brief
    *    Create 2D texture array from list of images
    *
    *  @param[in] images
    *    List of image filenames
    *  @param[in] width
    *    Width of a single image (if 0, determine size automatically)
    *  @param[in] height
    *    Height of a single image (if 0, determine size automatically)
    *  @param[in] repeat
    *    If 'true' use GL_REPEAT, else use GL_CLAMP_TO_EDGE
    *
    *  @return
    *    Texture
    */
    static std::unique_ptr<globjects::Texture> createTextureArray(const std::vector<std::string> & images, unsigned int width = 0, unsigned int height = 0, bool repeat = false);
};


} // namespace gtx
