
#pragma once


#include <string>

#include <glm/geometric.hpp>

#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Stage.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class AbstractStringSource;
    class Shader;
    class Texture;
}

namespace gloperate
{
    class AbstractDrawable;
}


namespace gtx
{


/**
*  @brief
*    Stage that creates the geometry for the world or map
*/
class GTXLIB_STAGES_API WorldGeometryStage : public gloperate::Stage
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        WorldGeometryStage, gloperate::Stage
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that creates the geometry for the world or map"
      , GTX_AUTHOR_ORGANIZATION
      , "v0.1.0"
    )


public:
    // Inputs
    Input<std::string> projection; ///< The chosen map projection

    // Outputs
    Output<gloperate::AbstractDrawable*> drawable;
    Output<globjects::Shader*>           vertexShader;
    Output<globjects::Shader*>           fragmentShader;
    Output<globjects::Shader*>           projectionVertexShader;
    Output<globjects::Shader*>           projectionGeometryShader;


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    WorldGeometryStage(gloperate::Environment * environment, const std::string & name = "WorldGeometryStage");

    /**
    * @brief
    *   Destructor
    */
    ~WorldGeometryStage();


protected:
    // Virtual Stage interface
    virtual void onContextInit(gloperate::AbstractGLContext * context) override;
    virtual void onContextDeinit(gloperate::AbstractGLContext * context) override;
    virtual void onProcess() override;


protected:
    std::unique_ptr<gloperate::AbstractDrawable>     m_geometry;                 ///< Map or globe geometry
    std::unique_ptr<globjects::AbstractStringSource> m_vertexShaderSource;       ///< Vertex shader source
    std::unique_ptr<globjects::Shader>               m_vertexShader;             ///< Vertex shader
    std::unique_ptr<globjects::AbstractStringSource> m_fragmentShaderSource;     ///< Fragment shader source
    std::unique_ptr<globjects::Shader>               m_fragmentShader;           ///< Fragment shader
    std::unique_ptr<globjects::AbstractStringSource> m_projectionShaderSource;   ///< Projection shader source
    std::unique_ptr<globjects::Shader>               m_projectionVertexShader;   ///< Projection shader (vertex shader)
    std::unique_ptr<globjects::Shader>               m_projectionGeometryShader; ///< Projection shader (geometry shader)
};


} // namespace gtx
