
#pragma once


#include <cppexpose/plugin/plugin_api.h>

#include <gloperate/pipeline/Pipeline.h>
#include <gloperate/pipeline/Input.h>
#include <gloperate/pipeline/Output.h>
#include <gloperate/base/ExtendedProperties.h>

#include <gtx/gtx-version.h>

#include <gtxlib-stages/gtxlib-stages_api.h>


namespace globjects
{
    class Texture;
    class Shader;
}

namespace gloperate
{
    class AbstractDrawable;
    class TextureLoadStage;
    class ProgramStage;
    class RenderPassStage;
    class Camera;
}


namespace gtx
{


class WorldGeometryStage;


/**
*  @brief
*    Stage that renders a world or map
*/
class GTXLIB_STAGES_API WorldStage : public gloperate::Pipeline
{
public:
    CPPEXPOSE_DECLARE_COMPONENT(
        WorldStage, gloperate::Pipeline
      , ""   // Tags
      , ""   // Icon
      , ""   // Annotations
      , "Stage that renders a world or map"
      , GTX_AUTHOR_ORGANIZATION
      , "v0.1.0"
    )


public:
    // Inputs
    Input<std::string>                    projection;               ///< The chosen map projection
    Input<gloperate::Camera *>            camera;                   ///< Camera configuration
    Input<globjects::Texture *>           mapLayersTexture;         ///< Texture containing additional map layers
    Input<glm::vec2>                      timeSpan;                 ///< Entire time span
    Input<glm::vec2>                      timeSpan1;                ///< Temporal filter span #1
    Input<glm::vec2>                      timeSpan2;                ///< Temporal filter span #2
    Input<glm::vec2>                      timeSpan3;                ///< Temporal filter span #3
    Input<glm::vec2>                      timeSpan4;                ///< Temporal filter span #4

    // Outputs
    Output<gloperate::AbstractDrawable *> renderPass;               ///< Render pass that renders the world
    Output<globjects::Shader *>           projectionVertexShader;   ///< Vertex shader for transforming geographic coordinates into world coordinates
    Output<globjects::Shader *>           projectionGeometryShader; ///< Geometry shader for transforming geographic coordinates into world coordinates


public:
    /**
    *  @brief
    *    Constructor
    *
    *  @param[in] environment
    *    Environment to which the stage belongs (must NOT be null!)
    *  @param[in] name
    *    Stage name
    */
    WorldStage(gloperate::Environment * environment, const std::string & name = "WorldStage");

    /**
    * @brief
    *   Destructor
    */
    ~WorldStage();


protected:
    std::unique_ptr<WorldGeometryStage>          m_geometry;   ///< Generates the geometry for the world map
    std::unique_ptr<gloperate::TextureLoadStage> m_texture;    ///< Loads the texture for the world map
    std::unique_ptr<gloperate::ProgramStage>     m_program;    ///< Builds the program for the world map
    std::unique_ptr<gloperate::RenderPassStage>  m_renderPass; ///< Builds the render pass for the world map
};


} // namespace gtx
