
#include <gtxlib-stages/AirTrafficPipeline.h>

#include <iostream>

#include <cppassist/memory/make_unique.h>
#include <cppassist/string/manipulation.h>

#include <cppexpose/variant/Variant.h>

#include <glbinding/gl/enum.h>

#include <gloperate/base/Environment.h>
#include <gloperate/input/MouseEvent.h>
#include <gloperate/stages/base/BasicFramebufferStage.h>
#include <gloperate/stages/base/TextureRenderTargetStage.h>
#include <gloperate/stages/base/ClearStage.h>
#include <gloperate/stages/base/RasterizationStage.h>
#include <gloperate/stages/base/TimerStage.h>
#include <gloperate/stages/base/TextureFromRenderTargetExtractionStage.h>
#include <gloperate/stages/base/BlitStage.h>

#include <gtxlib-core/gtxlib-core.h>

#include <gtxlib-stages/TrackballStage.h>
#include <gtxlib-stages/WorldStage.h>
#include <gtxlib-stages/LineStage.h>
#include <gtxlib-stages/TrajectoryStage.h>
#include <gtxlib-stages/DensityMapStage.h>
#include <gtxlib-stages/MapOverlayStage.h>
#include <gtxlib-stages/DataLoaderStage.h>
#include <gtxlib-stages/FilterStage.h>
#include <gtxlib-stages/TemporalFilterStage.h>
#include <gtxlib-stages/GpuDataStage.h>
#include <gtxlib-stages/MappingStage.h>
#include <gtxlib-stages/ColorMapStage.h>
#include <gtxlib-stages/TextureArrayStage.h>
#include <gtxlib-stages/OutputStage.h>
#include <gtxlib-stages/MeasurementStage.h>
#include <gtxlib-stages/PickingStage.h>
#include <gtxlib-stages/ScreenshotStage.h>


using namespace cppassist;
using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(AirTrafficPipeline, gloperate::Pipeline)


void AirTrafficPipeline::initialize()
{
}

AirTrafficPipeline::AirTrafficPipeline(Environment * environment, const std::string & name)
: Pipeline(environment, "AirTrafficPipeline", name)
, AbstractEventConsumer(environment->inputManager())
, canvasInterface(this)
, colorTarget("colorTarget", this)
, depthTarget("depthTarget", this)
, colorTargetOut("colorTargetOut", this)
, filename("filename", this)
, projection("projection", this, "Geocentric")
, backgroundColor("background", this, gloperate::Color(0.0f, 0.0f, 0.0f))
, widthScale("widthScale", this, 1.0f)
, linesVisible("linesVisible", this, false)
, nodeHeight("nodeHeight", this)
, lineColorMap("lineColorMap", this, "0005_hsv_saturation_orange")
, lineColor("lineColor", this)
, lineWidth("lineWidth", this)
, trajectoriesVisible("trajectoriesVisible", this, true)
, drawMode("drawMode", this, 5)
, maxNodes("maxNodes", this, 0)
, primaryAttribute("primaryAttribute", this)
, densityMapVisible("densityMapVisible", this, false)
, measureRendering("measureRendering", this, false)
, measureUpdate("measureUpdate", this, false)
, mousePos("mousePos", this)
, m_colorMap(cppassist::make_unique<ColorMapStage>(environment, "ColorMap"))
, m_textures(cppassist::make_unique<TextureArrayStage>(environment, "Textures"))
, m_trackball(cppassist::make_unique<TrackballStage>(environment, "Trackball"))
, m_timer(cppassist::make_unique<TimerStage>(environment, "Timer"))
, m_dataLoader(cppassist::make_unique<DataLoaderStage>(environment, "DataLoader"))
, m_gpuData(cppassist::make_unique<GpuDataStage>(environment, "GpuData"))
, m_mapping(cppassist::make_unique<MappingStage>(environment, "Mapping"))
, m_filter(cppassist::make_unique<FilterStage>(environment, "Filter"))
, m_temporalFilter(cppassist::make_unique<TemporalFilterStage>(environment, "TemporalFilter"))
, m_world(cppassist::make_unique<WorldStage>(environment, "World"))
, m_densityMap(cppassist::make_unique<DensityMapStage>(environment, "DensityMap"))
, m_mapOverlay(cppassist::make_unique<MapOverlayStage>(environment, "MapOverlay"))
, m_lines(cppassist::make_unique<LineStage>(environment, "Lines"))
, m_trajectories(cppassist::make_unique<TrajectoryStage>(environment, "Trajectories"))
, m_worldFramebuffer(cppassist::make_unique<BasicFramebufferStage>(environment, "WorldFramebuffer"))
, m_worldClear(cppassist::make_unique<ClearStage>(environment, "WorldClear"))
, m_worldRasterization(cppassist::make_unique<RasterizationStage>(environment, "WorldRasterization"))
, m_overlayRasterization(cppassist::make_unique<RasterizationStage>(environment, "OverlayRasterization"))
, m_worldOutput(cppassist::make_unique<TextureFromRenderTargetExtractionStage>(environment, "WorldOutput"))
, m_overlayFramebuffer(cppassist::make_unique<BasicFramebufferStage>(environment, "OverlayFramebuffer"))
, m_idTarget(cppassist::make_unique<TextureRenderTargetStage>(environment, "IdTarget"))
, m_overlayClear(cppassist::make_unique<ClearStage>(environment, "OverlayClear"))
, m_lineRasterization(cppassist::make_unique<RasterizationStage>(environment, "LineRasterization"))
, m_trajectoryRasterization(cppassist::make_unique<RasterizationStage>(environment, "TrajectoryRasterization"))
, m_helpersRasterization(cppassist::make_unique<RasterizationStage>(environment, "helpersRasterization"))
, m_overlayOutput(cppassist::make_unique<TextureFromRenderTargetExtractionStage>(environment, "OverlayOutput"))
, m_output(cppassist::make_unique<OutputStage>(environment, "Output"))
, m_outputFramebuffer(cppassist::make_unique<BasicFramebufferStage>(environment, "OutputFramebuffer"))
, m_outputClear(cppassist::make_unique<ClearStage>(environment, "OutputClear"))
, m_outputRasterization(cppassist::make_unique<RasterizationStage>(environment, "OutputRasterization"))
, m_picking(cppassist::make_unique<PickingStage>(environment, "Picking"))
, m_measurement(cppassist::make_unique<MeasurementStage>(environment, "Measurement"))
, m_screenshot(cppassist::make_unique<ScreenshotStage>(environment, "Screenshot"))
, m_blit(cppassist::make_unique<gloperate::BlitStage>(environment, "Blit"))
{
    // [DEBUG]
//  cppassist::setVerbosityLevel(cppassist::LogMessage::Level::Debug + 2);

    // Get data path
    std::string dataPath = gtx::dataPath();

    // Get dataset path
    std::string datasetPath = gtx::datasetPath();

    // Dataset options
    filename.setOption("category", "dataset");

    // General options
    projection.setOption("category", "general");
    projection.setOption("choices", m_world->projection.option("choices"));

    backgroundColor.setOption("category", "general");

    widthScale.setOption("category", "general");
    widthScale.setOption("minimumValue",  1.0f);
    widthScale.setOption("maximumValue", 10.0f);
    widthScale.setOption("updateOnDrag", true);

    // Lines options
    linesVisible.setOption("category", "lines");

    nodeHeight.setOption("category", "lines");
    nodeHeight.setOption("choices", Variant::array());

    lineColor.setOption("category", "lines");
    lineColor.setOption("choices", Variant::array());

    lineWidth.setOption("category", "lines");
    lineWidth.setOption("choices", Variant::array());

    lineColorMap.setOption("category", "lines");
    lineColorMap.setOption("choices", m_colorMap->colormaps());

    // Trajectory options
    trajectoriesVisible.setOption("category", "trajectories");

    primaryAttribute.setOption("category", "trajectories");
    primaryAttribute.setOption("choices", Variant::array());

    densityMapVisible.setOption("category", "trajectories");

    // Color map stage
    addStage(m_colorMap.get());
    m_colorMap->lineColorMap << this->lineColorMap;

    // Textures stage
    addStage(m_textures.get());
    m_textures->path = dataPath + "/textures";
    *m_textures->createInput<std::string>("tex0") = "tube_texture_none.png";
    *m_textures->createInput<std::string>("tex1") = "tube_texture_arrow.png";
    *m_textures->createInput<std::string>("tex2") = "tube_texture_grid.png";
    *m_textures->createInput<std::string>("tex3") = "tube_texture_daytime.png";

    // Trackball stage
    addStage(m_trackball.get());
    m_trackball->viewport << canvasInterface.viewport;

    // Timer stage
    addStage(m_timer.get());
    m_timer->timeDelta << canvasInterface.timeDelta;

    // Data loader stage
    addStage(m_dataLoader.get());
    m_dataLoader->filePath << this->filename;

    // Gpu data representation stage
    addStage(m_gpuData.get());
    m_gpuData->dataset << m_dataLoader->dataset;
    m_gpuData->maxNodes << maxNodes;
    m_gpuData->attributeNames.valueChanged.connect(this, &AirTrafficPipeline::updateMappingValues);

    // Create update trigger for UI when a new dataset has been loaded
    auto * updateTrigger = this->createInput("updateTrigger") << m_gpuData->attributeNames;
    updateTrigger->setOption("hidden", true);

    // Mapping stage
    addStage(m_mapping.get());
    m_mapping->attributes       << m_gpuData->attributeNames;
    m_mapping->nodeHeight       << this->nodeHeight;
    m_mapping->lineColor        << this->lineColor;
    m_mapping->lineWidth        << this->lineWidth;
    m_mapping->primaryAttribute << this->primaryAttribute;

    // Filter stage
    addStage(m_filter.get());
    m_filter->dataset << m_dataLoader->dataset;

    // Temporal filter stage
    addStage(m_temporalFilter.get());
    m_temporalFilter->dataset << m_dataLoader->dataset;

    // World map stage
    addStage(m_world.get());
    m_world->projection << this->projection;
    m_world->camera << m_trackball->camera;
    m_world->mapLayersTexture << m_gpuData->mapLayersTexture;
    m_world->timeSpan << m_temporalFilter->timeSpan;
    m_world->timeSpan1 << m_temporalFilter->timeSpan1;
    m_world->timeSpan2 << m_temporalFilter->timeSpan2;
    m_world->timeSpan3 << m_temporalFilter->timeSpan3;
    m_world->timeSpan4 << m_temporalFilter->timeSpan4;

    // Density map stage
    addStage(m_densityMap.get());
    m_densityMap->lineGeometry << m_gpuData->lineGeometry;
    m_densityMap->area << m_gpuData->area;
    m_densityMap->timeSpan << m_temporalFilter->timeSpan;
    m_densityMap->timeSpan1 << m_temporalFilter->timeSpan1;
    m_densityMap->timeSpan2 << m_temporalFilter->timeSpan2;
    m_densityMap->timeSpan3 << m_temporalFilter->timeSpan3;
    m_densityMap->timeSpan4 << m_temporalFilter->timeSpan4;

    // Map overlay stage
    addStage(m_mapOverlay.get());
    m_mapOverlay->viewport << canvasInterface.viewport;
    m_mapOverlay->camera << m_trackball->camera;
    m_mapOverlay->texture << m_densityMap->texture;
    m_mapOverlay->area << m_gpuData->area;
    m_mapOverlay->projectionVertexShader << m_world->projectionVertexShader;

    // Line stage
    addStage(m_lines.get());
    m_lines->projectionVertexShader << m_world->projectionVertexShader;
    m_lines->projectionGeometryShader << m_world->projectionGeometryShader;
    m_lines->lineGeometry << m_gpuData->lineGeometry;
    m_lines->camera << m_trackball->camera;
    m_lines->lineColorMapTexture << m_colorMap->lineColorMapTexture;
    m_lines->staticAttributesTexture << m_gpuData->staticAttributesTexture;
    m_lines->nodeAttributesTexture << m_gpuData->nodeAttributesTexture;
    m_lines->lineAttributesTexture << m_gpuData->lineAttributesTexture;
    m_lines->filtersTexture << m_filter->filtersTexture;
    m_lines->numStaticAttributes << m_gpuData->numStaticAttributes;
    m_lines->numNodeAttributes << m_gpuData->numNodeAttributes;
    m_lines->numEdgeAttributes << m_gpuData->numEdgeAttributes;
    m_lines->numLineAttributes << m_gpuData->numLineAttributes;
    m_lines->lineColorIndex << m_mapping->lineColorIndex;
    m_lines->lineWidthIndex << m_mapping->lineWidthIndex;
    m_lines->nodeHeightIndex << m_mapping->nodeHeightIndex;
    m_lines->widthScale << this->widthScale;
    m_lines->viewport << canvasInterface.viewport;

    // Trajectory stage
    addStage(m_trajectories.get());
    m_trajectories->drawMode << drawMode;
    m_trajectories->maxNodes << maxNodes;
    m_trajectories->projectionVertexShader << m_world->projectionVertexShader;
    m_trajectories->projectionGeometryShader << m_world->projectionGeometryShader;
    m_trajectories->dataset << m_dataLoader->dataset;
    m_trajectories->nodeGeometry << m_gpuData->nodeGeometry;
    m_trajectories->patchGeometry << m_gpuData->patchGeometry;
    m_trajectories->nodeGeometrySize << m_gpuData->nodeGeometrySize;
    m_trajectories->camera << m_trackball->camera;
    m_trajectories->staticAttributesTexture << m_gpuData->staticAttributesTexture;
    m_trajectories->nodeAttributesTexture << m_gpuData->nodeAttributesTexture;
    m_trajectories->lineAttributesTexture << m_gpuData->lineAttributesTexture;
    m_trajectories->filtersTexture << m_filter->filtersTexture;
    m_trajectories->timeSpan << m_temporalFilter->timeSpan;
    m_trajectories->timeSpan1 << m_temporalFilter->timeSpan1;
    m_trajectories->timeSpan2 << m_temporalFilter->timeSpan2;
    m_trajectories->timeSpan3 << m_temporalFilter->timeSpan3;
    m_trajectories->timeSpan4 << m_temporalFilter->timeSpan4;
    m_trajectories->colorMaps << m_colorMap->colorMapsTexture;
    m_trajectories->textures << m_textures->texture;
    m_trajectories->primaryAttribute << this->primaryAttribute;
    m_trajectories->primaryAttributeIndex << m_mapping->primaryAttributeIndex;
    m_trajectories->numStaticAttributes << m_gpuData->numStaticAttributes;
    m_trajectories->numNodeAttributes << m_gpuData->numNodeAttributes;
    m_trajectories->numEdgeAttributes << m_gpuData->numEdgeAttributes;
    m_trajectories->numLineAttributes << m_gpuData->numLineAttributes;
    m_trajectories->viewport << canvasInterface.viewport;
    m_trajectories->attributeNames << m_gpuData->attributeNames;
    m_trajectories->textureNames << m_textures->filenames;
    m_trajectories->colorMapNames << m_colorMap->colorMapNames;
    m_trajectories->time << m_timer->virtualTime;
    m_trajectories->measureUpdate << measureUpdate;
    m_trajectories->hoveredID << m_picking->hoveredID;
    m_trajectories->selectedIDs << m_picking->selectedIDs;
    m_trajectories->numIDs << m_picking->numIDs;

    // World framebuffer stage
    addStage(m_worldFramebuffer.get());
    m_worldFramebuffer->viewport << canvasInterface.viewport;

    // World clear stage
    addStage(m_worldClear.get());
    m_worldClear->renderInterface.viewport << canvasInterface.viewport;
    m_worldClear->createInput("colorTarget") << m_worldFramebuffer->colorBuffer;
    m_worldClear->createInput("colorValue") << this->backgroundColor;
    m_worldClear->createInput("depthTarget") << m_worldFramebuffer->depthBuffer;
    m_worldClear->createInput("depthValue") = 1.0f;
    m_worldClear->createInput("camera") << m_trackball->camera;
    m_worldClear->createInput("world") << m_world->renderPass;
    m_worldClear->createInput("densityMapVisible") << this->densityMapVisible;
    m_worldClear->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_worldClear->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");

    // World rasterization stage
    addStage(m_worldRasterization.get());
    m_worldRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_worldClear->output("colorTargetOut"));
    m_worldRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_worldRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_worldClear->output("depthTargetOut"));
    m_worldRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_worldRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_worldRasterization->drawable << m_world->renderPass;

    // Map overlay rasterization stage
    addStage(m_overlayRasterization.get());
    m_overlayRasterization->rasterize << this->densityMapVisible;
    m_overlayRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_worldRasterization->output("colorTargetOut"));
    m_overlayRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_overlayRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_worldRasterization->output("depthTargetOut"));
    m_overlayRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_overlayRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_overlayRasterization->drawable << m_mapOverlay->renderPass;

    // World output texture extraction stage
    addStage(m_worldOutput.get());
    m_worldOutput->colorRenderTarget.connect(m_overlayRasterization->output("colorTargetOut"));

    // Overlay framebuffer stage
    addStage(m_overlayFramebuffer.get());
    m_overlayFramebuffer->viewport << canvasInterface.viewport;

    // ID buffer render target stage
    addStage(m_idTarget.get());
    m_idTarget->format         = gl::GL_RED;
    m_idTarget->internalFormat = gl::GL_R32F;
    m_idTarget->type           = gl::GL_FLOAT;
    m_idTarget->size << canvasInterface.viewport;

    // Overlay clear stage
    addStage(m_overlayClear.get());
    m_overlayClear->renderInterface.viewport << canvasInterface.viewport;
    m_overlayClear->createInput("colorTarget") << m_overlayFramebuffer->colorBuffer;
    m_overlayClear->createInput("colorValue") = gloperate::Color(0.0f, 0.0f, 0.0f, 0.0f);
    m_overlayClear->createInput("depthTarget") << m_overlayFramebuffer->depthBuffer;
    m_overlayClear->createInput("depthValue") = 1.0f;
    m_overlayClear->createInput("idTarget") << m_idTarget->colorRenderTarget;
    m_overlayClear->createInput("idValue") = -1.0f;
    m_overlayClear->createInput("camera")              << m_trackball->camera;
    m_overlayClear->createInput("world")               << m_world->renderPass;
    m_overlayClear->createInput("lines")               << m_lines->renderPass;
    m_overlayClear->createInput("trajectories")        << m_trajectories->renderPass;
    m_overlayClear->createInput("widthScale")          << this->widthScale;
    m_overlayClear->createInput("linesVisible")        << this->linesVisible;
    m_overlayClear->createInput("trajectoriesVisible") << this->trajectoriesVisible;
    m_overlayClear->createInput("lineColor")           << this->lineColor;
    m_overlayClear->createInput("lineWidth")           << this->lineWidth;
    m_overlayClear->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_overlayClear->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_overlayClear->createOutput<gloperate::ColorRenderTarget *>("idTargetOut");

    // Line rasterization stage
    addStage(m_lineRasterization.get());
    m_lineRasterization->rasterize << this->linesVisible;
    m_lineRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_overlayClear->output("colorTargetOut"));
    m_lineRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_lineRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_overlayClear->output("depthTargetOut"));
    m_lineRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_lineRasterization->createInput<gloperate::ColorRenderTarget *>("idTarget")->connect(m_overlayClear->output("idTargetOut"));
    m_lineRasterization->createOutput<gloperate::ColorRenderTarget *>("idTargetOut");
    m_lineRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_lineRasterization->drawable << m_lines->renderPass;

    // Trajectory rasterization stage
    addStage(m_trajectoryRasterization.get());
    m_trajectoryRasterization->rasterize << this->trajectoriesVisible;
    m_trajectoryRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_lineRasterization->output("colorTargetOut"));
    m_trajectoryRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_trajectoryRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_lineRasterization->output("depthTargetOut"));
    m_trajectoryRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_trajectoryRasterization->createInput<gloperate::ColorRenderTarget *>("idTarget")->connect(m_lineRasterization->output("idTargetOut"));
    m_trajectoryRasterization->createOutput<gloperate::ColorRenderTarget *>("idTargetOut");
    m_trajectoryRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_trajectoryRasterization->drawable << m_trajectories->renderPass;

    // Measure rendering time
    m_trajectoryRasterization->setTimeMeasurement(true);
    m_trajectoryRasterization->timeMeasured.connect([this] (uint64_t, uint64_t gpuTime)
    {
        m_measurement->renderTime.setValue(gpuTime);
    });

    // Trajectory rasterization stage
    addStage(m_helpersRasterization.get());
    m_helpersRasterization->rasterize << this->trajectoriesVisible;
    m_helpersRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_trajectoryRasterization->output("colorTargetOut"));
    m_helpersRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_helpersRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_trajectoryRasterization->output("depthTargetOut"));
    m_helpersRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_helpersRasterization->createInput<gloperate::ColorRenderTarget *>("idTarget")->connect(m_trajectoryRasterization->output("idTargetOut"));
    m_helpersRasterization->createOutput<gloperate::ColorRenderTarget *>("idTargetOut");
    m_helpersRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_helpersRasterization->drawable << m_trajectories->helpersRenderPass;

    // Overlay texture extraction stage
    addStage(m_overlayOutput.get());
    m_overlayOutput->colorRenderTarget.connect(m_helpersRasterization->output("colorTargetOut"));

    // Output stage
    addStage(m_output.get());
    m_output->viewport << canvasInterface.viewport;
    m_output->mapColorTexture << m_worldOutput->texture;
    m_output->mapDepthTexture << m_worldFramebuffer->depthTexture;
    m_output->overlayColorTexture << m_overlayOutput->texture;
    m_output->overlayDepthTexture << m_overlayFramebuffer->depthTexture;

    // Output framebuffer stage
    addStage(m_outputFramebuffer.get());
    m_outputFramebuffer->viewport << canvasInterface.viewport;

    // Output clear stage
    addStage(m_outputClear.get());
    m_outputClear->renderInterface.viewport << canvasInterface.viewport;
    m_outputClear->createInput("colorTarget") << m_outputFramebuffer->colorBuffer;
    m_outputClear->createInput("colorValue") = gloperate::Color(0.0f, 0.0f, 0.0f, 0.0f);
    m_outputClear->createInput("camera") << m_trackball->camera;
    m_outputClear->createInput("time") << m_timer->virtualTime;
    m_outputClear->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");

    // Output rasterization stage
    addStage(m_outputRasterization.get());
    m_outputRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_outputClear->output("colorTargetOut"));
    m_outputRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_outputRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_outputRasterization->drawable << m_output->renderPass;

    // Picking stage
    addStage(m_picking.get());
    m_picking->idTarget.connect(m_trajectoryRasterization->output("idTargetOut"));
    m_picking->idTarget.setFeedback(true);
    m_picking->mousePos << mousePos;
    m_picking->hoveredID.setRequired(true);

    // Performance measurements
    addStage(m_measurement.get());
    m_measurement->colorTarget.connect(m_outputRasterization->output("colorTargetOut"));
    m_measurement->measureRendering << measureRendering;
    m_measurement->drawMode << drawMode;
    m_measurement->maxNodes << maxNodes;
    m_measurement->size << m_trajectories->size;
    m_measurement->setAlwaysProcessed(true);

    // Screenshot stage
    addStage(m_screenshot.get());
    m_screenshot->colorTarget << m_measurement->colorTargetOut;

    // Blit stage
    addStage(m_blit.get());
    m_blit->source << m_screenshot->colorTargetOut;
    m_blit->sourceViewport << canvasInterface.viewport;
    m_blit->target << this->colorTarget;
    m_blit->targetViewport << canvasInterface.viewport;

    // Outputs
    colorTargetOut << m_blit->targetOut;

    // Load default dataset
    filename = datasetPath + "/airtraffic/smallset.exp";
}

AirTrafficPipeline::~AirTrafficPipeline()
{
}

void AirTrafficPipeline::updateMappingValues(const std::vector<std::string> & values)
{
    // When a new dataset has been loaded

    // Update property choices
    Variant choices = Variant::array();

    for (auto value : values)
    {
        choices.asArray()->push_back(value);
    }

    nodeHeight      .setOption("choices", choices);
    lineColor       .setOption("choices", choices);
    lineWidth       .setOption("choices", choices);
    primaryAttribute.setOption("choices", choices);

    // Set mapping options to default values
    std::string defaultLineAttr = "Zero";

    for (auto attr : values)
    {
        if (string::hasPrefix(attr, "Line - ") && defaultLineAttr == "Zero")
        {
            defaultLineAttr = attr;
            break;
        }
    }

    nodeHeight.setValue("Node - height");
    lineColor.setValue(defaultLineAttr);
    lineWidth.setValue("Zero");
    primaryAttribute.setValue("Line - flighttype");
}

void AirTrafficPipeline::onEvent(InputEvent * event)
{
    switch(event->type())
    {
        case InputEvent::Type::MouseMove:
        {
            auto pos = static_cast<MouseEvent*>(event)->pos();

            m_mousePos.x = pos.x;
            m_mousePos.y = canvasInterface.viewport->w - pos.y;

            mousePos.setValue(m_mousePos);
        }
        break;

        default:
            break;
    }
}


} // namespace gtx
