
#include <gtxlib-stages/ClimatePipeline.h>

#include <glm/glm.hpp>

#include <cppassist/memory/make_unique.h>
#include <cppassist/string/manipulation.h>

#include <cppexpose/variant/Variant.h>

#include <glbinding/gl/enum.h>

#include <gloperate/base/Environment.h>
#include <gloperate/input/MouseEvent.h>
#include <gloperate/input/ButtonEvent.h>
#include <gloperate/stages/base/BasicFramebufferStage.h>
#include <gloperate/stages/base/TextureRenderTargetStage.h>
#include <gloperate/stages/base/ClearStage.h>
#include <gloperate/stages/base/RasterizationStage.h>
#include <gloperate/stages/base/TextureFromRenderTargetExtractionStage.h>

#include <gtxlib-core/gtxlib-core.h>

#include <gtxlib-stages/TrackballStage.h>
#include <gtxlib-stages/SelectionStage.h>
#include <gtxlib-stages/WorldStage.h>
#include <gtxlib-stages/NodeStage.h>
#include <gtxlib-stages/EdgeStage.h>
#include <gtxlib-stages/PlotStage.h>
#include <gtxlib-stages/DataLoaderStage.h>
#include <gtxlib-stages/FilterStage.h>
#include <gtxlib-stages/TemporalFilterStage.h>
#include <gtxlib-stages/GpuDataStage.h>
#include <gtxlib-stages/MappingStage.h>
#include <gtxlib-stages/ColorMapStage.h>
#include <gtxlib-stages/PickingStage.h>
#include <gtxlib-stages/OutputStage.h>



using namespace cppassist;
using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(ClimatePipeline, gloperate::Pipeline)


void ClimatePipeline::initialize()
{
}

ClimatePipeline::ClimatePipeline(Environment * environment, const std::string & name)
: Pipeline(environment, "ClimatePipeline", name)
, AbstractEventConsumer(environment->inputManager())
, canvasInterface(this)
, colorTarget("colorTarget", this)
, depthTarget("depthTarget", this)
, colorTargetOut("colorTargetOut", this)
, filename("filename", this)
, projection("projection", this, "Geocentric")
, backgroundColor("background", this, gloperate::Color(0.0f, 0.0f, 0.0f))
, nodeScale("nodeScale", this, 1.0f)
, widthScale("widthScale", this, 1.0f)
, heightScale("heightScale", this, 0.5f)
, arcScale("arcScale", this, 20.0f)
, arcTess("arcTess", this, 64.0f)
, edgesOnTop("edgesOnTop", this, false)
, neighborhood("neighborhood", this, 2.0f)
, unsharpMasking("unsharpMasking", this, true)
, unsharpFactor("unsharpFactor", this, 1.0f)
, nodesVisible("nodesVisible", this, true)
, nodeColorMap("nodeColorMap", this, "0005_hsv_saturation_orange")
, nodeColor("nodeColor", this)
, nodeSize("nodeSize", this)
, nodeHeight("nodeHeight", this)
, nodeContextColor("nodeContextColor", this, 0.4f)
, edgesVisible("edgesVisible", this, true)
, directConnection("directConnection", this, true)
, edgeColorMap("edgeColorMap", this, "0005_hsv_saturation_orange")
, edgeColor("edgeColor", this)
, edgeWidth("edgeWidth", this)
, edgeContextColor("edgeContextColor", this, 0.1f)
, plotVisible("plotVisible", this, true)
, plotColor("plotColor", this, gloperate::Color(0.0f, 1.0f, 0.0f, 1.0f))
, plotBackground("plotBackground", this, gloperate::Color(0.2f, 0.2f, 0.2f, 0.6f))
, mousePos("mousePos", this)
, selectedId("selectedId", this, -1)
, selectedIds("selectedIds", this)
, selectionArea("selectionArea", this)
, m_framebuffer(cppassist::make_unique<BasicFramebufferStage>(environment, "Framebuffer"))
, m_idTarget(cppassist::make_unique<TextureRenderTargetStage>(environment, "IdTarget"))
, m_colorMap(cppassist::make_unique<ColorMapStage>(environment, "ColorMap"))
, m_trackball(cppassist::make_unique<TrackballStage>(environment, "Trackball"))
, m_dataLoader(cppassist::make_unique<DataLoaderStage>(environment, "DataLoader"))
, m_gpuData(cppassist::make_unique<GpuDataStage>(environment, "GpuData"))
, m_mapping(cppassist::make_unique<MappingStage>(environment, "Mapping"))
, m_filter(cppassist::make_unique<FilterStage>(environment, "Filter"))
, m_temporalFilter(cppassist::make_unique<TemporalFilterStage>(environment, "TemporalFilter"))
, m_selection(cppassist::make_unique<SelectionStage>(environment, "Selection"))
, m_world(cppassist::make_unique<WorldStage>(environment, "World"))
, m_nodes(cppassist::make_unique<NodeStage>(environment, "Nodes"))
, m_edges(cppassist::make_unique<EdgeStage>(environment, "Edges"))
, m_plot(cppassist::make_unique<PlotStage>(environment, "Plot"))
, m_clear(cppassist::make_unique<ClearStage>(environment, "Clear"))
, m_worldRasterization(cppassist::make_unique<RasterizationStage>(environment, "WorldRasterization"))
, m_nodeRasterization(cppassist::make_unique<RasterizationStage>(environment, "NodeRasterization"))
, m_edgeRasterization(cppassist::make_unique<RasterizationStage>(environment, "EdgeRasterization"))
, m_bgRasterization(cppassist::make_unique<RasterizationStage>(environment, "BgRasterization"))
, m_plotRasterization(cppassist::make_unique<RasterizationStage>(environment, "PlotRasterization"))
, m_extractColor(cppassist::make_unique<TextureFromRenderTargetExtractionStage>(environment, "ExtractColor"))
, m_extractDepth(cppassist::make_unique<TextureFromRenderTargetExtractionStage>(environment, "ExtractDepth"))
, m_output(cppassist::make_unique<OutputStage>(environment, "Output"))
, m_outputRasterization(cppassist::make_unique<RasterizationStage>(environment, "OutputRasterization"))
, m_picking(cppassist::make_unique<PickingStage>(environment, "Picking"))
, m_selectedId(-1)
{
    // [DEBUG]
//  cppassist::setVerbosityLevel(cppassist::LogMessage::Level::Debug + 2);

    // Get data path
    std::string dataPath = gtx::dataPath();

    // Get dataset path
    std::string datasetPath = gtx::datasetPath();

    // Dataset options
    filename.setOption("category", "dataset");

    // General options
    projection.setOption("category", "general");
    projection.setOption("choices", m_world->projection.option("choices"));

    backgroundColor.setOption("category", "general");

    nodeScale.setOption("category", "general");
    nodeScale.setOption("minimumValue", 0.01f);
    nodeScale.setOption("maximumValue", 2.0f);
    nodeScale.setOption("updateOnDrag", true);

    widthScale.setOption("category", "general");
    widthScale.setOption("minimumValue",  1.0f);
    widthScale.setOption("maximumValue", 10.0f);
    widthScale.setOption("updateOnDrag", true);

    heightScale.setOption("category", "general");
    heightScale.setOption("minimumValue", 0.1f);
    heightScale.setOption("maximumValue", 1.0f);

    arcScale.setOption("category", "general");
    arcScale.setOption("minimumValue",  0.0f);
    arcScale.setOption("maximumValue", 50.0f);
    arcScale.setOption("updateOnDrag", true);

    arcTess.setOption("category", "general");
    arcTess.setOption("minimumValue",  16.0f);
    arcTess.setOption("maximumValue", 256.0f);
    arcTess.setOption("updateOnDrag", true);

    edgesOnTop.setOption("category", "general");

    neighborhood.setOption("category", "general");
    neighborhood.setOption("minimumValue",  1.0f);
    neighborhood.setOption("maximumValue", 10.0f);

    unsharpMasking.setOption("category", "general");

    unsharpFactor.setOption("category", "general");
    unsharpFactor.setOption("minimumValue",  1.0f);
    unsharpFactor.setOption("maximumValue", 50.0f);

    // Nodes options
    nodesVisible.setOption("category", "nodes");

    nodeColor.setOption("category", "nodes");
    nodeColor.setOption("choices", Variant::array());

    nodeSize.setOption("category", "nodes");
    nodeSize.setOption("choices", Variant::array());

    nodeHeight.setOption("category", "nodes");
    nodeHeight.setOption("choices", Variant::array());

    nodeContextColor.setOption("category", "nodes");
    nodeContextColor.setOption("minimumValue", 0.0f);
    nodeContextColor.setOption("maximumValue", 1.0f);

    nodeColorMap.setOption("category", "nodes");
    nodeColorMap.setOption("choices", m_colorMap->colormaps());

    // Edges options
    edgesVisible.setOption("category", "edges");

    directConnection.setOption("category", "edges");

    edgeColor.setOption("category", "edges");
    edgeColor.setOption("choices", Variant::array());

    edgeWidth.setOption("category", "edges");
    edgeWidth.setOption("choices", Variant::array());

    edgeContextColor.setOption("category", "edges");
    edgeContextColor.setOption("minimumValue", 0.0f);
    edgeContextColor.setOption("maximumValue", 1.0f);

    edgeColorMap.setOption("category", "edges");
    edgeColorMap.setOption("choices", m_colorMap->colormaps());

    // Plot options
    plotVisible.setOption("category", "plot");
    plotColor.setOption("category", "plot");
    plotBackground.setOption("category", "plot");

    // Framebuffer stage
    addStage(m_framebuffer.get());
    m_framebuffer->viewport << canvasInterface.viewport;

    // ID buffer render target stage
    addStage(m_idTarget.get());
    m_idTarget->format         = gl::GL_RED;
    m_idTarget->internalFormat = gl::GL_R32F;
    m_idTarget->type           = gl::GL_FLOAT;
    m_idTarget->size << canvasInterface.viewport;

    // Color map stage
    addStage(m_colorMap.get());
    m_colorMap->nodeColorMap << this->nodeColorMap;
    m_colorMap->edgeColorMap << this->edgeColorMap;

    // Trackball stage
    addStage(m_trackball.get());
    m_trackball->viewport << canvasInterface.viewport;

    // Data loader stage
    addStage(m_dataLoader.get());
    m_dataLoader->filePath << this->filename;

    // Gpu data representation stage
    addStage(m_gpuData.get());
    m_gpuData->dataset << m_dataLoader->dataset;
    m_gpuData->attributeNames.valueChanged.connect(this, &ClimatePipeline::updateMappingValues);

    // Create update trigger for UI when a new dataset has been loaded
    auto * updateTrigger = this->createInput("updateTrigger") << m_gpuData->attributeNames;
    updateTrigger->setOption("hidden", true);

    // Mapping stage
    addStage(m_mapping.get());
    m_mapping->attributes << m_gpuData->attributeNames;
    m_mapping->nodeColor  << this->nodeColor;
    m_mapping->nodeSize   << this->nodeSize;
    m_mapping->nodeHeight << this->nodeHeight;
    m_mapping->edgeColor  << this->edgeColor;
    m_mapping->edgeWidth  << this->edgeWidth;

    // Filter stage
    addStage(m_filter.get());
    m_filter->dataset << m_dataLoader->dataset;

    // Temporal filter stage
    addStage(m_temporalFilter.get());
    m_temporalFilter->dataset << m_dataLoader->dataset;

    // Selection stage
    addStage(m_selection.get());
    m_selection->dataset << m_dataLoader->dataset;
    m_selection->attributes << m_gpuData->attributeNames;
    m_selection->edgeGeometry << m_gpuData->edgeGeometry;
    m_selection->selectedIds << selectedIds;
    m_selection->staticAttributesTexture << m_gpuData->staticAttributesTexture;
    m_selection->nodeAttributesTexture << m_gpuData->nodeAttributesTexture;
    m_selection->numStaticAttributes << m_gpuData->numStaticAttributes;
    m_selection->numNodeAttributes << m_gpuData->numNodeAttributes;

    // World map stage
    addStage(m_world.get());
    m_world->projection << this->projection;
    m_world->camera << m_trackball->camera;
    m_world->mapLayersTexture << m_gpuData->mapLayersTexture;
    m_world->timeSpan << m_temporalFilter->timeSpan;
    m_world->timeSpan1 << m_temporalFilter->timeSpan1;
    m_world->timeSpan2 << m_temporalFilter->timeSpan2;
    m_world->timeSpan3 << m_temporalFilter->timeSpan3;
    m_world->timeSpan4 << m_temporalFilter->timeSpan4;

    // Nodes stage
    addStage(m_nodes.get());
    m_nodes->projectionVertexShader << m_world->projectionVertexShader;
    m_nodes->nodeGeometry << m_gpuData->nodeGeometry;
    m_nodes->camera << m_trackball->camera;
    m_nodes->nodeColorMapTexture << m_colorMap->nodeColorMapTexture;
    m_nodes->staticAttributesTexture << m_gpuData->staticAttributesTexture;
    m_nodes->filtersTexture << m_filter->filtersTexture;
    m_nodes->selection << m_selection->selectionActive;
    m_nodes->selectionTexture << m_selection->selectionTexture;
    m_nodes->nodeAttributesTexture << m_gpuData->nodeAttributesTexture;
    m_nodes->numStaticAttributes << m_gpuData->numStaticAttributes;
    m_nodes->numNodeAttributes << m_gpuData->numNodeAttributes;
    m_nodes->numEdgeAttributes << m_gpuData->numEdgeAttributes;
    m_nodes->nodeColorIndex << m_mapping->nodeColorIndex;
    m_nodes->nodeSizeIndex << m_mapping->nodeSizeIndex;
    m_nodes->nodeHeightIndex << m_mapping->nodeHeightIndex;
    m_nodes->nodeScale << this->nodeScale;
    m_nodes->nodeContextColor << this->nodeContextColor;
    m_nodes->viewport << canvasInterface.viewport;
    m_nodes->neighborhood << neighborhood;
    m_edges->heightScale << this->heightScale;
    m_nodes->timeSpan << m_temporalFilter->timeSpan;
    m_nodes->timeSpan1 << m_temporalFilter->timeSpan1;
    m_nodes->timeSpan2 << m_temporalFilter->timeSpan2;
    m_nodes->timeSpan3 << m_temporalFilter->timeSpan3;
    m_nodes->timeSpan4 << m_temporalFilter->timeSpan4;
    m_nodes->hoveredID << m_picking->hoveredID;
    m_nodes->hoveredID.setFeedback(true);

    // Edges stage
    addStage(m_edges.get());
    m_edges->projectionVertexShader << m_world->projectionVertexShader;
    m_edges->projectionGeometryShader << m_world->projectionGeometryShader;
    m_edges->edgeGeometry << m_gpuData->edgeGeometry;
    m_edges->camera << m_trackball->camera;
    m_edges->edgeColorMapTexture << m_colorMap->edgeColorMapTexture;
    m_edges->staticAttributesTexture << m_gpuData->staticAttributesTexture;
    m_edges->nodeAttributesTexture << m_gpuData->nodeAttributesTexture;
    m_edges->edgeAttributesTexture << m_gpuData->edgeAttributesTexture;
    m_edges->filtersTexture << m_filter->filtersTexture;
    m_edges->selection << m_selection->selectionActive;
    m_edges->selectionTexture << m_selection->selectionTexture;
    m_edges->numStaticAttributes << m_gpuData->numStaticAttributes;
    m_edges->numNodeAttributes << m_gpuData->numNodeAttributes;
    m_edges->numEdgeAttributes << m_gpuData->numEdgeAttributes;
    m_edges->edgeColorIndex << m_mapping->edgeColorIndex;
    m_edges->edgeWidthIndex << m_mapping->edgeWidthIndex;
    m_edges->nodeHeightIndex << m_mapping->nodeHeightIndex;
    m_edges->edgeContextColor << this->edgeContextColor;
    m_edges->heightScale << this->heightScale;
    m_edges->widthScale << this->widthScale;
    m_edges->arcScale << this->arcScale;
    m_edges->arcTess << this->arcTess;
    m_edges->edgesOnTop << this->edgesOnTop;
    m_edges->directConnection << this->directConnection;
    m_edges->viewport << canvasInterface.viewport;
    m_edges->neighborhood << neighborhood;
    m_edges->timeSpan << m_temporalFilter->timeSpan;
    m_edges->timeSpan1 << m_temporalFilter->timeSpan1;
    m_edges->timeSpan2 << m_temporalFilter->timeSpan2;
    m_edges->timeSpan3 << m_temporalFilter->timeSpan3;
    m_edges->timeSpan4 << m_temporalFilter->timeSpan4;

    // Plotting stage
    addStage(m_plot.get());
    m_plot->dataset << m_dataLoader->dataset;
    m_plot->selectedId << selectedId;
    m_plot->viewport << canvasInterface.viewport;
    m_plot->color << plotColor;
    m_plot->backgroundColor << plotBackground;

    // Clear stage
    addStage(m_clear.get());
    m_clear->renderInterface.viewport << canvasInterface.viewport;
    m_clear->createInput("colorTarget") << m_framebuffer->colorBuffer;
    m_clear->createInput("colorValue") << this->backgroundColor;
    m_clear->createInput("depthTarget") << m_framebuffer->depthBuffer;
    m_clear->createInput("depthValue") = 1.0f;
    m_clear->createInput("idTarget") << m_idTarget->colorRenderTarget;
    m_clear->createInput("idValue") = -1.0f;
    m_clear->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_clear->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_clear->createInput("camera")           << m_trackball->camera;
    m_clear->createInput("world")            << m_world->renderPass;
    m_clear->createInput("nodes")            << m_nodes->renderPass;
    m_clear->createInput("edges")            << m_edges->renderPass;
    m_clear->createInput("nodeScale")        << this->nodeScale;
    m_clear->createInput("widthScale")       << this->widthScale;
    m_clear->createInput("arcScale")         << this->arcScale;
    m_clear->createInput("arcTess")          << this->arcTess;
    m_clear->createInput("edgesOnTop")       << this->edgesOnTop;
    m_clear->createInput("nodesVisible")     << this->nodesVisible;
    m_clear->createInput("nodeColor")        << this->nodeColor;
    m_clear->createInput("nodeSize")         << this->nodeSize;
    m_clear->createInput("nodeHeight")       << this->nodeHeight;
    m_clear->createInput("edgesVisible")     << this->edgesVisible;
    m_clear->createInput("directConnection") << this->directConnection;
    m_clear->createInput("edgeColor")        << this->edgeColor;
    m_clear->createInput("edgeWidth")        << this->edgeWidth;
    m_clear->createInput("plotVisible")      << this->plotVisible;
    m_clear->createInput("plotColor")        << this->plotColor;
    m_clear->createInput("plotBackground")   << this->plotBackground;

    // World rasterization stage
    addStage(m_worldRasterization.get());
    m_worldRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_clear->output("colorTargetOut"));
    m_worldRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_worldRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_clear->output("depthTargetOut"));
    m_worldRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_worldRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_worldRasterization->drawable << m_world->renderPass;

    // Node rasterization stage
    addStage(m_nodeRasterization.get());
    m_nodeRasterization->rasterize << this->nodesVisible;
    m_nodeRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_worldRasterization->output("colorTargetOut"));
    m_nodeRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_nodeRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_worldRasterization->output("depthTargetOut"));
    m_nodeRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_nodeRasterization->createInput<gloperate::ColorRenderTarget *>("idTarget")->connect(&m_idTarget->colorRenderTarget);
    m_nodeRasterization->createOutput<gloperate::ColorRenderTarget *>("idTargetOut");
    m_nodeRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_nodeRasterization->drawable << m_nodes->renderPass;

    // Edge rasterization stage
    addStage(m_edgeRasterization.get());
    m_edgeRasterization->rasterize << this->edgesVisible;
    m_edgeRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_nodeRasterization->output("colorTargetOut"));
    m_edgeRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_edgeRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_nodeRasterization->output("depthTargetOut"));
    m_edgeRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_edgeRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_edgeRasterization->drawable << m_edges->renderPass;

    // Background rasterization stage
    addStage(m_bgRasterization.get());
    m_bgRasterization->rasterize << plotVisible;
    m_bgRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_edgeRasterization->output("colorTargetOut"));
    m_bgRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_bgRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_edgeRasterization->output("depthTargetOut"));
    m_bgRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_bgRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_bgRasterization->drawable << m_plot->bgRenderPass;

    // Plot rasterization stage
    addStage(m_plotRasterization.get());
    m_plotRasterization->rasterize << plotVisible;
    m_plotRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_bgRasterization->output("colorTargetOut"));
    m_plotRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_plotRasterization->createInput<gloperate::DepthRenderTarget *>("depthTarget")->connect(m_bgRasterization->output("depthTargetOut"));
    m_plotRasterization->createOutput<gloperate::DepthRenderTarget *>("depthTargetOut");
    m_plotRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_plotRasterization->drawable << m_plot->renderPass;

    // Output texture extraction stage
    addStage(m_extractColor.get());
    m_extractColor->colorRenderTarget.connect(m_plotRasterization->output("colorTargetOut"));

    addStage(m_extractDepth.get());
    m_extractDepth->depthRenderTarget.connect(m_plotRasterization->output("depthTargetOut"));

    // Output stage
    addStage(m_output.get());
    m_output->viewport << canvasInterface.viewport;
    m_output->overlayColorTexture << m_extractColor->texture;
    m_output->overlayDepthTexture << m_extractDepth->texture;
    m_output->unsharpMasking << unsharpMasking;
    m_output->unsharpFactor << unsharpFactor;

    // Output rasterization stage
    addStage(m_outputRasterization.get());
    m_outputRasterization->rasterize = true;
    *m_outputRasterization->createInput<gloperate::ColorRenderTarget *>("colorTarget") << this->colorTarget;
    m_outputRasterization->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_outputRasterization->renderInterface.viewport << canvasInterface.viewport;
    m_outputRasterization->drawable << m_output->renderPass;

    // Picking stage
    addStage(m_picking.get());
    m_picking->idTarget.connect(m_nodeRasterization->output("idTargetOut"));
    m_picking->mousePos << mousePos;
    m_picking->fixId = true;
    m_picking->hoveredID.setRequired(true);

    // Outputs
    colorTargetOut.connect(m_outputRasterization->output("colorTargetOut"));

    // Load default dataset
//  filename = datasetPath + "/ds1.dot";
}

ClimatePipeline::~ClimatePipeline()
{
}

void ClimatePipeline::updateMappingValues(const std::vector<std::string> & values)
{
    // When a new dataset has been loaded

    // Update property choices
    Variant choices = Variant::array();

    for (auto value : values)
    {
        choices.asArray()->push_back(value);
    }

    nodeColor.setOption("choices", choices);
    nodeSize.setOption("choices", choices);
    nodeHeight.setOption("choices", choices);
    edgeColor.setOption("choices", choices);
    edgeWidth.setOption("choices", choices);

    // Set mapping options to default values
    std::string defaultNodeAttr = "Zero";
    std::string defaultEdgeAttr = "Zero";

    for (auto attr : values)
    {
        if (string::hasPrefix(attr, "Node - ") && defaultNodeAttr == "Zero")
        {
            defaultNodeAttr = attr;
        }

        if (string::hasPrefix(attr, "Edge - ") && defaultEdgeAttr == "Zero")
        {
            defaultEdgeAttr = attr;
            break;
        }
    }

    nodeColor.setValue(defaultNodeAttr);
    nodeSize.setValue("Zero");
    nodeHeight.setValue("Zero");
    edgeColor.setValue(defaultEdgeAttr);
    edgeWidth.setValue("Zero");
}

void ClimatePipeline::onInputValueChanged(gloperate::AbstractSlot * slot)
{
    if (slot->name() == "selectionArea")
    {
        // Get selected area
        auto value = selectionArea.value();
        int x1 = (*value.asArray())[0].value<int>();
        int y2 = canvasInterface.viewport->w - (*value.asArray())[1].value<int>();
        int x2 = (*value.asArray())[2].value<int>();
        int y1 = canvasInterface.viewport->w - (*value.asArray())[3].value<int>();

        // Perform picking
        std::set<int> pickedIds;
        for (int y=y1; y<=y2; y++) {
            for (int x=x1; x<=x2; x++) {
                int id = m_picking->pickAt(x, y);
                if (id > -1) {
                    auto it = pickedIds.find(id);
                    if (it == pickedIds.end()) {
                        pickedIds.insert(id);
                    }
                }
            }
        }

        // Determine whether to select or deselect the nodes (if all are selected, deselect them)
        bool select = false;
        for (auto id : pickedIds) {
            if (!isNodeSelected(id)) {
                select = true;
                break;
            }
        }

        // Select/deselect all picked nodes
        for (auto id : pickedIds) {
            // Select or deselect node
            setNodeSelected(id, select);
        }
    }

    Pipeline::onInputValueChanged(slot);
}

void ClimatePipeline::onEvent(InputEvent * event)
{
    switch(event->type())
    {
        case InputEvent::Type::MouseMove:
        {
            auto pos = static_cast<MouseEvent*>(event)->pos();

            m_mousePos.x = pos.x;
            m_mousePos.y = canvasInterface.viewport->w - pos.y;

            mousePos.setValue(m_mousePos);
        }
        break;

        case InputEvent::Type::MouseButtonPress:
        {
            // Left mouse button?
            auto button = static_cast<MouseEvent*>(event)->button();
            if (button == 1) {
                // Get ID of currently hovered node
                int nodeId = *m_picking->hoveredID;
                if (nodeId > -1) {
                    // Select or deselect node
                    setNodeSelected(nodeId, !isNodeSelected(nodeId));
                }
            }
        }
        break;

        case InputEvent::Type::ButtonPress:
        {
            auto key = static_cast<ButtonEvent *>(event)->key();
            if (key == 82) // 'R'
            {
                resetSelection();
            }
        }
        break;

        default:
            break;
    }
}

bool ClimatePipeline::isNodeSelected(int nodeId) const
{
    auto it = m_selectedIds.find(nodeId);
    return (it != m_selectedIds.end());
}

void ClimatePipeline::setNodeSelected(int nodeId, bool selected)
{
    // Select/deselect node
    auto it = m_selectedIds.find(nodeId);
    if (it == m_selectedIds.end() && selected && m_selectedIds.size() < 50) {
        m_selectedIds.insert(nodeId);
    } else if (it != m_selectedIds.end() && !selected) {
        m_selectedIds.erase(nodeId);
    }

    // Get first selected ID
    m_selectedId = -1;
    it = m_selectedIds.begin();
    if (it != m_selectedIds.end()) {
        m_selectedId = *it;
    }

    // Update selected nodes
    selectedIds.setValue(m_selectedIds);
    selectedId.setValue(m_selectedId);
}

void ClimatePipeline::resetSelection()
{
    // Reset selection
    m_selectedIds.clear();
    m_selectedId = -1;

    // Update selected nodes
    selectedIds.setValue(m_selectedIds);
    selectedId.setValue(m_selectedId);
}


} // namespace gtx
