
#include <gtxlib-stages/ColorMapStage.h>

#include <cppexpose/variant/Variant.h>

#include <glbinding/gl/enum.h>

#include <globjects/Texture.h>

#include <gloperate/base/Environment.h>
#include <gloperate/base/ResourceManager.h>

#include <gtxlib-core/gtxlib-core.h>

#include <gtxlib-stages/Tools.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(ColorMapStage, gloperate::Stage)


ColorMapStage::ColorMapStage(gloperate::Environment * environment, const std::string & name)
: gloperate::Stage(environment, "ColorMapStage", name)
, nodeColorMap("nodeColorMap", this)
, edgeColorMap("edgeColorMap", this)
, lineColorMap("lineColorMap", this)
, colorMapNames("colorMapNames", this)
, colorMapsTexture("colorMapsTexture", this)
, nodeColorMapTexture("nodeColorMapTexture", this)
, edgeColorMapTexture("edgeColorMapTexture", this)
, lineColorMapTexture("lineColorMapTexture", this)
{
    m_filenames.push_back("0000_diverging_red_green");
    m_filenames.push_back("0001_diverging_blue_yellow");
    m_filenames.push_back("0002_diverging_green_violet");
    m_filenames.push_back("0003_diverging_violet_orange");
    m_filenames.push_back("0004_diverging_cyan_mauve");
    m_filenames.push_back("0005_hsv_saturation_orange");
    m_filenames.push_back("0006_bbr_cool");
    m_filenames.push_back("0007_bbr_warm_white_yellow_red_black_desaturated");
    m_filenames.push_back("0008_bbr_warm_white_yellow_red_black");
    m_filenames.push_back("0009_saturation_yellow");
    m_filenames.push_back("0010_saturation_blue");
    m_filenames.push_back("0011_saturation_red");
    m_filenames.push_back("0012_blue_red");
    m_filenames.push_back("0013_blue");
    m_filenames.push_back("0014_gradient_violet");
    m_filenames.push_back("color_discrete");
    m_filenames.push_back("color_gradient");
    m_filenames.push_back("color_gradient_blue");
    m_filenames.push_back("color_gradient_green");
    m_filenames.push_back("color_gradient_red");
    m_filenames.push_back("color_gradient_rev");
    m_filenames.push_back("color_gradient_yellow");
    m_filenames.push_back("color_transparency");
}

ColorMapStage::~ColorMapStage()
{
}

void ColorMapStage::onContextInit(AbstractGLContext *)
{
}

void ColorMapStage::onContextDeinit(AbstractGLContext *)
{
    // Release OpenGL objects
    m_colorMaps   = nullptr;
    m_nodeTexture = nullptr;
    m_edgeTexture = nullptr;
    m_lineTexture = nullptr;
}

const cppexpose::Variant & ColorMapStage::colormaps() const
{
    static Variant colorMaps = Variant::array();

    if (colorMaps.asArray()->size() == 0)
    {
        for (size_t i=0; i<m_filenames.size(); i++) {
            colorMaps.asArray()->push_back(m_filenames[i]);
        }
    }

    return colorMaps;
}

void ColorMapStage::onProcess()
{
    // Get data path
    std::string dataPath = gtx::dataPath() + "/colors";

    // Create texture array
    if (!m_colorMaps) {
        createTexture();
    }

    // Load node color map
    auto * texture = m_environment->resourceManager()->load<globjects::Texture>(dataPath + "/" + *nodeColorMap + ".png");
    m_nodeTexture = texture ? std::unique_ptr<globjects::Texture>(texture) : globjects::Texture::createDefault(gl::GL_TEXTURE_2D);

    // Load edge color map
    texture = m_environment->resourceManager()->load<globjects::Texture>(dataPath + "/" + *edgeColorMap + ".png");
    m_edgeTexture = texture ? std::unique_ptr<globjects::Texture>(texture) : globjects::Texture::createDefault(gl::GL_TEXTURE_2D);

    // Load line color map
    texture = m_environment->resourceManager()->load<globjects::Texture>(dataPath + "/" + *lineColorMap + ".png");
    m_lineTexture = texture ? std::unique_ptr<globjects::Texture>(texture) : globjects::Texture::createDefault(gl::GL_TEXTURE_2D);

    // Update outputs
    colorMapNames.setValue(m_filenames);
    colorMapsTexture.setValue(m_colorMaps.get());
    nodeColorMapTexture.setValue(m_nodeTexture.get());
    edgeColorMapTexture.setValue(m_edgeTexture.get());
    lineColorMapTexture.setValue(m_lineTexture.get());
}

void ColorMapStage::createTexture()
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Get filenames
    std::vector<std::string> textures;
    for (size_t i=0; i<m_filenames.size(); i++)
    {
        textures.push_back(dataPath + "/colors/" + m_filenames[i] + ".png");
    }

    // Create 2D texture array
    m_colorMaps = Tools::createTextureArray(textures, 128, 1, false);
}


} // namespace gtx
