
#include <gtxlib-stages/DensityMapStage.h>

#include <cppassist/memory/make_unique.h>

#include <glbinding/gl/enum.h>

#include <globjects/Texture.h>

#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>
#include <gloperate/stages/base/TextureRenderTargetStage.h>
#include <gloperate/stages/base/ClearStage.h>
#include <gloperate/stages/base/RasterizationStage.h>
#include <gloperate/stages/base/TextureFromRenderTargetExtractionStage.h>

#include <gtxlib-core/gtxlib-core.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(DensityMapStage, gloperate::Stage)


DensityMapStage::DensityMapStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, viewport("viewport", this, glm::vec4(0, 0, 1024, 1024))
, lineGeometry("lineGeometry", this)
, area("area", this)
, timeSpan("timeSpan", this)
, timeSpan1("timeSpan1", this)
, timeSpan2("timeSpan2", this)
, timeSpan3("timeSpan3", this)
, timeSpan4("timeSpan4", this)
, texture("texture", this)
, m_program(cppassist::make_unique<ProgramStage>(environment, "program"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "renderPass"))
, m_renderTarget(cppassist::make_unique<TextureRenderTargetStage>(environment, "renderTarget"))
, m_clear(cppassist::make_unique<ClearStage>(environment, "clear"))
, m_rasterize(cppassist::make_unique<RasterizationStage>(environment, "rasterize"))
, m_texture(cppassist::make_unique<TextureFromRenderTargetExtractionStage>(environment, "texture"))
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Density map program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/density/density.vert";
    *m_program->createInput<cppfs::FilePath>("geometryShader") = dataPath + "/shaders/density/density.geom";
    *m_program->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/density/density.frag";
    *m_program->createInput<cppfs::FilePath>("temporal")       = dataPath + "/shaders/base/temporal.vert";

    // Density map render pass stage
    addStage(m_renderPass.get());
    m_renderPass->drawable << lineGeometry;
    m_renderPass->program << m_program->program;
    m_renderPass->culling = false;
    m_renderPass->depthTest = false;
    m_renderPass->blending = true;
    *m_renderPass->createInput<glm::vec2>("timeSpan") << timeSpan;
    *m_renderPass->createInput<glm::vec2>("timeSpan1") << timeSpan1;
    *m_renderPass->createInput<glm::vec2>("timeSpan2") << timeSpan2;
    *m_renderPass->createInput<glm::vec2>("timeSpan3") << timeSpan3;
    *m_renderPass->createInput<glm::vec2>("timeSpan4") << timeSpan4;
    *m_renderPass->createInput<glm::vec4>("area") << area;

    // Density map render target stage
    addStage(m_renderTarget.get());
    m_renderTarget->format         = gl::GL_RGBA;
    m_renderTarget->internalFormat = gl::GL_RGBA32F;
    m_renderTarget->type           = gl::GL_FLOAT;
    m_renderTarget->size << viewport;

    // Density map clear stage
    addStage(m_clear.get());
    m_clear->renderInterface.viewport << viewport;
    m_clear->createInput("colorTarget") << m_renderTarget->colorRenderTarget;
    m_clear->createInput("colorValue") = gloperate::Color(0.0f, 0.0f, 0.0f, 0.0f);
    m_clear->createInput("renderPass") << m_renderPass->renderPass;
    m_clear->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");

    // Density map rasterization stage
    addStage(m_rasterize.get());
    m_rasterize->createInput<gloperate::ColorRenderTarget *>("colorTarget")->connect(m_clear->output("colorTargetOut"));
    m_rasterize->createOutput<gloperate::ColorRenderTarget *>("colorTargetOut");
    m_rasterize->renderInterface.viewport << viewport;
    m_rasterize->drawable << m_renderPass->renderPass;

    // Texture extraction stage
    addStage(m_texture.get());
    m_texture->colorRenderTarget.connect(m_rasterize->output("colorTargetOut"));

    // Outputs
    texture << m_texture->texture;
}

DensityMapStage::~DensityMapStage()
{
}


} // namespace gtx
