
#include <gtxlib-stages/EdgeStage.h>

#include <cppassist/memory/make_unique.h>

#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>

#include <gtxlib-core/gtxlib-core.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(EdgeStage, gloperate::Stage)


EdgeStage::EdgeStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, viewport("viewport", this)
, camera("camera", this)
, projectionVertexShader("projectionVertexShader", this)
, projectionGeometryShader("projectionGeometryShader", this)
, edgeGeometry("edgeGeometry", this)
, edgeColorMapTexture("edgeColorMapTexture", this)
, staticAttributesTexture("staticAttributesTexture", this)
, nodeAttributesTexture("nodeAttributesTexture", this)
, edgeAttributesTexture("edgeAttributesTexture", this)
, filtersTexture("filtersTexture", this)
, selectionTexture("selectionTexture", this)
, numStaticAttributes("numStaticAttributes", this)
, numNodeAttributes("numNodeAttributes", this)
, numEdgeAttributes("numEdgeAttributes", this)
, edgeColorIndex("edgeColorIndex", this)
, edgeWidthIndex("edgeWidthIndex", this)
, nodeHeightIndex("nodeHeightIndex", this)
, edgeContextColor("edgeContextColor", this)
, widthScale("widthScale", this)
, arcScale("arcScale", this)
, arcTess("arcTess", this)
, edgesOnTop("edgesOnTop", this)
, selection("selection", this)
, directConnection("directConnection", this)
, neighborhood("neighborhood", this)
, heightScale("heightScale", this, 0.5f)
, timeSpan("timeSpan", this)
, timeSpan1("timeSpan1", this)
, timeSpan2("timeSpan2", this)
, timeSpan3("timeSpan3", this)
, timeSpan4("timeSpan4", this)
, renderPass("renderPass", this)
, m_program(cppassist::make_unique<ProgramStage>(environment, "EdgeProgram"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "EdgeRenderPass"))
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Edge program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/edges/edges.vert";
    *m_program->createInput<cppfs::FilePath>("geometryShader") = dataPath + "/shaders/edges/edges.geom";
    *m_program->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/edges/edges.frag";
    *m_program->createInput<cppfs::FilePath>("attributes")     = dataPath + "/shaders/base/attributes.vert";
    *m_program->createInput<cppfs::FilePath>("temporal")       = dataPath + "/shaders/base/temporal.vert";
    *m_program->createInput<cppfs::FilePath>("colormap")       = dataPath + "/shaders/base/colormap.geom";
    *m_program->createInput<cppfs::FilePath>("screensize")     = dataPath + "/shaders/base/screensize.geom";
    *m_program->createInput<cppfs::FilePath>("selection")      = dataPath + "/shaders/base/selection.geom";
    *m_program->createInput<globjects::Shader*>("projectionVert") << projectionVertexShader;
    *m_program->createInput<globjects::Shader*>("projectionGeom") << projectionGeometryShader;

    // Edge render pass stage
    addStage(m_renderPass.get());
    m_renderPass->drawable << edgeGeometry;
    m_renderPass->program << m_program->program;
    m_renderPass->camera << camera;
    m_renderPass->blending = true;
    *m_renderPass->createInput<globjects::Texture *>("gradient") << edgeColorMapTexture;
    *m_renderPass->createInput<globjects::Texture *>("staticAttributes") << staticAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("nodeAttributes") << nodeAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("edgeAttributes") << edgeAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("filters") << filtersTexture;
    *m_renderPass->createInput<globjects::Texture *>("nodeSelection") << selectionTexture;
    *m_renderPass->createInput<int>("numStaticAttributes") << numStaticAttributes;
    *m_renderPass->createInput<int>("numNodeAttributes") << numNodeAttributes;
    *m_renderPass->createInput<int>("numEdgeAttributes") << numEdgeAttributes;
    *m_renderPass->createInput<int>("edgeColor") << edgeColorIndex;
    *m_renderPass->createInput<int>("edgeWidth") << edgeWidthIndex;
    *m_renderPass->createInput<int>("nodeHeight") << nodeHeightIndex;
    *m_renderPass->createInput<float>("edgeContextColor") << edgeContextColor;
    *m_renderPass->createInput<float>("heightScale") << heightScale;
    *m_renderPass->createInput<float>("heightOffset") = 0.0f;
    *m_renderPass->createInput<float>("widthScale") << widthScale;
    *m_renderPass->createInput<float>("arcScale") << arcScale;
    *m_renderPass->createInput<float>("arcTess") << arcTess;
    *m_renderPass->createInput<bool>("edgesOnTop") << edgesOnTop;
    *m_renderPass->createInput<glm::vec4>("viewport") << viewport;
    *m_renderPass->createInput<bool>("selection") << selection;
    *m_renderPass->createInput<bool>("directConnection") << directConnection;
    *m_renderPass->createInput<float>("neighborhood") << neighborhood;
    *m_renderPass->createInput<glm::vec2>("timeSpan") << timeSpan;
    *m_renderPass->createInput<glm::vec2>("timeSpan1") << timeSpan1;
    *m_renderPass->createInput<glm::vec2>("timeSpan2") << timeSpan2;
    *m_renderPass->createInput<glm::vec2>("timeSpan3") << timeSpan3;
    *m_renderPass->createInput<glm::vec2>("timeSpan4") << timeSpan4;

    // Outputs
    renderPass << m_renderPass->renderPass;
}

EdgeStage::~EdgeStage()
{
}


} // namespace gtx
