
#include <gtxlib-stages/GpuConfigurationStage.h>

#include <cppassist/memory/make_unique.h>

#include <glbinding/gl/gl.h>

#include <globjects/Texture.h>
#include <globjects/Buffer.h>


using namespace cppexpose;


namespace gtx
{


CPPEXPOSE_COMPONENT(GpuConfigurationStage, gloperate::Stage)


GpuConfigurationStage::GpuConfigurationStage(gloperate::Environment * environment, const std::string & name)
: gloperate::Stage(environment, "GpuConfigurationStage", name)
, configurations("configurations", this)
, texture("texture", this)
{
}

GpuConfigurationStage::~GpuConfigurationStage()
{
}

void GpuConfigurationStage::onProcess()
{
    // Get configurations
    std::vector<Configuration *> configs;
    for (auto * input : inputs<Configuration *>())
    {
        configs.push_back(input->value());
    }

    // Create texture
    createTexture();

    // Set outputs
    configurations.setValue(configs);
    texture.setValue(m_texture.get());
}

void GpuConfigurationStage::createTexture()
{
    static const unsigned int maxConfigs = 32; // Maximum number of configurations
    static const unsigned int maxValues  = 32; // Maximum number of values per configuration

    // Determine texture size
    unsigned int numFloats = maxConfigs * maxValues;

    // Create texture data
    float * data = (float*)malloc(numFloats * sizeof(float));
    for (unsigned int i=0; i<numFloats; i++) {
        data[i] = 0.0f;
    }

    // Insert configurations
    unsigned int i = 0;
    for (auto * input : inputs<Configuration *>())
    {
        // Get configuration
        auto * config = input->value();

        // Copy values
        data[i * maxValues +  0] = config->geometryType;
        data[i * maxValues +  1] = config->tessellation;
        data[i * maxValues +  2] = config->textureID;
        data[i * maxValues +  3] = config->colorMapID;
        data[i * maxValues +  4] = config->height;
        data[i * maxValues +  5] = config->radius;
        data[i * maxValues +  6] = config->color;
        data[i * maxValues +  7] = config->alpha;
        data[i * maxValues +  8] = config->textureX;
        data[i * maxValues +  9] = config->textureY;
        data[i * maxValues + 10] = config->minRadius;
        data[i * maxValues + 11] = config->maxRadius;
        data[i * maxValues + 12] = config->sizeOfSingleStretch;
        data[i * maxValues + 13] = config->sizeOfSingleTorsion;
        data[i * maxValues + 14] = config->animationSpeed;

        // Next
        i++;
    }

    // Create texture
    m_texture = cppassist::make_unique<globjects::Texture>(gl::GL_TEXTURE_BUFFER);
    globjects::Buffer * buffer = new globjects::Buffer();
    buffer->setData(numFloats * sizeof(float), data, gl::GL_STATIC_DRAW);
    m_texture->texBuffer(gl::GL_R32F, buffer);

    // Release texture data
    free(data);
}


} // namespace gtx
