
#include <gtxlib-stages/LineStage.h>

#include <cppassist/memory/make_unique.h>

#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>

#include <gtxlib-core/gtxlib-core.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(LineStage, gloperate::Stage)


LineStage::LineStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, viewport("viewport", this)
, camera("camera", this)
, projectionVertexShader("projectionVertexShader", this)
, projectionGeometryShader("projectionGeometryShader", this)
, lineGeometry("lineGeometry", this)
, lineColorMapTexture("lineColorMapTexture", this)
, staticAttributesTexture("staticAttributesTexture", this)
, nodeAttributesTexture("nodeAttributesTexture", this)
, lineAttributesTexture("lineAttributesTexture", this)
, filtersTexture("filtersTexture", this)
, numStaticAttributes("numStaticAttributes", this)
, numNodeAttributes("numNodeAttributes", this)
, numEdgeAttributes("numEdgeAttributes", this)
, numLineAttributes("numLineAttributes", this)
, lineColorIndex("lineColorIndex", this)
, lineWidthIndex("lineWidthIndex", this)
, nodeHeightIndex("nodeHeightIndex", this)
, widthScale("widthScale", this)
, renderPass("renderPass", this)
, m_program(cppassist::make_unique<ProgramStage>(environment, "LineProgram"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "LineRenderPass"))
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Line program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/lines/lines.vert";
    *m_program->createInput<cppfs::FilePath>("geometryShader") = dataPath + "/shaders/lines/lines.geom";
    *m_program->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/lines/lines.frag";
    *m_program->createInput<cppfs::FilePath>("attributesV")    = dataPath + "/shaders/base/attributes.vert";
    *m_program->createInput<cppfs::FilePath>("attributesG")    = dataPath + "/shaders/base/attributes.geom";
    *m_program->createInput<cppfs::FilePath>("temporal")       = dataPath + "/shaders/base/temporal.vert";
    *m_program->createInput<cppfs::FilePath>("colormap")       = dataPath + "/shaders/base/colormap.geom";
    *m_program->createInput<cppfs::FilePath>("screensize")     = dataPath + "/shaders/base/screensize.geom";
    *m_program->createInput<cppfs::FilePath>("depth")          = dataPath + "/shaders/base/depth.geom";
    *m_program->createInput<globjects::Shader*>("projectionVert") << projectionVertexShader;
    *m_program->createInput<globjects::Shader*>("projectionGeom") << projectionGeometryShader;

    // Line render pass stage
    addStage(m_renderPass.get());
    m_renderPass->drawable << lineGeometry;
    m_renderPass->program << m_program->program;
    m_renderPass->camera << camera;
    *m_renderPass->createInput<globjects::Texture *>("gradient") << lineColorMapTexture;
    *m_renderPass->createInput<globjects::Texture *>("staticAttributes") << staticAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("nodeAttributes") << nodeAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("lineAttributes") << lineAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("filters") << filtersTexture;
    *m_renderPass->createInput<int>("numStaticAttributes") << numStaticAttributes;
    *m_renderPass->createInput<int>("numNodeAttributes") << numNodeAttributes;
    *m_renderPass->createInput<int>("numEdgeAttributes") << numEdgeAttributes;
    *m_renderPass->createInput<int>("numLineAttributes") << numLineAttributes;
    *m_renderPass->createInput<int>("lineColor") << lineColorIndex;
    *m_renderPass->createInput<int>("lineWidth") << lineWidthIndex;
    *m_renderPass->createInput<int>("nodeHeight") << nodeHeightIndex;
    *m_renderPass->createInput<float>("widthScale") << widthScale;
//  *m_renderPass->createInput<glm::vec2>("timeSpan") = glm::vec2(0.0f, 0.0f);
//  *m_renderPass->createInput<glm::vec2>("timeSpan1") = glm::vec2(0.0f, 0.0f);
//  *m_renderPass->createInput<glm::vec2>("timeSpan2") = glm::vec2(0.0f, 0.0f);
//  *m_renderPass->createInput<glm::vec2>("timeSpan3") = glm::vec2(0.0f, 0.0f);
//  *m_renderPass->createInput<glm::vec2>("timeSpan4") = glm::vec2(0.0f, 0.0f);
    *m_renderPass->createInput<glm::vec4>("viewport") << viewport;

    // Outputs
    renderPass << m_renderPass->renderPass;
}

LineStage::~LineStage()
{
}


} // namespace gtx
