
#include <gtxlib-stages/MapOverlayStage.h>

#include <cppassist/memory/make_unique.h>

#include <gloperate/rendering/Shape.h>
#include <gloperate/stages/base/ShapeStage.h>
#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>

#include <gtxlib-core/gtxlib-core.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(MapOverlayStage, gloperate::Stage)


MapOverlayStage::MapOverlayStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, viewport("viewport", this)
, camera("camera", this)
, texture("texture", this)
, area("area", this)
, projectionVertexShader("projectionVertexShader", this)
, renderPass("renderPass", this)
, m_shape(cppassist::make_unique<ShapeStage>(environment, "Shape"))
, m_program(cppassist::make_unique<ProgramStage>(environment, "program"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "renderPass"))
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Shape stage
    addStage(m_shape.get());
    m_shape->shapeType = ShapeType::Quad;
    m_shape->width     = 2.0f;
    m_shape->height    = 2.0f;
    m_shape->texCoords = false;

    // Density map program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/density/overlay.vert";
    *m_program->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/density/overlay.frag";
    *m_program->createInput<globjects::Shader *>("projectionVert") << projectionVertexShader;

    // Density map render pass stage
    addStage(m_renderPass.get());
    m_renderPass->culling = false;
    m_renderPass->depthTest = false;
    m_renderPass->blending = true;
    m_renderPass->drawable << m_shape->drawable;
    m_renderPass->program << m_program->program;
    m_renderPass->camera << camera;
    *m_renderPass->createInput<globjects::Texture *>("overlayColor") << texture;
    *m_renderPass->createInput<glm::vec4>("viewport") << viewport;
    *m_renderPass->createInput<glm::vec4>("area") << area;

    // Outputs
    renderPass << m_renderPass->renderPass;
}

MapOverlayStage::~MapOverlayStage()
{
}


} // namespace gtx
