
#include <gtxlib-stages/MeasurementStage.h>

#include <iostream>

#include <gloperate/rendering/ColorRenderTarget.h>


using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(MeasurementStage, gloperate::Stage)


MeasurementStage::MeasurementStage(Environment * environment, const std::string & name)
: Stage(environment, "MeasurementStage", name)
, colorTarget("colorTarget", this)
, measureRendering("measureRendering", this, false)
, drawMode("drawMode", this, -1)
, maxNodes("maxNodes", this, 0)
, size("size", this, 0)
, renderTime("renderTime", this, 0)
, colorTargetOut("colorTargetOut", this)
, frame("frame", this)
, m_frame(-1)
, m_sum(0)
{
}

MeasurementStage::~MeasurementStage()
{
}

void MeasurementStage::onInputValueChanged(AbstractSlot * slot)
{
    // Call default implementation
    Stage::onInputValueChanged(slot);

    // Reset internal counters when time-measurement is toggled
    if (slot == &measureRendering)
    {
        static bool first = true;

        if (!first) {
            m_frame = 0;
            m_sum   = 0;
        } else first = false;
    }
}

void MeasurementStage::onProcess()
{
    if (m_frame < 100)
    {
        if (m_frame > 0)
        {
            std::string mode;
            if (*drawMode == 1) mode = "cpu-triangles-indexed";
            if (*drawMode == 2) mode = "cpu-strips-multiple-calls";
            if (*drawMode == 3) mode = "cpu-strips-primitive-restart";
            if (*drawMode == 4) mode = "gpu-geometry";
            if (*drawMode == 5) mode = "gpu-tessellation";

            m_sum += *renderTime;

            std::cout << "render_time"
                    << "," << m_frame
                    << "," << mode
                    << "," << *maxNodes
                    << "," << *size
                    << "," << *renderTime
                    << "," << (m_sum / m_frame)
                    << std::endl << std::flush;
        }

        m_frame++;
    }

    colorTargetOut.setValue(*colorTarget);
    frame.setValue(m_frame);
}


} // namespace gtx
