
#include <gtxlib-stages/NodeStage.h>

#include <cppassist/memory/make_unique.h>

#include <glbinding/gl/enum.h>

#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>

#include <gtxlib-core/gtxlib-core.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(NodeStage, gloperate::Stage)


NodeStage::NodeStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, viewport("viewport", this)
, camera("camera", this)
, projectionVertexShader("projectionVertexShader", this)
, projectionGeometryShader("projectionGeometryShader", this)
, nodeGeometry("nodeGeometry", this)
, nodeColorMapTexture("nodeColorMapTexture", this)
, staticAttributesTexture("staticAttributesTexture", this)
, nodeAttributesTexture("nodeAttributesTexture", this)
, filtersTexture("filtersTexture", this)
, selectionTexture("selectionTexture", this)
, numStaticAttributes("numStaticAttributes", this)
, numNodeAttributes("numNodeAttributes", this)
, numEdgeAttributes("numEdgeAttributes", this)
, nodeColorIndex("nodeColorIndex", this)
, nodeSizeIndex("nodeSizeIndex", this)
, nodeHeightIndex("nodeHeightIndex", this)
, nodeScale("nodeScale", this)
, nodeContextColor("nodeContextColor", this)
, selection("selection", this)
, neighborhood("neighborhood", this)
, heightScale("heightScale", this, 0.5f)
, timeSpan("timeSpan", this)
, timeSpan1("timeSpan1", this)
, timeSpan2("timeSpan2", this)
, timeSpan3("timeSpan3", this)
, timeSpan4("timeSpan4", this)
, hoveredID("hoveredID", this)
, renderPass("renderPass", this)
, m_program(cppassist::make_unique<ProgramStage>(environment, "NodeProgram"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "NodeRenderPass"))
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Node program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/nodes/nodes.vert";
    *m_program->createInput<cppfs::FilePath>("geometryShader") = dataPath + "/shaders/nodes/nodes.geom";
    *m_program->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/nodes/nodes.frag";
    *m_program->createInput<cppfs::FilePath>("attributes")     = dataPath + "/shaders/base/attributes.vert";
    *m_program->createInput<cppfs::FilePath>("temporal")       = dataPath + "/shaders/base/temporal.vert";
    *m_program->createInput<cppfs::FilePath>("colormap")       = dataPath + "/shaders/base/colormap.geom";
    *m_program->createInput<cppfs::FilePath>("lighting")       = dataPath + "/shaders/base/lighting.frag";
    *m_program->createInput<cppfs::FilePath>("selection")      = dataPath + "/shaders/base/selection.geom";
    *m_program->createInput<globjects::Shader*>("projectionVert") << projectionVertexShader;

    // Node render pass stage
    addStage(m_renderPass.get());
    m_renderPass->drawable << nodeGeometry;
    m_renderPass->program << m_program->program;
    m_renderPass->camera << camera;
    m_renderPass->blending = true;
    m_renderPass->depthTest = true;
    m_renderPass->depthFunc = gl::GL_LESS;
    *m_renderPass->createInput<globjects::Texture *>("gradient") << nodeColorMapTexture;
    *m_renderPass->createInput<globjects::Texture *>("staticAttributes") << staticAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("nodeAttributes") << nodeAttributesTexture;
    *m_renderPass->createInput<globjects::Texture *>("filters") << filtersTexture;
    *m_renderPass->createInput<globjects::Texture *>("nodeSelection") << selectionTexture;
    *m_renderPass->createInput<int>("numStaticAttributes") << numStaticAttributes;
    *m_renderPass->createInput<int>("numNodeAttributes") << numNodeAttributes;
    *m_renderPass->createInput<int>("numEdgeAttributes") << numEdgeAttributes;
    *m_renderPass->createInput<int>("nodeColor") << nodeColorIndex;
    *m_renderPass->createInput<int>("nodeSize") << nodeSizeIndex;
    *m_renderPass->createInput<int>("nodeHeight") << nodeHeightIndex;
    *m_renderPass->createInput<float>("nodeScale") << nodeScale;
    *m_renderPass->createInput<float>("nodeContextColor") << nodeContextColor;
    *m_renderPass->createInput<float>("heightScale") << heightScale;
    *m_renderPass->createInput<float>("heightOffset") = 0.0f;
    *m_renderPass->createInput<glm::vec4>("viewport") << viewport;
    *m_renderPass->createInput<bool>("selection") << selection;
    *m_renderPass->createInput<float>("neighborhood") << neighborhood;
    *m_renderPass->createInput<glm::vec2>("timeSpan") << timeSpan;
    *m_renderPass->createInput<glm::vec2>("timeSpan1") << timeSpan1;
    *m_renderPass->createInput<glm::vec2>("timeSpan2") << timeSpan2;
    *m_renderPass->createInput<glm::vec2>("timeSpan3") << timeSpan3;
    *m_renderPass->createInput<glm::vec2>("timeSpan4") << timeSpan4;
    *m_renderPass->createInput<int>("hoveredID") << hoveredID;

    // Outputs
    renderPass << m_renderPass->renderPass;
}

NodeStage::~NodeStage()
{
}


} // namespace gtx
