
#include <gtxlib-stages/OutputStage.h>

#include <cppassist/memory/make_unique.h>

#include <gloperate/rendering/Shape.h>
#include <gloperate/stages/base/ShapeStage.h>
#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>

#include <gtxlib-core/gtxlib-core.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(OutputStage, gloperate::Stage)


OutputStage::OutputStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, viewport("viewport", this)
, mapColorTexture("colorTexture", this)
, mapDepthTexture("depthTexture", this)
, overlayColorTexture("colorTexture", this)
, overlayDepthTexture("depthTexture", this)
, unsharpMasking("unsharpMasking", this, true)
, unsharpFactor("unsharpFactor", this, 1.0f)
, renderPass("renderPass", this)
, m_shape(cppassist::make_unique<ShapeStage>(environment, "Shape"))
, m_program(cppassist::make_unique<ProgramStage>(environment, "Program"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "RenderPass"))
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Shape stage
    addStage(m_shape.get());
    m_shape->shapeType = ShapeType::Quad;
    m_shape->width     = 2.0f;
    m_shape->height    = 2.0f;
    m_shape->texCoords = true;

    // Program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/output/output.vert";
    *m_program->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/output/output.frag";

    // Render pass stage
    addStage(m_renderPass.get());
    m_renderPass->drawable << m_shape->drawable;
    m_renderPass->program << m_program->program;
    m_renderPass->depthTest = false;
    m_renderPass->blending = true;
    *m_renderPass->createInput<globjects::Texture *>("mapColor")     << mapColorTexture;
    *m_renderPass->createInput<globjects::Texture *>("mapDepth")     << mapDepthTexture;
    *m_renderPass->createInput<globjects::Texture *>("overlayColor") << overlayColorTexture;
    *m_renderPass->createInput<globjects::Texture *>("overlayDepth") << overlayDepthTexture;
    *m_renderPass->createInput<glm::vec4>("viewport") << viewport;
    *m_renderPass->createInput<bool>("useUnsharpMasking") << unsharpMasking;
    *m_renderPass->createInput<float>("unsharpFactor") << unsharpFactor;

    // Outputs
    renderPass << m_renderPass->renderPass;
}

OutputStage::~OutputStage()
{
}


} // namespace gtx
