
#include <gtxlib-stages/PlotGeometryStage.h>

#include <glbinding/gl/enum.h>

#include <gloperate/base/Environment.h>
#include <gloperate/base/ResourceManager.h>

#include <gtxlib-core/data/DataSet.h>
#include <gtxlib-core/geometry/PlotGeometry.h>


namespace gtx
{


CPPEXPOSE_COMPONENT(PlotGeometryStage, gloperate::Stage)


PlotGeometryStage::PlotGeometryStage(gloperate::Environment * environment, const std::string & name)
: gloperate::Stage(environment, "PlotGeometryStage", name)
, dataset("dataset", this)
, selectedId("selectedId", this)
, plotGeometry("plotGeometry", this)
{
}

PlotGeometryStage::~PlotGeometryStage()
{
}

void PlotGeometryStage::onContextDeinit(gloperate::AbstractGLContext *)
{
    // Release OpenGL objects
    m_plotGeometry = nullptr;
}

void PlotGeometryStage::onProcess()
{
    // Create geometries
    m_plotGeometry = cppassist::make_unique<gtx::PlotGeometry>();

    // Create GPU data
    if (dataset.value())
    {
        m_plotGeometry->setData(*dataset.value(), *selectedId);
    }

    // Update outputs
    plotGeometry.setValue(m_plotGeometry.get());
}


} // namespace gtx
