
#include <gtxlib-stages/PlotStage.h>

#include <cppassist/memory/make_unique.h>

#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>
#include <gloperate/stages/base/ShapeStage.h>
#include <gloperate/rendering/ShapeType.h>

#include <gtxlib-core/data/DataSet.h>
#include <gtxlib-stages/PlotGeometryStage.h>

#include <gtxlib-core/gtxlib-core.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(PlotStage, gloperate::Stage)


PlotStage::PlotStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, dataset("dataset", this)
, selectedId("selectedId", this)
, viewport("viewport", this)
, color("color", this)
, backgroundColor("backgroundColor", this)
, bgRenderPass("bgRenderPass", this)
, renderPass("renderPass", this)
, m_bgGeometry(cppassist::make_unique<gloperate::ShapeStage>(environment, "BgGeometry"))
, m_bgProgram(cppassist::make_unique<ProgramStage>(environment, "BgProgram"))
, m_bgRenderPass(cppassist::make_unique<RenderPassStage>(environment, "BgRenderPass"))
, m_geometry(cppassist::make_unique<PlotGeometryStage>(environment, "Geometry"))
, m_program(cppassist::make_unique<ProgramStage>(environment, "Program"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "RenderPass"))
{
    // Get data path
    std::string dataPath = gtx::dataPath();

    // Background geometry stage
    addStage(m_bgGeometry.get());
    m_bgGeometry->shapeType = ShapeType::Quad;
    m_bgGeometry->width     = 2.0f;
    m_bgGeometry->height    = 2.0f;

    // Background program stage
    addStage(m_bgProgram.get());
    *m_bgProgram->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/plot/background.vert";
    *m_bgProgram->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/plot/background.frag";

    // Background render pass stage
    addStage(m_bgRenderPass.get());
    m_bgRenderPass->drawable << m_bgGeometry->drawable;
    m_bgRenderPass->program << m_bgProgram->program;
    *m_bgRenderPass->createInput<glm::vec4>("viewport") << viewport;
    *m_bgRenderPass->createInput<Color>    ("backgroundColor") << backgroundColor;
    *m_bgRenderPass->createInput<int>      ("selectedId") << selectedId;
    m_bgRenderPass->blending = true;
    m_bgRenderPass->depthTest = false;
    m_bgRenderPass->culling = false;

    // Plot geometry stage
    addStage(m_geometry.get());
    m_geometry->dataset << dataset;
    m_geometry->selectedId << selectedId;

    // Plot program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("vertexShader")   = dataPath + "/shaders/plot/plot.vert";
    *m_program->createInput<cppfs::FilePath>("fragmentShader") = dataPath + "/shaders/plot/plot.frag";

    // Plot render pass stage
    addStage(m_renderPass.get());
    m_renderPass->drawable << m_geometry->plotGeometry;
    m_renderPass->program << m_program->program;
    *m_renderPass->createInput<glm::vec4>("viewport") << viewport;
    *m_renderPass->createInput<Color>    ("color")    << color;
    m_renderPass->blending = false;
    m_renderPass->depthTest = false;
    m_renderPass->culling = false;

    // Outputs
    renderPass   << m_renderPass->renderPass;
    bgRenderPass << m_bgRenderPass->renderPass;
}

PlotStage::~PlotStage()
{
}


} // namespace gtx
