
#include <gtxlib-stages/TemporalFilterStage.h>

#include <iostream>

#include <gtxlib-core/data/DataSet.h>


using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(TemporalFilterStage, gloperate::Stage)


TemporalFilterStage::TemporalFilterStage(Environment * environment, const std::string & name)
: Stage(environment, name)
, dataset("dataset", this)
, minTime1("minTime1", this, 0.0f)
, maxTime1("maxTime1", this, 0.0f)
, minTime2("minTime2", this, 0.0f)
, maxTime2("maxTime2", this, 0.0f)
, minTime3("minTime3", this, 0.0f)
, maxTime3("maxTime3", this, 0.0f)
, minTime4("minTime4", this, 0.0f)
, maxTime4("maxTime4", this, 0.0f)
, minTime("minTime", this, 0.0f)
, maxTime("maxTime", this, 0.0f)
, timeSpan("timeSpan", this)
, timeSpan1("timeSpan1", this)
, timeSpan2("timeSpan2", this)
, timeSpan3("timeSpan3", this)
, timeSpan4("timeSpan4", this)
, m_minTime(0.0f)
, m_maxTime(0.0f)
{
    // Connect event when data set has changed
    dataset.valueChanged.connect([this] (const DataSet * dataset)
    {
        // Get minimum and maximum time in dataset
        m_minTime = dataset ? dataset->minimumTime() : 0.0f;
        m_maxTime = dataset ? dataset->maximumTime() : 0.0f;

        // Reset time ranges
        minTime1.setValue(0.0f);
        maxTime1.setValue(0.0f);
        minTime2.setValue(0.0f);
        maxTime2.setValue(0.0f);
        minTime3.setValue(0.0f);
        maxTime3.setValue(0.0f);
        minTime4.setValue(0.0f);
        maxTime4.setValue(0.0f);

        // Invalidate outputs
        minTime.invalidate();
        maxTime.invalidate();
        timeSpan.invalidate();
        timeSpan1.invalidate();
        timeSpan2.invalidate();
        timeSpan3.invalidate();
        timeSpan4.invalidate();
    });
}

TemporalFilterStage::~TemporalFilterStage()
{
}

void TemporalFilterStage::onProcess()
{
    minTime.setValue(m_minTime);
    maxTime.setValue(m_maxTime);

    timeSpan .setValue(glm::vec2(m_minTime, m_maxTime));
    timeSpan1.setValue(glm::vec2(*minTime1, *maxTime1));
    timeSpan2.setValue(glm::vec2(*minTime2, *maxTime2));
    timeSpan3.setValue(glm::vec2(*minTime3, *maxTime3));
    timeSpan4.setValue(glm::vec2(*minTime4, *maxTime4));
}


} // namespace gtx
