
#include <gtxlib-stages/TextureArrayStage.h>

#include <vector>
#include <string>

#include <QImage>

#include <glbinding/gl/enum.h>

#include <globjects/Texture.h>

#include <gloperate/base/Environment.h>
#include <gloperate/base/ResourceManager.h>

#include <gtxlib-stages/Tools.h>


using namespace cppexpose;


namespace gtx
{


CPPEXPOSE_COMPONENT(TextureArrayStage, gloperate::Stage)


TextureArrayStage::TextureArrayStage(gloperate::Environment * environment, const std::string & name)
: gloperate::Stage(environment, "TextureArrayStage", name)
, path("path", this, "")
, width("width", this, 256)
, height("height", this, 256)
, filenames("filenames", this)
, texture("texture", this)
{
}

TextureArrayStage::~TextureArrayStage()
{
}

void TextureArrayStage::onProcess()
{
    // Create texture
    createTexture();
}

void TextureArrayStage::createTexture()
{
    std::vector<std::string> files;
    std::vector<std::string> textures;

    // Get filenames
    for (auto * input : inputs<std::string>())
    {
        if (input != &path) {
            std::string filename = input->value();
            std::string fullpath = filename;
            if (path.value() != "") fullpath = path.value() + "/" + filename;

            files.push_back(filename);
            textures.push_back(fullpath);
        }
    }

    // Get image width and height
    auto width  = this->width.value();
    auto height = this->height.value();

    // Create 2D texture array
    m_texture = Tools::createTextureArray(textures, width, height, true);

    // Set outputs
    filenames.setValue(files);
    texture.setValue(m_texture.get());
}


} // namespace gtx
