
#include <gtxlib-stages/Tools.h>

#include <QImage>

#include <glbinding/gl/enum.h>

#include <globjects/Texture.h>


namespace gtx
{


std::unique_ptr<globjects::Texture> Tools::createTextureArray(const std::vector<std::string> & images, unsigned int width, unsigned int height, bool repeat)
{
    // Get number of images
    auto numImages = images.size();

    // Determine with and height
    unsigned int w = width;
    unsigned int h = height;
    if ((w <= 0 || h <= 0) && numImages > 0)
    {
        QImage image;
        if (image.load(QString::fromStdString(images[0]))) {
            w = image.width();
            h = image.height();
        }
    }

    // Create output image
    unsigned char * data = (unsigned char*)malloc(w * h * numImages * 4 * sizeof(unsigned char));

    // Combine images into one
    for (unsigned int i=0; i<numImages; i++)
    {
        // Load image
        QImage image;
        if (image.load(QString::fromStdString(images[i])))
        {
            // Copy image content into output image
            for (unsigned int y=0; y<h; y++)
            {
                for (unsigned int x=0; x<w; x++)
                {
                    auto color = image.pixel(x, y);

                    data[i * (w*h*4) + (h-y-1) * (w*4) + x * 4 + 0] = qRed(color);
                    data[i * (w*h*4) + (h-y-1) * (w*4) + x * 4 + 1] = qGreen(color);
                    data[i * (w*h*4) + (h-y-1) * (w*4) + x * 4 + 2] = qBlue(color);
                    data[i * (w*h*4) + (h-y-1) * (w*4) + x * 4 + 3] = qAlpha(color);
                }
            }
        }
    }

    // Create texture
    auto texture = globjects::Texture::createDefault(gl::GL_TEXTURE_2D_ARRAY);
    texture->storage3D(1, gl::GL_RGBA8, w, h, numImages);
    texture->subImage3D(0, 0, 0, 0, w, h, numImages, gl::GL_RGBA, gl::GL_UNSIGNED_BYTE, data);
    texture->setParameter(gl::GL_TEXTURE_MIN_FILTER, gl::GL_LINEAR);
    texture->setParameter(gl::GL_TEXTURE_MAG_FILTER, gl::GL_LINEAR);
    texture->setParameter(gl::GL_TEXTURE_WRAP_S,     repeat ? gl::GL_REPEAT : gl::GL_CLAMP_TO_EDGE);
    texture->setParameter(gl::GL_TEXTURE_WRAP_T,     repeat ? gl::GL_REPEAT : gl::GL_CLAMP_TO_EDGE);

    // Destroy output image
    free(data);

    // Return texture
    return texture;
}


} // namespace gtx
