
#include <gtxlib-stages/WorldStage.h>

#include <cppassist/memory/make_unique.h>

#include <gloperate/stages/base/TextureLoadStage.h>
#include <gloperate/stages/base/ProgramStage.h>
#include <gloperate/stages/base/RenderPassStage.h>

#include <gtxlib-core/gtxlib-core.h>

#include <gtxlib-stages/WorldGeometryStage.h>


using namespace cppexpose;
using namespace gloperate;


namespace gtx
{


CPPEXPOSE_COMPONENT(WorldStage, gloperate::Stage)


WorldStage::WorldStage(Environment * environment, const std::string & name)
: Pipeline(environment, name)
, projection("projection", this, "Geocentric")
, camera("camera", this)
, mapLayersTexture("mapLayersTexture", this)
, timeSpan("timeSpan", this)
, timeSpan1("timeSpan1", this)
, timeSpan2("timeSpan2", this)
, timeSpan3("timeSpan3", this)
, timeSpan4("timeSpan4", this)
, renderPass("renderPass", this)
, projectionVertexShader("projectionVertexShader", this)
, projectionGeometryShader("projectionGeometryShader", this)
, m_geometry(cppassist::make_unique<WorldGeometryStage>(environment, "World"))
, m_texture(cppassist::make_unique<TextureLoadStage>(environment, "WorldTexture"))
, m_program(cppassist::make_unique<ProgramStage>(environment, "WorldProgram"))
, m_renderPass(cppassist::make_unique<RenderPassStage>(environment, "WorldRenderPass"))
{
    // Set options
    projection.setOption("choices", m_geometry->projection.option("choices"));

    // Get data path
    std::string dataPath = gtx::dataPath();

    // World geometry stage
    addStage(m_geometry.get());
    m_geometry->projection << this->projection;

    // World texture stage
    addStage(m_texture.get());
    m_texture->filename = dataPath + "/textures/world.png";

    // World program stage
    addStage(m_program.get());
    *m_program->createInput<cppfs::FilePath>("map")      = dataPath + "/shaders/base/map.frag";
    *m_program->createInput<cppfs::FilePath>("temporal") = dataPath + "/shaders/base/temporal.frag";
    *m_program->createInput<globjects::Shader*>("vertexShader")   << m_geometry->vertexShader;
    *m_program->createInput<globjects::Shader*>("fragmentShader") << m_geometry->fragmentShader;

    // World render pass stage
    addStage(m_renderPass.get());
    m_renderPass->drawable << m_geometry->drawable;
    m_renderPass->program << m_program->program;
    m_renderPass->camera << camera;
    m_renderPass->createInput("worldTexture") << m_texture->texture;
    m_renderPass->createInput("mapLayersTexture") << mapLayersTexture;
    *m_renderPass->createInput<glm::vec2>("timeSpan") << timeSpan;
    *m_renderPass->createInput<glm::vec2>("timeSpan1") << timeSpan1;
    *m_renderPass->createInput<glm::vec2>("timeSpan2") << timeSpan2;
    *m_renderPass->createInput<glm::vec2>("timeSpan3") << timeSpan3;
    *m_renderPass->createInput<glm::vec2>("timeSpan4") << timeSpan4;

    // Outputs
    renderPass               << m_renderPass->renderPass;
    projectionVertexShader   << m_geometry->projectionVertexShader;
    projectionGeometryShader << m_geometry->projectionGeometryShader;
}

WorldStage::~WorldStage()
{
}


} // namespace gtx
